@@ -150,6 +150,7 @@ void acs_load(struct acs *acs, const char *data){
 		c = strtok_r(NULL, AC_SEP, &token_ptr);	 // asl in feets
         if (!c) goto x;
 		ac->asl = atoi(c) * 0.3048;
+		if (ac->asl < cfg->ac_minalt) goto x; // unusable altitude
 
 		//if (ac->asl < 2000) goto abrt;
 		if (ac->asl > maxasl) maxasl = ac->asl;
@@ -410,6 +411,41 @@ void plot_path(struct subwin *sw, SDL_Surface *surface, struct band *band, struc
 	
 }
 
+void ac_zorder(struct subwin *sw, struct ac *ac){
+	int px, py;
+	int vx,vy,vx1,vy1,vx2,vy2;
+	int undercursor;
+	double rad, r;
+    struct ac_counterpart *ctp = sw->type == SWT_KST ? &gacs->infoqso : &gacs->tmpqso;
+	struct ac_result *res = sw->type == SWT_KST ? &ac->infores : &ac->tmpres;
+
+	km2px_d(sw, ac->kx, ac->ky, &px, &py);
+	r = 16;
+	rad = atan2((double)(ac->paky - ac->ky), (double)(ac->pakx - ac->kx));
+
+	vx  = (int)(px + 0.6*r * cos(rad) + 0.5);
+    vy  = (int)(py + 0.6*r * sin(rad) + 0.5);
+	vx1 = (int)(px - 0.4*r * cos(rad + 0.6) + 0.5);
+    vy1 = (int)(py - 0.4*r * sin(rad + 0.6) + 0.5);
+    vx2 = (int)(px - 0.4*r * cos(rad - 0.6) + 0.5);
+    vy2 = (int)(py - 0.4*r * sin(rad - 0.6) + 0.5);
+
+	undercursor = 0;
+	if (sw->mx >= z_min3_d(vx, vx1, vx2) && 
+		sw->my >= z_min3_d(vy, vy1, vy2) &&
+		sw->mx <= z_max3_d(vx, vx1, vx2) &&
+		sw->my <= z_max3_d(vy, vy1, vy2)) undercursor = 1;
+
+	if (undercursor){
+		ac->zorder = ACZO_CURSOR;
+	}else if (ctp->path_valid == 1 && res->under){
+        ac->zorder = ACZO_LOWALT;
+	}else if (ctp->path_valid == 1 && res->crossing){
+		ac->zorder = ACZO_CROSSING;
+    }else{
+		ac->zorder = ACZO_NORMAL;
+    }
+}
 
 // main thread, acs->sh locked
 void plot_ac(struct subwin *sw, struct ac *ac){
@@ -427,12 +463,7 @@ void plot_ac(struct subwin *sw, struct ac *ac){
     //dbg("qtf=%f %f \n", ac->qtf, ac->qtf * 180.0 / M_PI);
 
     rad = atan2((double)(ac->paky - ac->ky), (double)(ac->pakx - ac->kx));
-    /*vx  = px; 
-    vy  = py;
-    vx1 = (int)(px - r * cos(rad + 0.3));
-    vy1 = (int)(py - r * sin(rad + 0.3));
-    vx2 = (int)(px - r * cos(rad - 0.3));
-    vy2 = (int)(py - r * sin(rad - 0.3));*/
+    
 	vx  = (int)(px + 0.6*r * cos(rad) + 0.5);
     vy  = (int)(py + 0.6*r * sin(rad) + 0.5);
 	vx1 = (int)(px - 0.4*r * cos(rad + 0.6) + 0.5);
@@ -473,22 +504,15 @@ void plot_ac(struct subwin *sw, struct ac *ac){
     vy1 = (int)(py - 0.4*r * sin(rad + 0.6));
     vx2 = (int)(px - 0.4*r * cos(rad - 0.6));
     vy2 = (int)(py - 0.4*r * sin(rad - 0.6));
-    /*z_line(sw->screen, vx, vy, vx1, vy1, fcolor | 0xff00);
-    z_line(sw->screen, vx, vy, vx2, vy2, fcolor | 0xff00);
-    z_line(sw->screen, vx1, vy1, vx2, vy2, fcolor | 0xff00);
-	z_putpixel(sw->screen, px, py, fcolor | 0xff00);*/
-    
 
     if (undercursor)	
 	{
-
 		opx = px;
 		opy = py;
 		km2px_d(sw, ac->pakx, ac->paky, &px, &py);
 		z_line(sw->screen, opx, opy, px, py, fcolor);
 
 		if (!sw->ac_plotinfo) plot_info_ac(sw, sw->screen, ac);
-
 		return;
 	}
 
@@ -558,7 +582,7 @@ void plot_info_ac(struct subwin *sw, SDL_Surface *surface, struct ac *ac){
 
 void acs_redraw(struct subwin *sw, struct acs *acs){
     struct ac *ac;
-    int i;
+    int i, z;
 	//dbg("------- acs_redraw\n");
     if (!gacs) return;
 
@@ -567,7 +591,14 @@ void acs_redraw(struct subwin *sw, struct acs *acs){
 	if (acs->sh.acs){
 		for (i = 0; i < acs->sh.acs->len; i++){
 			ac = (struct ac *)g_ptr_array_index(acs->sh.acs, i);
-			plot_ac(sw, ac);
+			ac_zorder(sw, ac);
+		}
+
+		for (z = ACZO_LOWALT; z <= ACZO_CURSOR; z++){
+			for (i = 0; i < acs->sh.acs->len; i++){
+				ac = (struct ac *)g_ptr_array_index(acs->sh.acs, i);
+				if (ac->zorder == z) plot_ac(sw, ac);
+			}
 		}
 	}
 	if (!sw->ac_plotinfo) plot_info_ac(sw, sw->screen, NULL);
