@@ -32,6 +32,12 @@ struct ac_result{
 	double angle; 
 };
 
+#define ACZO_LOWALT 0
+#define ACZO_NORMAL 1
+#define ACZO_CROSSING 2
+#define ACZO_CURSOR 3
+
+
 struct ac{
 	double w, h;
 	double qtf; // radians
@@ -44,6 +50,7 @@ struct ac{
 	double pakx, paky; // predicted path after 10 minutes
 
 	struct ac_result tmpres, infores, qrvres; 
+	int zorder;
 };
 
 struct acs_shared{
