@@ -1,6 +1,7 @@
+
 /*
     Tucnak - VHF contest log
-    Copyright (C) 2002-2006  Ladislav Vaiz <ok1zia@nagano.cz>
+    Copyright (C) 2002-2015  Ladislav Vaiz <ok1zia@nagano.cz>
     and authors of web browser Links 0.96
 
     This program is free software; you can redistribute it and/or
@@ -11,6 +12,17 @@
 
 #include "header.h"
 
+#include "bfu.h"
+//#include "charsets.h"
+#include "kbdbind.h"
+#include "kbd.h"
+//#include "language2.h"
+#include "main.h"
+#include "menu.h"
+#include "rc.h"
+#include "session.h"
+#include "terminal.h"
+#include "zosk.h"
 
 struct memory_list *getml(void *p, ...)
 {
@@ -20,7 +32,7 @@ struct memory_list *getml(void *p, ...)
     void *q = p;
     va_start(ap, p);
     while (q) n++, q = va_arg(ap, void *);
-    if (!(ml = mem_alloc(sizeof(struct memory_list) + n * sizeof(void *)))) return NULL;
+    if (!(ml = (struct memory_list*)g_malloc(sizeof(struct memory_list) + n * sizeof(void *)))) return NULL;
     ml->n = n;
     n = 0;
     q = p;
@@ -36,12 +48,12 @@ void add_to_ml(struct memory_list **ml, ...)
     int n = 0;
     void *q;
     if (!*ml) {
-        if (!(*ml = mem_alloc(sizeof(struct memory_list)))) return;
+        if (!(*ml = (struct memory_list*)g_malloc(sizeof(struct memory_list)))) return;
         (*ml)->n = 0;
     }
     va_start(ap, ml);
     while ((q = va_arg(ap, void *))) n++;
-    if (!(nml = mem_realloc(*ml, sizeof(struct memory_list) + (n + (*ml)->n) * sizeof(void *))))
+    if (!(nml = (struct memory_list*)g_realloc(*ml, sizeof(struct memory_list) + (n + (*ml)->n) * sizeof(void *))))
         return;
     va_start(ap, ml);
     while ((q = va_arg(ap, void *))) nml->p[nml->n++] = q;
@@ -52,17 +64,17 @@ void freeml(struct memory_list *ml)
 {
     int i;
     if (!ml) return;
-    for (i = 0; i < ml->n; i++) mem_free(ml->p[i]);
-    mem_free(ml);
+    for (i = 0; i < ml->n; i++) g_free(ml->p[i]);
+    g_free(ml);
 }
 
 char m_bar = 0;
 
 
-void do_menu_selected(struct menu_item *items, cba_t data, int selected)
+void do_menu_selected(struct menu_item *items, void *data, int selected)
 {
     struct menu *menu;
-    if ((menu = mem_alloc(sizeof(struct menu)))) {
+    if ((menu = g_new0(struct menu, 1))) {
         menu->selected = selected;
         menu->view = 0;
         menu->items = items;
@@ -71,14 +83,14 @@ void do_menu_selected(struct menu_item *items, cba_t data, int selected)
     } else if (items->free_i) {
         int i;
         for (i = 0; items[i].text; i++) {
-            if (items[i].free_i & 2) mem_free(items[i].text);
-            if (items[i].free_i & 4) mem_free(items[i].rtext);
+            if (items[i].free_i & 2) g_free(items[i].text);
+            if (items[i].free_i & 4) g_free(items[i].rtext);
         }
-        mem_free(items);
+        g_free(items);
     }
 }
 
-void do_menu(struct menu_item *items, cba_t data)
+void do_menu(struct menu_item *items, void *data)
 {
     do_menu_selected(items, data, 0);
 }
@@ -88,17 +100,30 @@ static void select_menu(struct menu *menu)
     /*int x = menu->x + 4;
     int y = menu->y + menu->selected - menu->view + 2;*/
     struct menu_item *it = &menu->items[menu->selected];
-    void (*func)(cba_t, cba_t) = it->func;
-    cba_t data1 = it->data;
-    cba_t data2 = menu->data;
+    void (*func)(void *, void *) = it->func;
+	void *arg1, *arg2;
+	int dofunc = 1;
     
     if (menu->selected < 0 || menu->selected >= menu->ni || it->hotkey == M_BAR) return;
+	if (menu->items[menu->selected].funcall){
+		menu->items[menu->selected].funcall(menu);
+		dofunc = 0;
+	}
+	arg1 = it->data;
+	arg2 = menu->data;
     if (!it->in_m) {
         struct window *win, *win1;
-        for (win = term->windows.next; (void *)win != &term->windows && (win->handler == menu_func || win->handler == mainmenu_func); win1 = win->next, delete_window(win), win = win1) ;
+        for (win = term->windows.next; 
+			 (void *)win != &term->windows && (win->handler == menu_func || win->handler == mainmenu_func); ){
+			win1 = win->next;
+			delete_window(win);
+			win = win1;
+		}
+
     }
-    /*dbg("select_menu:  %p(%p,%p,%p)\n",term, data1, data2);*/
-    func(data1, data2);
+	if (dofunc){
+	    func(arg1, arg2);
+	}
 }
 
 static void count_menu_size(struct terminal *term, struct menu *menu)
@@ -107,10 +132,17 @@ static void count_menu_size(struct terminal *term, struct menu *menu)
     int sy = term->y;
     int mx = 4;
     int my;
+	int s;
     for (my = 0; menu->items[my].text; my++) {
-        int s = strlen(_(menu->items[my].text)) + strlen(_(menu->items[my].rtext)) + MENU_HOTKEY_SPACE * (_(menu->items[my].rtext)[0] != 0) + 4;
+        s = strlen(_(menu->items[my].text)) + strlen(_(menu->items[my].rtext)) + MENU_HOTKEY_SPACE * (_(menu->items[my].rtext)[0] != 0) + 4;
+		if (menu->items[my].checkbox) s += 4;
         if (s > mx) mx = s;
     }
+	if (menu->items[my].rtext){
+		menu->title = menu->items[my].rtext;
+		s = strlen(menu->title) + 4;
+        if (s > mx) mx = s;
+	}
     menu->ni = my;
     my += 2;
     if (mx > sx) mx = sx;
@@ -157,6 +189,11 @@ static void display_menu(struct menu *menu)
     int p, s;
     fill_area(menu->x+1, menu->y+1, menu->xw-2, menu->yw-2, COLOR_MENU1);
     draw_frame(menu->x, menu->y, menu->xw, menu->yw, COLOR_MENU_FRAME, 1);
+	if (menu->title) {
+		set_char(menu->x + 1, menu->y, COLOR_MENU_FRAME | ' ');
+		print_text(menu->x + 2, menu->y, strlen(menu->title), menu->title, COLOR_MENU_FRAME);
+		set_char(menu->x + 2 + strlen(menu->title), menu->y, COLOR_MENU_FRAME | ' ');
+	}
     for (p = menu->view, s = menu->y + 1; p < menu->ni && p < menu->view + menu->yw - 2; p++, s++) {
         int x;
         int h = 0;
@@ -170,11 +207,18 @@ static void display_menu(struct menu *menu)
             fill_area(menu->x+1, s, menu->xw-2, 1, co);
         }
         if (menu->items[p].hotkey != M_BAR || (tmptext && tmptext[0])) {
+			int xc = 0;
             int l = strlen(_(menu->items[p].rtext));
             for (x = l - 1; x >= 0 && menu->xw - 4 >= l - x && (c = _(menu->items[p].rtext)[x]); x--)
                 set_char(menu->x + menu->xw - 2 - l + x, s, c | co);
+			if (menu->items[p].checkbox) {
+                set_char(menu->x + x + 2 + 1, s, '[' | co);
+				if (menu->items[p].checked) set_char(menu->x + x + 2 + 2, s, 'X' | co);
+                set_char(menu->x + x + 2 + 3, s, ']' | co);
+				xc = 4;
+			}
             for (x = 0; x < menu->xw - 4 && (c = tmptext[x]); x++)
-                set_char(menu->x + x + 2, s, !h && strchr(_(menu->items[p].hotkey), upcase(c)) ? h = 1 , COLOR_MENU_HOTKEY | c : co | c);
+                set_char(menu->x + x + 2 + xc, s, !h && strchr(_(menu->items[p].hotkey), z_char_uc(c)) ? h = 1 , COLOR_MENU_HOTKEY | c : co | c);
         } else {
             set_char(menu->x, s, COLOR_MENU_FRAME | ATTR_FRAME | 0xc3);
             fill_area(menu->x+1, s, menu->xw-2, 1, COLOR_MENU_FRAME | ATTR_FRAME | 0xc4);
@@ -216,13 +260,38 @@ void menu_func(struct window *win, struct event *ev, int fwd)
                     if (ev->x > m1->x && ev->x < m1->x+m1->xw-1 && ev->y > m1->y && ev->y < m1->y+m1->yw-1) goto del;
                 }
             } else {
-                if (!(ev->x < menu->x || ev->x >= menu->x+menu->xw || ev->y < menu->y+1 || ev->y >= menu->y+menu->yw-1)) {
+                if (!(ev->x < menu->x || ev->x >= menu->x+menu->xw || ev->y < menu->y/*+1*/ || ev->y >= menu->y+menu->yw/*-1*/)) {
                     int s = ev->y - menu->y-1 + menu->view;
-                    if (s >= 0 && s < menu->ni && menu->items[s].hotkey != M_BAR) {
+					if (menu->items[s].hotkey == M_BAR){
+#ifdef Z_HAVE_SDL
+						if (zsdl){
+							double m = fmod((double)ev->my, (double)zsdl->font_h);
+							if (m < zsdl->font_h / 2) s--;
+							else s++;
+						}else
+#endif
+							 {
+							if (menu->selected < 0) break;
+							s = menu->selected;
+						}
+					}
+					if (s < 0) s = 0;
+					if (s >= menu->ni) s = menu->ni - 1;
+
+                    if (s >= 0 && s < menu->ni) {
                         menu->selected = s;
                         scroll_menu(menu, 0);
                         display_menu(menu);
-                        if ((ev->b & BM_ACT) == B_UP || menu->items[s].in_m) select_menu(menu);
+                        if ((ev->b & BM_ACT) == B_UP /*|| menu->items[s].in_m*/) {
+							struct menu_item *it = &menu->items[s];
+							int relx = ev->x - menu->x;
+							if (it->checkbox && relx < 6) {
+								it->checked = !it->checked;
+								redraw_later();
+							}else{
+								select_menu(menu);
+							}
+						}
                     }
                 }
             }
@@ -266,7 +335,7 @@ void menu_func(struct window *win, struct event *ev, int fwd)
                     if (ev->x > ' ' && ev->x < 256) {
                         int i;
                         for (i = 0; i < menu->ni; i++)
-                            if (strchr(_(menu->items[i].hotkey), upcase(ev->x))) {
+                            if (strchr(_(menu->items[i].hotkey), z_char_uc(ev->x))) {
                                 menu->selected = i;
                                 scroll_menu(menu, 0);
                                 s = 1;
@@ -274,8 +343,12 @@ void menu_func(struct window *win, struct event *ev, int fwd)
                     }
                     break;
             }
+			if (ev->x == ' '){
+				menu->items[menu->selected].checked = !menu->items[menu->selected].checked;
+				scroll_menu(menu, 1);
+			}
             display_menu(menu);
-            if (s || ev->x == KBD_ENTER || ev->x == ' ') {
+            if (s || ev->x == KBD_ENTER) {
                 enter:
                 select_menu(menu);
             }
@@ -285,19 +358,19 @@ void menu_func(struct window *win, struct event *ev, int fwd)
             if (menu->items->free_i) {
                 int i;
                 for (i = 0; menu->items[i].text; i++) {
-                    if (menu->items[i].free_i & 2) mem_free(menu->items[i].text);
-                    if (menu->items[i].free_i & 4) mem_free(menu->items[i].rtext);
+                    if (menu->items[i].free_i & 2) g_free(menu->items[i].text);
+                    if (menu->items[i].free_i & 4) g_free(menu->items[i].rtext);
                 }
-                mem_free(menu->items);
+                g_free(menu->items);
             }
             break;
     }
 }
 
-void do_mainmenu(struct menu_item *items, cba_t data, int sel)
+void do_mainmenu(struct menu_item *items, void *data, int sel)
 {
     struct mainmenu *menu;
-    if ((menu = mem_alloc(sizeof(struct mainmenu)))) {
+    if ((menu = g_malloc(sizeof(struct mainmenu)))) {
         menu->selected = sel == -1 ? 0 : sel;
         menu->items = items;
         menu->data = data;
@@ -330,7 +403,7 @@ static void display_mainmenu(struct mainmenu *menu)
         }
         p += 2;
         for (j = 0; (c = tmptext[j]); j++, p++)
-            set_char(p, 0, (!s && strchr(_(menu->items[i].hotkey), upcase(c)) ? s = 1, COLOR_MAINMENU_HOTKEY : co) | c);
+            set_char(p, 0, (!s && strchr(_(menu->items[i].hotkey), z_char_uc(c)) ? s = 1, COLOR_MAINMENU_HOTKEY : co) | c);
         p += 2;
     }
     menu->ni = i;
@@ -400,7 +473,7 @@ void mainmenu_func(struct window *win, struct event *ev, int fwd)
                 int i;
                 s = 1;
                 for (i = 0; i < menu->ni; i++)
-                    if (strchr(_(menu->items[i].hotkey), upcase(ev->x))) {
+                    if (strchr(_(menu->items[i].hotkey), z_char_uc(ev->x))) {
                         menu->selected = i;
                         s = 2;
                     }
@@ -419,18 +492,18 @@ void mainmenu_func(struct window *win, struct event *ev, int fwd)
 struct menu_item *new_menu(int free_i)
 {
     struct menu_item *mi;
-    if (!(mi = (struct menu_item*)mem_alloc(sizeof(struct menu_item)))) return NULL;
+    if (!(mi = (struct menu_item*)g_malloc(sizeof(struct menu_item)))) return NULL;
     memset(mi, 0, sizeof(struct menu_item));
     mi->free_i = free_i;
     return mi;
 }
 
-void add_to_menu(struct menu_item **mi, char *text, char *rtext, char *hotkey, void (*func)(cba_t, cba_t), cba_t data, int in_m)
+struct menu_item *add_to_menu(struct menu_item **mi, char *text, char *rtext, char *hotkey, void (*func)(void *, void *), void *data, int in_m)
 {
     struct menu_item *mii;
     int n;
     for (n = 0; (*mi)[n].text; n++) ;
-    if (!(mii = (struct menu_item*)mem_realloc(*mi, (n + 2) * sizeof(struct menu_item)))) return;
+    mii = (struct menu_item*)g_realloc(*mi, (n + 2) * sizeof(struct menu_item));
     *mi = mii;
     memcpy(mii + n + 1, mii + n, sizeof(struct menu_item));
     mii[n].text = text;
@@ -439,6 +512,9 @@ void add_to_menu(struct menu_item **mi, char *text, char *rtext, char *hotkey, v
     mii[n].func = func;
     mii[n].data = data;
     mii[n].in_m = in_m;
+    mii[n].checkbox = 0;
+    mii[n].checked = 0;
+	return mii + n;
 }
 
 void do_dialog(struct dialog *dlg, struct memory_list *ml)
@@ -447,7 +523,7 @@ void do_dialog(struct dialog *dlg, struct memory_list *ml)
     struct dialog_item *d;
     int n = 0;
     for (d = dlg->items; d->type != D_END; d++) n++;
-    if (!(dd = (struct dialog_data *)mem_alloc(sizeof(struct dialog_data) + sizeof(struct dialog_item_data) * n))) return;
+    if (!(dd = (struct dialog_data *)g_malloc(sizeof(struct dialog_data) + sizeof(struct dialog_item_data) * n))) return;
     dd->dlg = dlg;
     dd->n = n;
     dd->ml = ml;
@@ -509,8 +585,10 @@ void display_dlg_item(struct dialog_data *dlg, struct dialog_item_data *di, int
             /* Draw a hierarchy box */
             show_dlg_item_box(dlg, di);
             break;
+		case D_TEXT:
+			break;
         default:
-            debug("Tried to draw unknown ");
+            dbg("Tried to draw unknown ");
     }
 }
 
@@ -568,6 +646,7 @@ static int dlg_mouse(struct dialog_data *dlg, struct dialog_item_data *di, struc
     switch (di->item->type) {
         case D_BUTTON:
             if (ev->y != di->y || ev->x < di->x || ev->x >= di->x + strlen(_(di->item->text)) + 4) return 0;
+			if (dlg->dlg->items[di - dlg->items].type == D_TEXT) return 0;
             display_dlg_item(dlg, &dlg->items[dlg->selected], 0);
             dlg->selected = di - dlg->items;
             display_dlg_item(dlg, di, 1);
@@ -576,10 +655,21 @@ static int dlg_mouse(struct dialog_data *dlg, struct dialog_item_data *di, struc
         case D_FIELD_PASS:
         case D_FIELD:
             if (ev->y != di->y || ev->x < di->x || ev->x >= di->x + di->l) return 0;
-            if ((di->cpos = di->vpos + ev->x - di->x) > strlen(di->cdata)) di->cpos = strlen(di->cdata);
+			if (dlg->dlg->items[di - dlg->items].type == D_TEXT) return 0;
+			if (!cfg->usetouch){
+				if ((di->cpos = di->vpos + ev->x - di->x) > strlen(di->cdata)) di->cpos = strlen(di->cdata);
+			}
             display_dlg_item(dlg, &dlg->items[dlg->selected], 0);
             dlg->selected = di - dlg->items;
             display_dlg_item(dlg, di, 1);
+#ifdef Z_HAVE_SDL
+			if (gses && cfg->usetouch){
+				if (gses->osk) zosk_free(gses->osk);
+				gses->osk = zosk_init(sdl->screen, ZOSK_ENTERCLOSE, di->cdata);
+				zosk_portrait(gses->osk);
+			}
+#endif
+
             return 1;
         case D_CHECKBOX:
         case D_CHECKBOX3:
@@ -602,6 +692,9 @@ static int dlg_mouse(struct dialog_data *dlg, struct dialog_item_data *di, struc
             } /*else if ((ev->b & BM_ACT) == B_DRAG) {
                 debug("drag");
             }*/
+		case D_TEXT:
+
+			break;
     }
     return 0;
 }
@@ -638,7 +731,7 @@ void dialog_func(struct window *win, struct event *ev, int fwd)
                 struct dialog_item_data *di = &dlg->items[i];
                 memset(di, 0, sizeof(struct dialog_item_data));
                 di->item = &dlg->dlg->items[i];
-                if ((di->cdata = (char *)mem_alloc(di->item->dlen)))
+                if ((di->cdata = (char *)g_malloc(di->item->dlen)))
                     memcpy(di->cdata, di->item->data, di->item->dlen);
                 if (di->item->type == D_CHECKBOX) {
                     if (di->item->gid) {
@@ -650,7 +743,7 @@ void dialog_func(struct window *win, struct event *ev, int fwd)
                 } 
                 if (di->item->type == D_BOX) {
                     /* Freed in bookmark_dialog_abort_handler() */
-                    di->cdata = mem_alloc( sizeof(struct dlg_data_item_data_box) );
+                    di->cdata = g_malloc( sizeof(struct dlg_data_item_data_box) );
                     ((struct dlg_data_item_data_box*)di->cdata)->sel = -1;
                     ((struct dlg_data_item_data_box*)di->cdata)->box_top = 0;
                     ((struct dlg_data_item_data_box*)di->cdata)->list_len = -1;
@@ -664,7 +757,7 @@ void dialog_func(struct window *win, struct event *ev, int fwd)
                         /*int l = di->item->dlen;*/
                         foreach(j, di->item->history->items) {
                             struct history_item *hi;
-                            if (!(hi = mem_alloc(sizeof(struct history_item) + strlen(j->d) + 1))) continue;
+                            if (!(hi = g_malloc(sizeof(struct history_item) + strlen(j->d) + 1))) continue;
                             strcpy(hi->d, j->d);
                             add_to_list(di->history, hi);
                         }
@@ -673,6 +766,7 @@ void dialog_func(struct window *win, struct event *ev, int fwd)
                 }
             }
             dlg->selected = 0;
+			while (dlg->dlg->items[dlg->selected].type == D_TEXT) dlg->selected ++;  // no focus for D_TEXT
         case EV_RESIZE:
         case EV_REDRAW:
             dlg->dlg->fn(dlg);
@@ -680,6 +774,7 @@ void dialog_func(struct window *win, struct event *ev, int fwd)
             break;
         case EV_MOUSE:
             if (ev->b & B_MOVE) break;
+
             for (i = 0; i < dlg->n; i++) if (dlg_mouse(dlg, &dlg->items[i], ev)) break;
             break;
         case EV_KBD:
@@ -754,7 +849,7 @@ void dialog_func(struct window *win, struct event *ev, int fwd)
                         strncpy(di->cdata , clipboard, di->item->dlen);
                         di->cdata[di->item->dlen - 1] = 0;
                         di->cpos = strlen(di->cdata);
-                        mem_free(clipboard);
+                        g_free(clipboard);
                         goto dsp_f;
                     }
                     default:
@@ -778,7 +873,7 @@ void dialog_func(struct window *win, struct event *ev, int fwd)
             }
             gh:
             if (ev->x > ' ' && ev->x < 0x100) for (i = 0; i < dlg->n; i++)
-                if (dlg->dlg->items[i].type == D_BUTTON && upcase(_(dlg->dlg->items[i].text)[0]) == upcase(ev->x)) {
+                if (dlg->dlg->items[i].type == D_BUTTON && z_char_uc(_(dlg->dlg->items[i].text)[0]) == z_char_uc(ev->x)) {
                     sel:
                     if (dlg->selected != i) {
                         display_dlg_item(dlg, &dlg->items[dlg->selected], 0);
@@ -794,14 +889,18 @@ void dialog_func(struct window *win, struct event *ev, int fwd)
                 if (dlg->dlg->items[i].type == D_BUTTON && dlg->dlg->items[i].gid & B_ESC) goto sel;
             if ((ev->x == KBD_TAB && !ev->y) || ev->x == KBD_DOWN || ev->x == KBD_RIGHT) {
                 display_dlg_item(dlg, &dlg->items[dlg->selected], 0);
-                if ((++dlg->selected) >= dlg->n) dlg->selected = 0;
+				do{
+					if ((++dlg->selected) >= dlg->n) dlg->selected = 0;
+				}while (dlg->dlg->items[dlg->selected].type == D_TEXT);
                 display_dlg_item(dlg, &dlg->items[dlg->selected], 1);
                 redraw_from_window(dlg->win);
                 break;
             }
             if ((ev->x == KBD_TAB && ev->y) || ev->x == KBD_UP || ev->x == KBD_LEFT) {
                 display_dlg_item(dlg, &dlg->items[dlg->selected], 0);
-                if ((--dlg->selected) < 0) dlg->selected = dlg->n - 1;
+				do{ 
+					if ((--dlg->selected) < 0) dlg->selected = dlg->n - 1;
+				}while (dlg->dlg->items[dlg->selected].type == D_TEXT);
                 display_dlg_item(dlg, &dlg->items[dlg->selected], 1);
                 redraw_from_window(dlg->win);
                 break;
@@ -813,7 +912,7 @@ void dialog_func(struct window *win, struct event *ev, int fwd)
             if (dlg->dlg->abort) dlg->dlg->abort(dlg);
             for (i = 0; i < dlg->n; i++) {
                 struct dialog_item_data *di = &dlg->items[i];
-                if (di->cdata) mem_free(di->cdata);
+                if (di->cdata) g_free(di->cdata);
                 free_list(di->history);
             }
             freeml(dlg->ml);
@@ -836,6 +935,35 @@ int check_number(struct dialog_data *dlg, struct dialog_item_data *di)
     return 0;
 }
 
+int check_hex(struct dialog_data *dlg, struct dialog_item_data *di)
+{
+    char *end;
+    long l = strtol(di->cdata, (char **)(void *)&end, 16);
+    if (!*di->cdata || *end) {
+        msg_box(NULL, VTEXT(T_BAD_NUMBER), AL_CENTER, VTEXT(T_NUMBER_EXPECTED), NULL, 1, VTEXT(T_CANCEL), NULL, B_ENTER | B_ESC);
+        return 1;
+    }
+    if (l < di->item->gid || l > di->item->gnum) {
+        msg_box(NULL, VTEXT(T_BAD_NUMBER), AL_CENTER, VTEXT(T_NUMBER_OUT_OF_RANGE), NULL, 1, VTEXT(T_CANCEL), NULL, B_ENTER | B_ESC);
+        return 1;
+    }
+    return 0;
+}
+
+int check_double(struct dialog_data *dlg, struct dialog_item_data *di)
+{
+    double d = atof(di->cdata);
+    if (!*di->cdata) {
+        msg_box(NULL, VTEXT(T_BAD_NUMBER), AL_CENTER, VTEXT(T_NUMBER_EXPECTED), NULL, 1, VTEXT(T_CANCEL), NULL, B_ENTER | B_ESC);
+        return 1;
+    }
+    if (d < (double)di->item->gid || d > (double)di->item->gnum) {
+        msg_box(NULL, VTEXT(T_BAD_NUMBER), AL_CENTER, VTEXT(T_NUMBER_OUT_OF_RANGE), NULL, 1, VTEXT(T_CANCEL), NULL, B_ENTER | B_ESC);
+        return 1;
+    }
+    return 0;
+}
+
 int check_qrg(struct dialog_data *dlg, struct dialog_item_data *di)
 {
     char *end;
@@ -863,7 +991,7 @@ int cancel_dialog(struct dialog_data *dlg, struct dialog_item_data *di)
 {
     /*struct dialog *dl = dlg->dlg;*/
     delete_window(dlg->win);
-    /*mem_free(dl);*/
+    /*g_free(dl);*/
     return 0;
 }
 
@@ -1263,11 +1391,12 @@ static void msg_box_fn(struct dialog_data *dlg)
     int w, rw;
     int y = 0;
     char **ptr;
-    char *text = init_str();
-    int textl = 0;
-    for (ptr = dlg->dlg->udata; *ptr; ptr++) add_to_str(&text, &textl, _(*ptr));
-    max_text_width(term, text, &max);
-    min_text_width(term, text, &min);
+    //t textl2 = 0;
+    GString *text = g_string_new("");
+
+    for (ptr = dlg->dlg->udata; *ptr; ptr++) g_string_append(text, _(*ptr));
+    max_text_width(term, text->str, &max);
+    min_text_width(term, text->str, &min);
     max_buttons_width(term, dlg->items, dlg->n, &max);
     min_buttons_width(term, dlg->items, dlg->n, &min);
     w = term->x * 9 / 10 - 2 * DIALOG_LB;
@@ -1276,7 +1405,7 @@ static void msg_box_fn(struct dialog_data *dlg)
     if (w > term->x - 2 * DIALOG_LB) w = term->x - 2 * DIALOG_LB;
     if (w < 1) w = 1;
     rw = 0;
-    dlg_format_text(NULL, term, text, 0, &y, w, &rw, COLOR_DIALOG_TEXT, dlg->dlg->align);
+    dlg_format_text(NULL, term, text->str, 0, &y, w, &rw, COLOR_DIALOG_TEXT, dlg->dlg->align);
     y++;
     dlg_format_buttons(NULL, term, dlg->items, dlg->n, 0, &y, w, &rw, AL_CENTER);
     w = rw;
@@ -1285,18 +1414,18 @@ static void msg_box_fn(struct dialog_data *dlg)
     center_dlg(dlg);
     draw_dlg(dlg);
     y = dlg->y + DIALOG_TB + 1;
-    dlg_format_text(term, term, text, dlg->x + DIALOG_LB, &y, w, NULL, COLOR_DIALOG_TEXT, dlg->dlg->align);
+    dlg_format_text(term, term, text->str, dlg->x + DIALOG_LB, &y, w, NULL, COLOR_DIALOG_TEXT, dlg->dlg->align);
     y++;
     dlg_format_buttons(term, term, dlg->items, dlg->n, dlg->x + DIALOG_LB, &y, w, NULL, AL_CENTER);
-    mem_free(text);
+    g_string_free(text, TRUE);
 }
 
 static int msg_box_button(struct dialog_data *dlg, struct dialog_item_data *di)
 {
-    void (*fn)(void *) = (void (*)(void *))di->item->udata;
+    void (*fn)(void *, struct dialog_item_data *) = (void (*)(void *, struct dialog_item_data *di))di->item->udata;
     void *data = dlg->dlg->udata2;
     /*struct dialog *dl = dlg->dlg;*/
-    if (fn) fn(data);
+    if (fn) fn(data, di);
     cancel_dialog(dlg, di);
     return 0;
 }
@@ -1313,14 +1442,14 @@ void msg_box(struct memory_list *ml, char *title,
     int udatan;
     va_list ap;
     va_start(ap, align);
-    udata = DUMMY;
+    udata = NULL;
     udatan = 0;
     do {
         char **udata_;
         text = va_arg(ap, char *);
         na_kovarne__to_je_narez:
-        if (!(udata_ = mem_realloc(udata, ++udatan * sizeof(char *)))) {
-            mem_free(udata);
+        if (!(udata_ = g_realloc(udata, ++udatan * sizeof(char *)))) {
+            g_free(udata);
             return;
         }
         udata = udata_;
@@ -1332,8 +1461,8 @@ void msg_box(struct memory_list *ml, char *title,
     } while (text);
     udata2 = va_arg(ap, void *);
     n = va_arg(ap, int);
-    if (!(dlg = mem_alloc(sizeof(struct dialog) + (n + 1) * sizeof(struct dialog_item)))) {
-        mem_free(udata);
+    if (!(dlg = g_malloc(sizeof(struct dialog) + (n + 1) * sizeof(struct dialog_item)))) {
+        g_free(udata);
         return;
     }
     memset(dlg, 0, sizeof(struct dialog) + (n + 1) * sizeof(struct dialog_item));
@@ -1371,13 +1500,13 @@ void add_to_history(struct history *h, char *t)
     int l;
     if (!h || !t || !*t) return;
     l = strlen(t) + 1;
-    if (!(hi = mem_alloc(sizeof(struct history_item) + l))) return;
+    if (!(hi = g_malloc(sizeof(struct history_item) + l))) return;
     memcpy(hi->d, t, l);
     foreach(hs, h->items) if (!strcmp(hs->d, t)) {
         struct history_item *hd = hs;
         hs = hs->prev;
         del_from_list(hd);
-        mem_free(hd);
+        g_free(hd);
         h->n--;
     }
     add_to_list(h->items, hi);
@@ -1385,12 +1514,12 @@ void add_to_history(struct history *h, char *t)
     while (h->n > MAX_HISTORY_ITEMS) {
         struct history_item *hd = h->items.prev;
         if ((void *)hd == &h->items) {
-            internal_("history is empty");
+            zinternal("history is empty");
             h->n = 0;
             return;
         }
         del_from_list(hd);
-        mem_free(hd);
+        g_free(hd);
         h->n--;
     }
 }
@@ -1426,25 +1555,31 @@ static int input_field_complete(struct dialog_data *dlg, struct dialog_item_data
 
 char new_file[MAX_STR_LEN];
 
-void input_field_fn(struct dialog_data *dlg)
+void input_field_fn_i(struct dialog_data *dlg, int i)
 {
     if (strlen(new_file)){
-        char *c = strrchr(dlg->items[0].cdata, '/');
+        char *c = strrchr(dlg->items[i].cdata, '/');
+		if (!c) c = strrchr(dlg->items[i].cdata, '\\');
         if (c){
 			char *p;
             *c = '\0';
-            p = g_strconcat(dlg->items[0].cdata, "/", new_file, NULL);
-            safe_strncpy0(dlg->items[0].cdata, p, dlg->items[0].item->dlen);
+            p = g_strconcat(dlg->items[i].cdata, "/", new_file, NULL);
+			z_wokna(p);
+            safe_strncpy0(dlg->items[i].cdata, p, dlg->items[i].item->dlen);
             g_free(p);
         }else{
-            safe_strncpy0(dlg->items[0].cdata, new_file, dlg->items[0].item->dlen);
+            safe_strncpy0(dlg->items[i].cdata, new_file, dlg->items[i].item->dlen);
         }
-        dlg->items[0].cpos = strlen(dlg->items[0].cdata);
-//        safe_strncpy0(dlg->items[0].cdata, new_file, EQSO_LEN);
+        dlg->items[i].cpos = strlen(dlg->items[i].cdata);
         strcpy(new_file, "");
     }
 }
 
+void input_field_fn(struct dialog_data *dlg){
+	input_field_fn_i(dlg, 0);
+}
+
+
 
 void input_field(struct memory_list *ml, char *title, char *text, char *okbutton, char *cancelbutton, void *data, struct history *history, int l, char *def, int min, int max, int (*check)(struct dialog_data *, struct dialog_item_data *), void (*fn)(void *, char *), void (*cancelfn)(void *), int tabcompl)
 {                                                                                                                                   
@@ -1454,7 +1589,7 @@ void input_field(struct memory_list *ml, char *title, char *text, char *okbutton
 
     strcpy(new_file, "");
 
-    if (!(dlg = mem_alloc(sizeof(struct dialog) + 4 * sizeof(struct dialog_item) + l)))
+    if (!(dlg = g_malloc(sizeof(struct dialog) + 4 * sizeof(struct dialog_item) + l)))
         return;
     memset(dlg, 0, sizeof(struct dialog) + 4 * sizeof(struct dialog_item) + l);
     *(field = (char *)dlg + sizeof(struct dialog) + 4 * sizeof(struct dialog_item)) = 0;
@@ -1557,7 +1692,7 @@ void box_sel_move(struct dialog_item_data *box_item_data, int dist) {
         new_top = new_sel - box_item_data->item->gid + 1;
 #ifdef DEBUG
         if (new_top < 0)
-            debug("Newly calculated box_top is an extremely wrong value (%d). It should not be below zero.", new_top);
+            dbg("Newly calculated box_top is an extremely wrong value (%d). It should not be below zero.", new_top);
 #endif  
     } else if ( new_sel < new_top ) {
         /* Move the display up (if necessary) */    
@@ -1602,16 +1737,17 @@ void show_dlg_item_box(struct dialog_data *dlg, struct dialog_item_data *box_ite
         
 void errbox(char *text, int errcode, ...){
     static char str[1026];
-	char errbuf[1024];
     va_list l;
     
     va_start(l, errcode);
     if (errcode){
-        gchar *c;
-
-        c=g_strdup_vprintf(text, l);
-        g_snprintf(str, 1024, "%s\n%s (%d)", c, strerror_r(errcode, errbuf, sizeof(errbuf)), errcode);
-        g_free(c);
+        GString *gs = g_string_new("");
+        g_string_append_vprintf(gs, text, l);
+        g_string_sprintfa(gs, "\n");
+        z_strerror(gs, errcode);
+        g_string_sprintfa(gs, " (%d)", errcode);
+        g_snprintf(str, 1024, "%s", gs->str);
+        g_string_free(gs, TRUE);
     }else{
         g_vsnprintf(str, 1024, text, l);
     }
@@ -1647,6 +1783,7 @@ void dlg_pf_init(int phase, struct dialog_data *dlgd){
             dlgd->xw = dlgd->w + 2 * DIALOG_LB;
             dlgd->yw = dlgd->yy + 2 * DIALOG_TB;
             center_dlg(dlgd);
+			if (dlgd->dlg->position_fn) dlgd->dlg->position_fn(dlgd);
             draw_dlg(dlgd);
             dlgd->yy = dlgd->y + DIALOG_TB + dlgd->dlg->y0;
             dlgd->ofs = 0;
@@ -1776,10 +1913,11 @@ void dlg_pf_fn(struct dialog_data *dlgd)
             dlgd->yy += dlgd->items[i].item->wrap;
         }
     }
+
 }
 
 static char *complete_file;
-static int filter_complete(const struct dirent *de){
+static int filter_complete(const char *dir, const struct dirent *de){
     int ret;
 
     if (strcmp((char *)de->d_name, ".") == 0) return 0;
@@ -1788,10 +1926,9 @@ static int filter_complete(const struct dirent *de){
     return !ret;
 }
 
-void complete_choose(cba_t dir, cba_t unused){
+void complete_choose(void *dir, void *unused){
     /*dbg("menu_load_contest('%s')\n", dir);*/
-    safe_strncpy0(new_file, (char *)GETCBA(dir, charp), MAX_STR_LEN);
-  
+    safe_strncpy0(new_file, (char *)dir, MAX_STR_LEN);
     redraw_later();
 } 
 
@@ -1805,10 +1942,11 @@ int do_tab_compl(struct dialog_item_data *di, int tabi){
 	dbg("complete: %d '%s'\n", tabi, di->cdata);
 
     slash = strrchr(di->cdata, '/');
+	if (!slash) slash = strrchr(di->cdata, '\\');
     if (slash){
         if (di->cdata[0] == '~'){
             char *p = g_strndup(di->cdata + 1 , slash - di->cdata - 1);
-            path = g_strconcat(getenv("HOME"), "/", p, NULL);
+            path = g_strconcat(home_dir, "/", p, NULL);
             g_free(p);
         }else{
             path = g_strndup(di->cdata, slash - di->cdata);
@@ -1823,7 +1961,7 @@ int do_tab_compl(struct dialog_item_data *di, int tabi){
     free_namelist();
 
     complete_file = file;
-    n = tucnak_scandir(path, &namelist, filter_complete, tucnak_alphasort);    
+    n = z_scandir(path, &namelist, filter_complete, z_scandir_alphasort);    
     complete_file = NULL;
     dbg("n=%d\n", n);
     if (n <= 0) goto x;
@@ -1835,24 +1973,23 @@ int do_tab_compl(struct dialog_item_data *di, int tabi){
 
     if (n == 1){
         char *p = g_strconcat(path, "/", (char*)namelist[0]->d_name, NULL);
+		z_wokna(p);
         safe_strncpy0(di->cdata, p, di->item->dlen);
         di->cpos = strlen(di->cdata);
         g_free(p);
     }else{
         if (!(mi = new_menu(3))) goto x;
         for (i=0; i<n; i++){
-            cba_t cba2;
             char *c = (char*)namelist[i]->d_name;
             if (strlen(c)>max) max=strlen(c);
-            SETCBA(cba2, charp, namelist[i]->d_name);
             add_to_menu(&mi, 
-                    stracpy(c), 
+                    g_strdup(c), 
                     "", "", 
-                    complete_choose, cba2, 0);    
+                    complete_choose, namelist[i]->d_name, 0);    
         }
 //        set_window_ptr(gses->win, (term->x-6-max)/2,(term->y-2-i)/2);
 
-        do_menu(mi, CBA0);
+        do_menu(mi, NULL);
     }
 
 x:;
