@@ -1,6 +1,6 @@
 /*
     Tucnak - VHF contest log
-    Copyright (C) 2002-2006  Ladislav Vaiz <ok1zia@nagano.cz>
+    Copyright (C) 2002-2015  Ladislav Vaiz <ok1zia@nagano.cz>
 
     This program is free software; you can redistribute it and/or 
     modify it under the terms of the GNU General Public License
@@ -9,8 +9,27 @@
 */
 
 
+
 #include "header.h"
 
+#include "control.h"
+#include "cwdaemon.h"
+#include "davac4.h"
+#include "fifo.h"
+#include "inpout.h"
+#include "inputln.h"
+#include "main.h"
+#include "namedb.h"
+#include "ppdev.h"
+#include "qsodb.h"
+#include "rc.h"
+#include "session.h"
+#include "ssbd.h"
+#include "terminal.h"
+#include "trig.h"
+#include "tsdl.h"
+#include "ttys.h"
+#include "winkey.h"
 
 struct cwdaemon *cwda;
 
@@ -18,6 +37,8 @@ struct cwdaemon *init_cwdaemon(void){
     struct cwdaemon *cwda;
     int fds[2];
 
+	progress(VTEXT(T_INITIALIZING_CWPTT_CONTROL));		
+
   /*  sound(880);
     sleep(1);
     sound(0);
@@ -43,7 +64,7 @@ struct cwdaemon *init_cwdaemon(void){
             cwda->ssbway  = parport_ssbway;
             cwda->back    = cwdaemon_back;
             cwda->band    = parport_band;
-#elif defined(__CYGWIN__)
+#elif defined(Z_MSC_MINGW_CYGWIN)
             /* inpout.h */
             cwda->init    = parport_init;
             cwda->free    = parport_free;
@@ -57,7 +78,7 @@ struct cwdaemon *init_cwdaemon(void){
             cwda->back    = cwdaemon_back;
             cwda->band    = parport_band;
 #else
-            log_addf("Support for ppdev/inpout isn't compiled in");
+            log_addf(VTEXT(T_NO_PPDEV));
             dbg("Support for ppdev/inpout isn't compiled in\n");
 #endif            
             break;
@@ -75,7 +96,7 @@ struct cwdaemon *init_cwdaemon(void){
             cwda->back    = cwdaemon_back;
             break;
 		case CWD_DAVAC4:
-#ifdef HAVE_LIBFTDI
+#ifdef Z_HAVE_LIBFTDI
             cwda->init    = davac4_init;
             cwda->free    = davac4_free;
             cwda->reset   = davac4_reset;
@@ -88,9 +109,9 @@ struct cwdaemon *init_cwdaemon(void){
             cwda->monitor = davac4_monitor;
             cwda->back    = cwdaemon_back;
             cwda->band    = davac4_band;
-            cwda->ftdi_mutex = g_mutex_new();
+            MUTEX_INIT(cwda->ftdi);
 #else			
-            log_addf("Support for davac4 isn't compiled in, check for libftdi");
+            log_addf(VTEXT(T_NO_DAVAC4_SUPPORT));
             dbg("Support for davac4 isn't compiled in, check for libftdi");
 #endif
             break;
@@ -111,7 +132,11 @@ struct cwdaemon *init_cwdaemon(void){
             break;
 			
 		case CWD_WINKEY:
-            cwda->init    = winkey_init;
+		case CWD_WINKEYTCP:
+            if (cfg->cwda_type == CWD_WINKEY) 
+				cwda->init = winkey_init_serial;
+			else
+			   	cwda->init = winkey_init_tcp;
             cwda->free    = winkey_free;
             cwda->reset   = winkey_reset;
             cwda->text    = winkey_text;
@@ -123,7 +148,7 @@ struct cwdaemon *init_cwdaemon(void){
             cwda->dtune   = winkey_tune;
             cwda->back    = winkey_back;
             break;
-        case CWD_WINKEY4:
+/*        case CWD_WINKEY4:
             cwda->init    = winkey4_init;
             cwda->free    = winkey4_free;
             cwda->reset   = winkey4_reset;
@@ -135,14 +160,14 @@ struct cwdaemon *init_cwdaemon(void){
             cwda->ssbway  = winkey_ssbway;
             cwda->dtune   = winkey_tune;
             cwda->back    = winkey_back;
-#ifdef HAVE_LIBFTDI
+#ifdef Z_HAVE_LIBFTDI
             cwda->ptt     = davac4_ptt;
             cwda->ssbway  = davac4_ssbway;
             cwda->monitor = davac4_monitor;
             cwda->band    = davac4_band;
-            cwda->ftdi_mutex = g_mutex_new();
+            MUTEX_INIT(cwda->ftdi);
 #endif
-            break;
+            break; */
                 
     }
     cwda->speed=cfg->cwda_speed;
@@ -150,12 +175,12 @@ struct cwdaemon *init_cwdaemon(void){
     
     
     /*dbg("init_cwdaemon\n"); */
-    if (pipe(fds)) internal_("Can't create pipe");
+    if (z_pipe(fds)) zinternal(VTEXT(T_CANT_CREATE_PIPE));
     cwda->pipe_read=fds[0];
     cwda->pipe_write=fds[1];
     cwda->code=g_string_sized_new(100);
     cwda->played=g_string_sized_new(100);
-    cwda->thread=g_thread_create(cwdaemon_thread_func, (gpointer)cwda, TRUE, NULL);
+	cwda->thread=g_thread_try_new("cwdaemon", cwdaemon_thread_func, (gpointer)cwda, NULL);
     if (cwda->init) cwda->init(cwda);
 /*    cwdaemon_cw_string(cwda, "cq cq de ok1zia"); */
     /*cwdaemon_cw_string(cwda, "cq"); */
@@ -164,19 +189,20 @@ struct cwdaemon *init_cwdaemon(void){
 
 void free_cwdaemon(struct cwdaemon *cwda){
     char c;
-    int ret;
 
     if (!cwda) return;
+	progress(VTEXT(T_TERMINATING_CWDAEMON));		
+
     cwda->freeing = 1;
     cq_abort(1);
 
     c=0;
-    ret = write(cwda->pipe_write, &c, 1);
+    z_pipe_write(cwda->pipe_write, &c, 1);
     dbg("join cwda...\n");
     g_thread_join(cwda->thread);
     dbg("done\n");
-    closesocket(cwda->pipe_write);
-    closesocket(cwda->pipe_read);
+    z_pipe_close(cwda->pipe_write);
+    z_pipe_close(cwda->pipe_read);
     g_string_free(cwda->code, 1);
     g_string_free(cwda->played, 1);
     if (cwda->free) cwda->free(cwda);
@@ -195,7 +221,7 @@ static int qi;
 
 static void cwenqueue(struct cwdaemon *cwda){
     char s[256], *c, ch;
-    int i, ret;
+    int i;
     struct timeval tv;
     
     if (cwda->tune > 0){
@@ -232,9 +258,10 @@ static void cwenqueue(struct cwdaemon *cwda){
         
         if (!ch) {
             qi=-1;
-            c = g_strdup_printf("CW;e;%s\n", cwda->played->str);
+            /*c = g_strdup_printf("CW;e;%s\n", cwda->played->str);
             ret = write(tpipe->threadpipe_write, c, strlen(c));
-            g_free(c);
+            g_free(c);*/
+			zselect_msg_send(zsel, "%s;%s;%s", "CW", "e", cwda->played->str);
             g_string_truncate(cwda->played, 0);
             if (cwda->ptt && !cwda->hold_ptt) cwda->ptt(cwda, 0);
             return;
@@ -295,7 +322,7 @@ static void cwenqueue(struct cwdaemon *cwda){
         break;        
     }
     
-//    dbg("enqueue %c '%s'  played='%s'\n", ch, s, cwda->played->str);
+    dbg("enqueue %c '%s'  played='%s'\n", ch, s, cwda->played->str);
     
     gettimeofday(&tv, NULL);
     i=0;
@@ -386,11 +413,13 @@ gpointer cwdaemon_thread_func(gpointer data){
     fd_set rd;
     struct timeval *tv, interval, now;
     int ret, i;
-    char buf[1030], s[256];
+    char buf[1030];
     struct cwdaemon *cwda;
 
     cwda=(struct cwdaemon*)data;
-
+	zg_thread_set_name("Tucnak cwdaemon");
+	zg_thread_set_priority(2);
+		
     qi=-1;
     while(1){
         FD_ZERO(&rd);
@@ -422,11 +451,12 @@ gpointer cwdaemon_thread_func(gpointer data){
         }
         if (FD_ISSET(cwda->pipe_read, &rd)){
             /*dbg("event on fd\n");*/
-            ret=read(cwda->pipe_read, buf, 1024);
+            ret=z_pipe_read(cwda->pipe_read, buf, 1024);
             if (ret<=0){
-                strcpy(s, "CW;!\n");
+                //strcpy(s, "CW;!\n");
                 /*dbg("%s\n", s);*/
-                ret = write(tpipe->threadpipe_write, s, strlen(s));
+                //ret = write(tpipe->threadpipe_write, s, strlen(s));
+				zselect_msg_send(zsel, "%s;%s", "CW", "!");
                 break;
             }
             buf[ret]='\0';
@@ -441,7 +471,7 @@ gpointer cwdaemon_thread_func(gpointer data){
                         if (cwda->ptt && !cwda->hold_ptt) cwda->ptt(cwda, 1);
                         if (!cwda->hold_ptt) g_string_append_c(cwda->code, '\x09');
                     }
-                    g_string_append_c(cwda->code, upcase(buf[i]));
+                    g_string_append_c(cwda->code, z_char_uc(buf[i]));
                     continue;
                 }
                 switch(buf[i]){	   
@@ -509,7 +539,7 @@ void cwdaemon_read_handler(struct cwdaemon *cwda, char *s, char *played){
     char *c;                    
     
     if (gses->last_cq_timer_id){ /* CQ was aborted while playing */
-        kill_timer(gses->last_cq_timer_id);
+        zselect_timer_kill(zsel, gses->last_cq_timer_id);
         gses->last_cq_timer_id = 0;
     }
 
@@ -526,7 +556,7 @@ void cwdaemon_read_handler(struct cwdaemon *cwda, char *s, char *played){
                     aband->iscall = 0;
                     dbg("set iscall=%d\n", aband->iscall);
                     if (!aband->call_played){
-                        aband->call_played = g_strdup_trim(played);
+                        aband->call_played = z_strdup_trim(played);
                         if (aband->call_played){
                             c = strchr(aband->call_played, ' ');
                             if (c) *c = '\0';
@@ -579,7 +609,7 @@ void cq_cw_wait(struct cq *cq){
 
     dbg("cq_cw_wait\n");
 #ifdef HAVE_SNDFILE    
-    ssbd_abort(ssbd,0);
+    ssbd_abort(gssbd,0);
 #endif
     cwdaemon_ptt(cwda, 0, 0);
     cwdaemon_ssbway(cwda, 0); /* microphone */
@@ -591,7 +621,7 @@ void cq_cw_wait(struct cq *cq){
     }
 
     dbg("installing wait timer (%d ms)\n", remains);
-    gses->last_cq_timer_id = install_timer(remains, cq_timer_cw2, (cba_t)cq);
+    gses->last_cq_timer_id = zselect_timer_new(zsel, remains, cq_timer_cw2, cq);
 }
 
 
@@ -603,13 +633,14 @@ static void cwdaemon_send_defaults(struct cwdaemon *cwda, int speed){
 }
 
 void cwdaemon_abort(struct cwdaemon *cwda){
-    int ret;
-
     if (!cwda) return;
+	//dbg("cwdaemon_abort(cwda=%p)\n", cwda);
     cwda->hold_ptt = 0;
     if (cwda->reset) cwda->reset(cwda);
     cwda->tune = 0;
-    ret = write(cwda->pipe_write, "\x03", 1);
+    z_pipe_write(cwda->pipe_write, "\x03", 1);
+
+	//dbg("cwdaemon_abort returned\n");
 }
 
 /* called from SEGV handler */
@@ -621,12 +652,10 @@ void cwdaemon_safe_abort(struct cwdaemon *cwda){
 
 
 int cwdaemon_text(struct cwdaemon *cwda, char *text){
-    int ret;
-    
     if (!cwda) return 0;
     
     dbg("cwdaemon_cw_string('%s')\n", text);
-    ret = write(cwda->pipe_write, text, strlen(text));
+    z_pipe_write(cwda->pipe_write, text, strlen(text));
     return 0;
 }
 
@@ -637,11 +666,14 @@ void cwdaemon_ptt(struct cwdaemon *cwda, int ptt, int hold_ptt){
     cwda->hold_ptt = hold_ptt;
     cwda->ptt(cwda, ptt);        
     if (cwda->hold_ptt) g_string_append_c(cwda->code, '\x09');
+#ifdef HAVE_HAMLIB
+	trigs_set_ptt(gtrigs, ptt ? RIG_PTT_ON : RIG_PTT_OFF);
+#endif
 }
 
 /* 0=microphone, 1=soundcard */
 void cwdaemon_ssbway(struct cwdaemon *cwda, int ssbway){
-    /*dbg("cwdaemon_ssbway(%d)\n", ssbway);*/
+    dbg("cwdaemon_ssbway(%d %s)\n", ssbway, ssbway ? "soundcard": "microphone");
     
     if (!cwda || !cwda->ssbway) return;
     
@@ -687,8 +719,6 @@ int cwdaemon_weight(struct cwdaemon *cwda, int weight){
 }
 
 void cwdaemon_tune(struct cwdaemon *cwda, int tune){
-    int ret;
-
     if (!cwda) return;
     dbg("cwdaemon_tune(%d)\n", tune);
 	
@@ -699,23 +729,21 @@ void cwdaemon_tune(struct cwdaemon *cwda, int tune){
 	}
 		
     if (tune>0)
-		ret = write(cwda->pipe_write, "\x01", 1);
+		z_pipe_write(cwda->pipe_write, "\x01", 1);
 	else 
         cwdaemon_abort(cwda);
 }
 
 int cwdaemon_back(struct cwdaemon *cwda){
-    int ret;
 
     dbg("cwdaemon_back\n");
-	ret = write(cwda->pipe_write, "\x08", 1);
+	z_pipe_write(cwda->pipe_write, "\x08", 1);
     return 0;
 }
 
 int cwdaemon_band(struct cwdaemon *cwda, int bandsw){
     if (!cwda || !cwda->band) return 0;
     
-    dbg("cwdaemon_band: %d\n", bandsw);
     cwda->band(cwda, bandsw);
 
     return 0;
@@ -724,19 +752,17 @@ int cwdaemon_band(struct cwdaemon *cwda, int bandsw){
 
 /*********** external cwdaemon ***********************************/
 int cwd_init(struct cwdaemon *cwda){
-    int on;
     struct sockaddr_in sin;
 
     cwda->sock = socket(PF_INET, SOCK_DGRAM, IPPROTO_UDP);
     if (cwda->sock < 0) goto err;
 
-    on=1;
-    if (setsockopt(cwda->sock, SOL_SOCKET, SO_REUSEADDR, SO4 &on, sizeof(on))){
+    if (z_sock_reuse(cwda->sock, 1)){
         dbg("Can't set SO_REUSEADDR\n");
         goto err;
     }
     
-    if (fcntl(cwda->sock, F_SETFL, O_NONBLOCK)){
+    if (z_sock_nonblock(cwda->sock, 1)){
         dbg("Can't set O_NONBLOCK\n");
         goto err;
     }
@@ -751,7 +777,7 @@ int cwd_init(struct cwdaemon *cwda){
         dbg("Can't connect\n");
         goto err;
     }
-    set_handlers(cwda->sock, cwd_read_handler, NULL, NULL, (cba_t)cwda);
+    zselect_set(zsel, cwda->sock, cwd_read_handler, NULL, NULL, cwda);
 
     
     cwdaemon_send_defaults(cwda, cwda->speed);
@@ -767,21 +793,21 @@ int cwd_free(struct cwdaemon *cwda){
     dbg("cwd_free(%p) sock=%d\n", cwda, cwda?cwda->sock:-1234);
     if (cwda->sock>=0) {
         cwdaemon_abort(cwda);
-        set_handlers(cwda->sock, NULL, NULL, NULL, CBA0);
+        zselect_set(zsel, cwda->sock, NULL, NULL, NULL, NULL);
         closesocket(cwda->sock);
     }
 	return 0;
 }
 
-void cwd_read_handler(cba_t cba){
+void cwd_read_handler(void *arg){
     char s[1024];
     struct sockaddr_in sin;
     socklen_t socklen;
-    int rcvd, ret;          
+    int rcvd;          
     struct cwdaemon *cwda;
 
     dbg("cwd_read_handler\n");
-    cwda = (struct cwdaemon *)GETCBA(cba, cwda);
+    cwda = (struct cwdaemon *)arg;
 #if 0    
     if (!ctest) {
         /* we must clear kernel queue */
@@ -801,11 +827,13 @@ void cwd_read_handler(cba_t cba){
     switch(s[0]){
         case '!':  /* error */
             log_addf("cwdaemon: %s", s+1);
-            ret = write(tpipe->threadpipe_write, "CW;!\n", 5);
+            zselect_msg_send(zsel, "%s;%s", "CW", "!");
+				//ret = write(tpipe->threadpipe_write, "CW;!\n", 5);
             break;
         case 'h':  /* cw text played */
             dbg("cwdaemon text played\n");
-            ret = write(tpipe->threadpipe_write, "CW;e\n", 5);
+			zselect_msg_send(zsel, "%s;%s", "CW", "e");
+            //ret = write(tpipe->threadpipe_write, "CW;e\n", 5);
             break;
     }
 }
@@ -957,10 +985,11 @@ int cq_run_cw(struct cq *cq){
 
     dbg("cq_run_cw\n");
 #ifdef HAVE_SNDFILE
-    ssbd_abort(ssbd,1); /*aborts playing or recording */
+    ssbd_abort(gssbd,1); /*aborts playing or recording */
 #endif
 
     cq->type=MOD_CW_CW;
+    cwdaemon_ssbway(cwda, 1); /* soundcard but here to allow CQ TX on both bands */
     redraw_later();
 
     if (!cq->cw_str) {
@@ -975,8 +1004,8 @@ int cq_run_cw(struct cq *cq){
         remains = cq_remains_run();
         brk = cq_remains_brk();
 
-        if (gses->extcq_break_timer_id) kill_timer(gses->extcq_break_timer_id);
-        gses->extcq_break_timer_id = install_timer(brk, cq_timer_brk, CBA0);
+        if (gses->extcq_break_timer_id) zselect_timer_kill(zsel, gses->extcq_break_timer_id);
+        gses->extcq_break_timer_id = zselect_timer_new(zsel, brk, cq_timer_brk, NULL);
 
         dbg("  remains=%d brk=%d\n", remains, brk);
         if (remains > 0){
@@ -1005,18 +1034,13 @@ x:;
     return 0;
 }
 
-/*void cq_timer_cw(cba_t cq){
-    cwdaemon_abort(cwda);
-    cq_run_cw(cq.cq);
-} */
-
-void cq_timer_cw2(cba_t cq){
+void cq_timer_cw2(void *cq){
     
     dbg("cq_timer_cw2\n");
-    cq_run_cw((struct cq *)GETCBA(cq, cq));
+    cq_run_cw((struct cq *)cq);
 }
 
-void cq_timer_brk(cba_t cba){
+void cq_timer_brk(void *cq){
     gses->extcq_break_timer_id = 0;
     dbg("cq_timer_brk\n");
     cwdaemon_abort(cwda);
@@ -1025,40 +1049,52 @@ void cq_timer_brk(cba_t cba){
 
 
 /*************** SSB *************************/
-#ifdef HAVE_SNDFILE
 int cq_run_ssb(struct cq *cq){
-    int ret;
+#if defined(HAVE_SNDFILE) || defined(Z_ANDROID)
+    int ret = 0;
 
     dbg("cq_run_ssb\n");
     cq->type=MOD_SSB_SSB;
     cwdaemon_ptt(cwda, 1, 0);
     cwdaemon_ssbway(cwda, 1); /* soundcard */
     
-    CONDGFREE(ssbd->pfilename);
-    ssbd->pfilename=convert_esc(cq->ssb_file, NULL, CE_NONE, time(NULL));
-    if (!ssbd->pfilename) return -1;
-    ret=ssbd_play_file(ssbd, ssbd->pfilename);
+#ifdef Z_ANDROID
+    char *filename = convert_esc(cq->ssb_file, NULL, CE_NONE, time(NULL));
+    if (!filename) return -1;
+    zandroid_play_wav(filename);
+#else
+    zg_free0(gssbd->pfilename);
+    gssbd->pfilename=convert_esc(cq->ssb_file, NULL, CE_NONE, time(NULL));
+    if (!gssbd->pfilename) return -1;
+    ret=ssbd_play_file(gssbd, gssbd->pfilename);
+#endif
 
     gses->last_cq = cq;
     gses->last_cq_timer_id = 0;
     peer_tx(aband, 2);
     redraw_later();
     return ret;
+#else
+    log_adds("Tucnak is compiled without libsndfile, no SSB CQ available");
+#endif
 }
 
+#if defined(HAVE_SNDFILE) || defined(Z_ANDROID)
 void cq_ssb_wait(struct cq *cq){
     /*dbg("cq_ssb_wait\n");*/
-    ssbd_abort(ssbd,0);
+#ifdef HAVE_SNDFILE
+    ssbd_abort(gssbd,0);
+#endif
     cwdaemon_ptt(cwda, 0, 0);
     cwdaemon_ssbway(cwda, 0); /* microphone */
     peer_tx(aband, 0);
-    gses->last_cq_timer_id = install_timer(cq->ssb_ts*100, cq_timer_ssb2, (cba_t)cq);
+    gses->last_cq_timer_id = zselect_timer_new(zsel, cq->ssb_ts*100, cq_timer_ssb2, cq);
 }
 
-void cq_timer_ssb2(cba_t cq){
+void cq_timer_ssb2(void *cq){
     
     /*dbg("cq_timer_ssb2\n");*/
-    cq_run_ssb(cq.cq);
+    cq_run_ssb((struct cq *)cq);
 }
 #endif
 
@@ -1082,7 +1118,7 @@ int cq_run_by_number(int no){
     }
     
     /*if (gses->last_cq_timer_id0){
-        kill_timer(gses->last_cq_timer_id);
+        zselect_timer_kill(zsel, gses->last_cq_timer_id);
         gses->last_cq_timer_id = 0;
     } */
     
@@ -1091,10 +1127,8 @@ int cq_run_by_number(int no){
         /*dbg(" speed=%d repeat=%d ts=%d '%s'\n", cq->cw_speed, cq->cw_repeat, cq->cw_ts, cq->cw_allowifundef, cq->cw_str);*/
         ret=cq_run_cw(cq);
     }else if (get_mode() == MOD_SSB_SSB){
-#ifdef HAVE_SNDFILE
         /*dbg(" repeat=%d ts=%d '%s'\n", cq->ssb_repeat, cq->ssb_ts, cq->ssb_file);*/
         ret=cq_run_ssb(cq);
-#endif
     }else
         ret=-1;        
 
@@ -1109,21 +1143,19 @@ int cq_abort(int abort_rec){
     
     if (gses) gses->last_cq=NULL;
     if (gses && gses->last_cq_timer_id){
-        kill_timer(gses->last_cq_timer_id);
+        zselect_timer_kill(zsel, gses->last_cq_timer_id);
         gses->last_cq_timer_id = 0;
     }
     if (gses && gses->extcq_break_timer_id){
-        kill_timer(gses->extcq_break_timer_id);
+        zselect_timer_kill(zsel, gses->extcq_break_timer_id);
         gses->extcq_break_timer_id = 0;
     }
     
     cwdaemon_abort(cwda);
     redraw_later();
-#ifdef HAVE_SNDFILE    
-    if (!abort_rec && ssbd_recording(ssbd)) return 0;
+    if (!abort_rec && ssbd_recording(gssbd)) return 0;
 
-    ssbd_abort(ssbd,abort_rec);
-#endif
+    ssbd_abort(gssbd,abort_rec);
     return 0;
 }
 
@@ -1176,7 +1208,7 @@ gchar *convert_esc(gchar *format, int *undef, int flags, time_t now){
     
     for (c=sft_buf; *c!='\0'; c++){
         if (*c=='~' && c==sft_buf){
-            g_string_append(gs, getenv("HOME"));
+            g_string_append(gs, home_dir);
             continue;
         }
         if (*c!='$') {
@@ -1184,7 +1216,7 @@ gchar *convert_esc(gchar *format, int *undef, int flags, time_t now){
             continue;
         }
         c++;
-        switch (lowcase(*c)){
+        switch (z_char_lc(*c)){
             case '\0':
                 goto brk2;
                 break;
@@ -1198,7 +1230,7 @@ gchar *convert_esc(gchar *format, int *undef, int flags, time_t now){
                         aband->call_played != NULL && 
                         strcmp(TMPQ.callsign, aband->call_played) != 0){
                     g_string_append(gs, TMPQ.callsign);
-                    //CONDGFREE(aband->call_played);
+                    //zg_free0(aband->call_played);
                     //aband->call_played = g_strdup(TMPQ.callsign);
                     //dbg("convert_esc2: set call_played='%s'\n", aband->call_played);
                     //aband->iscall = 1;
@@ -1209,7 +1241,7 @@ gchar *convert_esc(gchar *format, int *undef, int flags, time_t now){
                 if (ctest && TMPQ.callsign) {
                     g_string_append(gs, TMPQ.callsign);
                     if (!strcasecmp(TMPQ.callsign, "O""K""1""Z""I""A")) g_string_append(gs, " ""a""h""o""j");
-                    //CONDGFREE(aband->call_played);
+                    //zg_free0(aband->call_played);
                     //aband->iscall = 1;
                     //dbg("convert_esc3: set iscall=%d\n", aband->iscall);
                     //aband->call_played = g_strdup(TMPQ.callsign);
@@ -1224,7 +1256,7 @@ gchar *convert_esc(gchar *format, int *undef, int flags, time_t now){
                 break;
             case 'e':
 #ifdef HAVE_SNDFILE
-                if (ssbd->code) g_string_append(gs, "code");
+                if (gssbd->code) g_string_append(gs, "code");
 #endif
                 break;
             case 'i':
@@ -1233,6 +1265,15 @@ gchar *convert_esc(gchar *format, int *undef, int flags, time_t now){
                 g_string_append(gs, c2);
                 g_free(c2);
                 break;
+            case 'j':
+                if (ctest && TMPQ.callsign) {
+                    char *name = find_name_by_call(namedb, TMPQ.callsign);
+                    if (name) g_string_append(gs, name);
+                }else{
+                    *undef = 1;
+                }
+                break;    
+                break;
             case 'n':
                 if (ctest && TMPQ.qsonrr) g_string_append(gs, conv_nr(TMPQ.qsonrr));
                 else *undef=1;
@@ -1248,18 +1289,18 @@ gchar *convert_esc(gchar *format, int *undef, int flags, time_t now){
                 break; 
             case 's': 
 #ifdef HAVE_SNDFILE
-                g_string_sprintfa(gs, "%d", ssbd->serno);
+                g_string_sprintfa(gs, "%d", gssbd->serno);
 #endif
                 break;
             case 't':
                 if (ctest) g_string_append(gs, ctest->directory);
-				else g_string_append_printf(gs, "%s/tucnak/00000000", getenv("HOME"));
+				else g_string_append_printf(gs, "%s/00000000", tucnak_dir);
                 break;
             case 'v':
 #ifdef HAVE_SNDFILE
-                if (ssbd->callsign) {
+                if (gssbd->callsign) {
                     char *d;
-                    for (d=ssbd->callsign; *d!='\0'; d++){
+                    for (d=gssbd->callsign; *d!='\0'; d++){
                         if (*d=='/')
                             g_string_append_c(gs, '_');
                         else
@@ -1279,7 +1320,7 @@ gchar *convert_esc(gchar *format, int *undef, int flags, time_t now){
                 break;    
             case 'm':
                 c++;
-                switch(lowcase(*c)){
+                switch(z_char_lc(*c)){
                     case '\0':
                         goto brk2;
                         break;
@@ -1324,7 +1365,7 @@ brk2:;
     }
 x:;    
     ret = g_strdup(gs->str);
-    ret = optimize_path(ret);
+    ret = z_optimize_path(ret);
     g_string_free(gs, TRUE);
     
     return ret;
@@ -1358,54 +1399,13 @@ gchar *convert_cq(struct cq *cq){
                 if (c4) {mem_free(c4); c4=NULL;}\
                 if (c5) {mem_free(c5); c5=NULL;}
 
-gchar *optimize_path(gchar *src){
 
-    int first, last, i;
-    gchar **items, *c;
-    GString *gs;
 
-    if (!src) return NULL;
-    if (src[0]=='\0') return src;
-    if (strcmp(src, ".")==0) return src;
-    if (strcmp(src, "./")==0) return src;
-    
-    first=src[0]=='/';
-    last=src[0]&&src[strlen(src)-1]=='/';
-    items=g_strsplit(src, "/", 0); 
-    if (!items) return src;
-
-    gs=g_string_sized_new(strlen(src));
-    g_free(src);
-    src=NULL;
-    if (first) g_string_append_c(gs, '/');
-
-    for (i=0; items[i]; i++){
-        if (strcmp(items[i], ".")==0){
-            continue;
-        }
-        if (items[i+1] && strcmp(items[i+1], "..")==0){
-            i++;
-            continue;
-        }
-        if (strlen(items[i])==0) continue;
-        g_string_append(gs, items[i]);
-        if (items[i+1]) g_string_append_c(gs, '/');
-    }
-    if (last && (!gs->str[0] || (gs->str[0] && gs->str[strlen(gs->str)-1]!='/'))) 
-        g_string_append_c(gs, '/');
-
-    c=g_strdup(gs->str);
-    g_string_free(gs, TRUE);
-    g_strfreev(items);
-    return c;
-    
-}
-
-void menu_runmode(cba_t cba){
+void menu_runmode(void *arg){
     runmode(1);
 }
 
-void menu_spmode(cba_t cba){
+void menu_spmode(void *arg){
     runmode(0);
 }
 
@@ -1486,32 +1486,32 @@ static char *ac_txts[] = {
     NULL
 };
 
-void ac_cq3(cba_t cba){
+void ac_cq3(void *arg){
     enum extcq conv[4] = {EC_1ST, EC_2ND, EC_ODD, EC_EVEN};
 
     //log_addf("ac_cq3 %d", cba.int_);
-    ac_cq_i2 = (int)GETCBA(cba, int_);
+    ac_cq_i2 = GPOINTER_TO_INT(arg);
     
     if (!gses) return;
-    gses->extcq = conv[(int)GETCBA(cba, int_)];
+    gses->extcq = conv[GPOINTER_TO_INT(arg)];
     gses->ac_cq->cw_str = ac_txts[ac_cq_i1];
     cq_run_by_number(-1);
 }
 
-void ac_cq2(cba_t cba){
+void ac_cq2(void *arg){
     struct menu_item *mi = NULL;
 
     //log_addf("ac_cq2 %d", cba.int_);
-    ac_cq_i1 = (int)GETCBA(cba, int_);
+    ac_cq_i1 = GPOINTER_TO_INT(arg);
     
     if (!(mi = new_menu(1))) return;
 
-    add_to_menu(&mi,CTEXT(T_1ST),  CTEXT(T_HK_1ST),  CTEXT(T_HK_1ST),  MENU_FUNC ac_cq3, (cba_t)0, 0);    
-    add_to_menu(&mi,CTEXT(T_2ND),  CTEXT(T_HK_2ND),  CTEXT(T_HK_2ND),  MENU_FUNC ac_cq3, (cba_t)1, 0);    
-    add_to_menu(&mi,CTEXT(T_ODD),  CTEXT(T_HK_ODD),  CTEXT(T_HK_ODD),  MENU_FUNC ac_cq3, (cba_t)2, 0);    
-    add_to_menu(&mi,CTEXT(T_EVEN), CTEXT(T_HK_EVEN), CTEXT(T_HK_EVEN), MENU_FUNC ac_cq3, (cba_t)3, 0);    
+    add_to_menu(&mi,CTEXT(T_1ST),  CTEXT(T_HK_1ST),  CTEXT(T_HK_1ST),  MENU_FUNC ac_cq3, GINT_TO_POINTER(0), 0);    
+    add_to_menu(&mi,CTEXT(T_2ND),  CTEXT(T_HK_2ND),  CTEXT(T_HK_2ND),  MENU_FUNC ac_cq3, GINT_TO_POINTER(1), 0);    
+    add_to_menu(&mi,CTEXT(T_ODD),  CTEXT(T_HK_ODD),  CTEXT(T_HK_ODD),  MENU_FUNC ac_cq3, GINT_TO_POINTER(2), 0);    
+    add_to_menu(&mi,CTEXT(T_EVEN), CTEXT(T_HK_EVEN), CTEXT(T_HK_EVEN), MENU_FUNC ac_cq3, GINT_TO_POINTER(3), 0);    
 
-    do_menu_selected(mi, CBA0, ac_cq_i2);
+    do_menu_selected(mi, NULL, ac_cq_i2);
 
 }
 
@@ -1521,11 +1521,11 @@ void ac_cq(void){
     
     if (!(mi = new_menu(3))) return;
     for (i=0; ac_txts[i]!=NULL; i++){
-        add_to_menu(&mi,stracpy(ac_txts[i]),"", "", MENU_FUNC ac_cq2, (cba_t)i, 0);    
+        add_to_menu(&mi,g_strdup(ac_txts[i]),"", "", MENU_FUNC ac_cq2, GINT_TO_POINTER(i), 0);    
         if (strlen(ac_txts[i])>max) max = strlen(ac_txts[i]);
     }
 
     set_window_ptr(gses->win, (term->x-max)/2,(term->y-2-i)/2);
-    do_menu_selected(mi, CBA0, ac_cq_i1);
+    do_menu_selected(mi, NULL, ac_cq_i1);
     
 }
