@@ -1,8 +1,6 @@
 /*
     Tucnak - VHF contest log
-    Copyright (C) 2002-2006  Ladislav Vaiz <ok1zia@nagano.cz>
-    and Even Nermerson (ftdi_new and ftdi_free) and other
-    authors of libftdi
+    Copyright (C) 2002-2015  Ladislav Vaiz <ok1zia@nagano.cz>
 
     This program is free software; you can redistribute it and/or
     modify it under the terms of the GNU General Public License
@@ -10,10 +8,16 @@
 
 */
 
+
 #include "header.h"
 
-#ifdef HAVE_LIBFTDI
+#include "cwdaemon.h"
+#include "davac4.h"
+#include "fifo.h"
+#include "language2.h"
+#include "rc.h"
 
+#ifdef Z_HAVE_LIBFTDI
 int davac4_init(struct cwdaemon *cwda){
     cwda->ftdi = NULL;
     MUTEX_LOCK(cwda->ftdi);
@@ -45,7 +49,7 @@ int davac4_open(struct cwdaemon *cwda, int verbose){
 
     cwda->ftdi = ftdi_new();
     if (!cwda->ftdi){
-        if (verbose) log_addf("Can't init ftdi library for davac4");
+        if (verbose) log_addf(VTEXT(T_CANT_INIT_FTDI));
         dbg("davac4_open=%d\n", 1);
         ret = 1;
         goto x;
@@ -55,7 +59,7 @@ int davac4_open(struct cwdaemon *cwda, int verbose){
     dbg("ftdi_usb_open cwda->ftdi=%p ret=%d\n", cwda->ftdi, ret);
     if (ret){
         if (verbose) {
-            log_addf("Can't open davac4 device %04x:%04x, error=%d %s", cfg->cwda_vid, cfg->cwda_pid, ret, ftdi_get_error_string(cwda->ftdi));
+            log_addf(VTEXT(T_CANT_OPEN_DAVAC4), cfg->cwda_vid, cfg->cwda_pid, ret, ftdi_get_error_string(cwda->ftdi));
             if (ret==-8) log_addf("Maybe try to run as root: \"adduser %s davac4\" and relogin", getenv("USER"));
         }
         davac4_free(cwda);
@@ -64,7 +68,7 @@ int davac4_open(struct cwdaemon *cwda, int verbose){
     }
     ret=ftdi_set_baudrate(cwda->ftdi, 115200);
     if (ret){
-        if (verbose) log_addf("Can't set baudrate for davac4, error=%d %s", ret, ftdi_get_error_string(cwda->ftdi));
+        if (verbose) log_addf(VTEXT(T_CANT_SET_BAUDRATE), ret, ftdi_get_error_string(cwda->ftdi));
         davac4_free(cwda);
         dbg("davac4_open=%d\n", ret);
         goto x;
@@ -72,7 +76,7 @@ int davac4_open(struct cwdaemon *cwda, int verbose){
     //ret=ftdi_enable_bitbang(cwda->ftdi, 0xff);
     ret=ftdi_set_bitmode(cwda->ftdi, 0xff, BITMODE_BITBANG);
     if (ret){
-        if (verbose) log_addf("Can't enable bitbang, error=%d %s", ret, ftdi_get_error_string(cwda->ftdi));
+        if (verbose) log_addf(VTEXT(T_CANT_ENABLE_BITBANG), ret, ftdi_get_error_string(cwda->ftdi));
         davac4_free(cwda);
         dbg("davac4_open=%d\n", ret);
         goto x;
@@ -109,7 +113,7 @@ int davac4_cw(struct cwdaemon *cwda, int onoff){
     
     ret=ftdi_write_data(cwda->ftdi, &cwda->ftdi_state, 1);
     if (ret!=1){
-        log_addf("Can't write to davac4, error=%d %s", ret, ftdi_get_error_string(cwda->ftdi)); 
+        log_addf(VTEXT(T_CANT_WRITE_TO_DAVAC4), ret, ftdi_get_error_string(cwda->ftdi)); 
         davac4_free(cwda);
         goto x;
     }
@@ -135,7 +139,7 @@ int davac4_ptt(struct cwdaemon *cwda, int onoff){
     ret=ftdi_write_data(cwda->ftdi, &cwda->ftdi_state, 1);
     
     if (ret!=1){
-        log_addf("Can't write to davac4, error=%d %s", ret, ftdi_get_error_string(cwda->ftdi)); 
+        log_addf(VTEXT(T_CANT_WRITE_TO_DAVAC4), ret, ftdi_get_error_string(cwda->ftdi)); 
         davac4_free(cwda);
         goto x;
     }
@@ -160,7 +164,7 @@ int davac4_ssbway(struct cwdaemon *cwda, int onoff){
     
     ret=ftdi_write_data(cwda->ftdi, &cwda->ftdi_state, 1);
     if (ret!=1){
-        log_addf("Can't write to davac4, error=%d %s", ret, ftdi_get_error_string(cwda->ftdi)); 
+        log_addf(VTEXT(T_CANT_WRITE_TO_DAVAC4), ret, ftdi_get_error_string(cwda->ftdi)); 
         davac4_free(cwda);
         goto x;
     }      
@@ -184,7 +188,7 @@ int davac4_monitor(struct cwdaemon *cwda, int onoff){
     
     ret=ftdi_write_data(cwda->ftdi, &cwda->ftdi_state, 1);
     if (ret!=1){
-        log_addf("Can't write to davac4, error=%d %s", ret, ftdi_get_error_string(cwda->ftdi)); 
+        log_addf(VTEXT(T_CANT_WRITE_TO_DAVAC4), ret, ftdi_get_error_string(cwda->ftdi)); 
         davac4_free(cwda);
         goto x;
     }
@@ -209,7 +213,7 @@ int davac4_band (struct cwdaemon *cwda, int bandsw){
     
     ret=ftdi_write_data(cwda->ftdi, &cwda->ftdi_state, 1);
     if (ret!=1){
-        log_addf("Can't write to davac4, error=%d %s", ret, ftdi_get_error_string(cwda->ftdi)); 
+        log_addf(VTEXT(T_CANT_WRITE_TO_DAVAC4), ret, ftdi_get_error_string(cwda->ftdi)); 
         davac4_free(cwda);
         goto x;
     }
@@ -218,102 +222,84 @@ x:;
     return ret;
 }
 
-#ifndef HAVE_FTDI_NEW
-
-/* compatibility with libftdi < 0.11 */
-/* code from libftdi 0.11 */
-
-struct ftdi_context *ftdi_new()
-{
-    struct ftdi_context * ftdi = (struct ftdi_context *)malloc(sizeof(struct ftdi_context));
-
-    if (ftdi == NULL) {
-        return NULL;
-    }
-
-    if (ftdi_init(ftdi) != 0) {
-        free(ftdi);
-        return NULL;
-    }
-
-    return ftdi;
-}
-
-void ftdi_free(struct ftdi_context *ftdi)
-{
-    ftdi_deinit(ftdi);
-    free(ftdi);
-}
-
-#endif
-
 #define DUMP_EEPROMx
 
-void usb_info(void){
+void usb_info(GString *gs){
     int ret;
     struct ftdi_context ftdi;
     struct ftdi_device_list *list, *dev;
     int ids[] = {0x0403, 0x6001, 
-                 0xa600, 0xE110,
-                 0xa600, 0xE111,
-                 0xa600, 0xE112,
+                 0xA600, 0xE110,
+                 0xA600, 0xE111,
+                 0xA600, 0xE112,
+                 0xA600, 0xE113,
+                 0xA600, 0xE114,
+                 0x04D8, 0xFB56, // FUNcube Dongle
+                 0x04D8, 0xFB31, // FUNcube Dongle Pro+
                  0};
     int idi;
-    char manufacturer[128], description[128], serial[128];
+#ifdef Z_HAVE_FTDI_USB_GET_STRINGS
+    char manufacturer[128], description[128];
+	char serial[128];
+#endif
 #ifdef DUMP_EEPROM
     unsigned char eeprom_buf[128];
 #endif
     
-    printf("\n  usb_info:\n");
+    g_string_sprintfa(gs, "\n  usb_info:\n");
     ret = ftdi_init(&ftdi); 
     if (ret < 0) {
-        printf("error calling ftdi_init, error=%d %s", ret, ftdi_get_error_string(&ftdi));
+        g_string_sprintfa(gs, "error calling ftdi_init, error=%d %s", ret, ftdi_get_error_string(&ftdi));
         return;
     }
     for (idi = 0; ids[idi] != 0; idi += 2){
         ret = ftdi_usb_find_all(&ftdi, &list, ids[idi], ids[idi+1]);
         if (ret < 0) {
-            printf("error calling ftdi_usb_find_all, error=%d %s", ret, ftdi_get_error_string(&ftdi));
+            g_string_sprintfa(gs, "error calling ftdi_usb_find_all, error=%d %s\n", ret, ftdi_get_error_string(&ftdi));
             return;
         }
         if (ret == 0) continue;
         for (dev = list; dev != NULL; dev = dev->next){
-            printf("%04x:%04x  ", dev->dev->descriptor.idVendor, dev->dev->descriptor.idProduct);
-#if HAVE_FTDI_USB_GET_STRINGS
+            g_string_sprintfa(gs, "%04x:%04x  ", dev->dev->descriptor.idVendor, dev->dev->descriptor.idProduct);
+#ifdef Z_HAVE_FTDI_USB_GET_STRINGS
             ret = ftdi_usb_get_strings(&ftdi, dev->dev, manufacturer, 128, description, 128, serial, 128);
             if (ret < 0){
-                printf("Can't get strings, error=%d %s", ret, ftdi_get_error_string(&ftdi));
+                g_string_sprintfa(gs, "Can't get strings, error=%d %s\n", ret, ftdi_get_error_string(&ftdi));
             }else{
-                printf("%s %s ", manufacturer, description);
+                g_string_sprintfa(gs, "%s %s ", manufacturer, description);
                 if (strlen(serial)>0 && serial[0]!='?')
-                    printf("%s  ", serial);
+                    g_string_sprintfa(gs, "%s  ", serial);
             }
 #endif
-            printf("\n");
-#ifdef DUMP_EEPROM
+            if (ids[idi] == 0x04D8){
+                g_string_sprintfa(gs, "\n");
+                continue;
+            }
             ret = ftdi_usb_open_dev(&ftdi, dev->dev);
             if (ret < 0){
-                printf("Can't open device, error=%d %s", ret, ftdi_get_error_string(&ftdi));
+                g_string_sprintfa(gs, "Can't open device, error=%d %s\n", ret, ftdi_get_error_string(&ftdi));
             }else{
+                g_string_append(gs, "Open OK ");
+#ifdef DUMP_EEPROM
                 ret = ftdi_read_eeprom(&ftdi, eeprom_buf);
                 if (ret < 0){
-                    printf("Can't open device, error=%d %s", ret, ftdi_get_error_string(&ftdi));
+                    g_string_sprintfa(gs, "Can't open device, error=%d %s\n", ret, ftdi_get_error_string(&ftdi));
                 }else{
                     int i;
                     unsigned short chksum_in_eeprom, chksum_computed;
                     for (i=0; i<128; i++){
-                        printf("%02x ", (unsigned char)eeprom_buf[i]);
-                        if (i%16==15) printf("\n");
+                        g_string_sprintfa(gs, "%02x ", (unsigned char)eeprom_buf[i]);
+                        if (i%16==15) g_string_sprintfa(gs, "\n");
                         
                     }
                     chksum_in_eeprom = eeprom_buf[0x7e] + 256 * eeprom_buf[0x7f];
                     chksum_computed = ftdi_checksum(eeprom_buf, 0x80);
-                    printf("Checksum eeprom=%04x computed=%04x ", chksum_in_eeprom, chksum_computed);
+                    g_string_sprintfa(gs, "Checksum eeprom=%04x computed=%04x ", chksum_in_eeprom, chksum_computed);
                 }
+#endif
                 ftdi_usb_close(&ftdi);
             }
-            printf("\n");
-#endif
+            g_string_sprintfa(gs, "\n");
         }
     }
     ftdi_list_free(&list);
