@@ -1,6 +1,6 @@
 /*
     Tucnak - VHF contest log
-    Copyright (C) 2002-2006  Ladislav Vaiz <ok1zia@nagano.cz>
+    Copyright (C) 2002-2015  Ladislav Vaiz <ok1zia@nagano.cz>
 
     This program is free software; you can redistribute it and/or
     modify it under the terms of the GNU General Public License
@@ -8,8 +8,22 @@
 
 */
 
+
 #include "header.h"
 
+#include "cwdb.h"
+#include "dwdb.h"
+#include "fifo.h"
+#include "inputln.h"
+#include "main.h"
+#include "namedb.h"
+#include "qrvdb.h"
+#include "qsodb.h"
+#include "session.h"
+#include "stats.h"
+#include "tregex.h"
+#include "tsdl.h"
+
 
 struct dw *dw;
 static gchar *dw_next;  /* wwl1 */
@@ -18,6 +32,8 @@ static gchar *wd_next;  /* dxc1 */
 struct dw *init_dw(void){
     struct dw *dw;
 
+	progress(VTEXT(T_INIT_DWDB));		
+
     dw = g_new0(struct dw,1 );
 
     dw->pd = g_hash_table_new(g_str_hash, g_str_equal);
@@ -32,11 +48,6 @@ struct dw *init_dw(void){
 
 
 
-gboolean free_gpointer_item(gpointer key, gpointer value, gpointer user_data){
-    if (key) g_free(key);
-    if (value) g_free(value);
-    return TRUE;
-}
 
 static gboolean free_dw_item(gpointer key, gpointer value, gpointer user_data){
     struct dw_item *dwi;
@@ -68,6 +79,8 @@ static gboolean free_wd_item(gpointer key, gpointer value, gpointer user_data){
 
 
 void free_dw(struct dw *dw){
+	progress(VTEXT(T_FREE_DWDB));		
+
     g_hash_table_foreach_remove(dw->pd, free_gpointer_item, NULL);
     g_hash_table_destroy(dw->pd);
     g_hash_table_foreach_remove(dw->dw, free_dw_item, NULL);
@@ -92,74 +105,11 @@ gint get_wd_size(struct dw *dw){
 
 #define DW_DELIM " \t\r\n"
 
-gchar *safe_fgets(GString *gs, FILE *f, int stripcomment){
-    char s[102], *c;
-    int len;
-    
-    
-    g_string_truncate(gs, 0);
-
-    while (1){
-        c = fgets(s, 100, f);
-        if (!c) return NULL;
-        
-        len = strlen(s);
-        if (len>0 && s[len-1]=='\n'){ /* MAC ?*/
-            s[len-1]='\0';
-            if (len>1 && s[len-2]=='\r') s[len-2]='\0';
-            g_string_append(gs, s);
-            break;
-        }
-        g_string_append(gs, s);
-    }
-    
-    if (stripcomment){
-        char *c;
-
-        c = strchr(gs->str, '#');
-        if (c) g_string_truncate(gs, c - gs->str);
-    }
-        
-     /*dbg("%3d '%s'\n",gs->len, gs->str);*/
-    return gs->str;    
-}
-
-
-gchar *safe_mgets(GString *gs, const char *file, long int *pos, const long int len, int stripcomment){
-	const char *c;
-
-    /*dbg("safe_mgets(%d,%d,%d)\n", *pos, len, stripcomment);*/
-	g_string_truncate(gs, 0);
-    
-    if (*pos>=len) return NULL;
-
-	for (c=file+(*pos); c!=file+len; c++,(*pos)++){
-		switch(*c){
-			case '\r':
-				continue;
-			case '\n':
-				c++;(*pos)++;
-				goto brk;
-		}
-		g_string_append_c(gs, *c);
-	}
-brk:;  
-/*    dbg("brk at %d\n", *pos);*/
-    if (stripcomment){
-        char *c;
-
-        c = strchr(gs->str, '#');
-        if (c) g_string_truncate(gs, c - gs->str);
-    }
-//    dbg("safe_mgets: %3d '%s' *pos=%d len=%d\n",gs->len, gs->str, *pos, len);
-    if (*pos>len) return NULL;
-    return gs->str;    
-}
 
 void add_pd(struct dw *dw, gchar *prefix, gchar *dxc){
     gpointer orig_prefix, orig_dxc;
 
-/*    dbg("add_pd(prefix='%s', dxc='%s'\n", prefix, dxc );*/
+//    dbg("add_pd(prefix='%s', dxc='%s'\n", prefix, dxc );
     
     if (g_hash_table_lookup_extended(dw->pd, (gpointer) prefix,
                &orig_prefix, &orig_dxc)){
@@ -175,10 +125,10 @@ void add_pd(struct dw *dw, gchar *prefix, gchar *dxc){
 struct dw_item *add_dxc(struct dw *dw, gchar *dxc, gchar *wwl0, gchar *wwl1, int waz, int itu, gchar *continent, gchar *dxcname, double latitude, double longitude){
     struct dw_item *dxci;
 
-    /*dbg("add_dxc(dxc='%s', wwl='%s' '%s')\n",dxc, wwl0, wwl1);*/
+//    dbg("add_dxc(dxc='%s', wwl='%s' '%s')\n",dxc, wwl0, wwl1);
     
 
-    dxci = g_hash_table_lookup(dw->dw, dxc);
+    dxci = (struct dw_item*)g_hash_table_lookup(dw->dw, dxc);
     if (!dxci) {
        dxci = g_new0(struct dw_item, 1); 
        
@@ -306,7 +256,7 @@ int load_dw_from_mem(struct dw *dw, const char *file, const long int len){
 	pos=0;
     while(1){
         do {
-           if (!safe_mgets(pref,  file, &pos, len, 1)) goto abort;
+           if (!zfile_mgets(pref,  file, &pos, len, 1)) goto abort;
         }while (pref->len==0);
         
         dxc = strtok_r(pref->str, DW_DELIM, &token_ptr);
@@ -317,10 +267,10 @@ int load_dw_from_mem(struct dw *dw, const char *file, const long int len){
             add_pd(dw, prefix, dxc);
         } 
         
-        if (!safe_mgets(wwls,    file, &pos, len, 1)) break;
+        if (!zfile_mgets(wwls,    file, &pos, len, 1)) break;
         if (!wwls->len) continue;
 
-        if (!safe_mgets(centers, file, &pos, len, 1)) break;
+        if (!zfile_mgets(centers, file, &pos, len, 1)) break;
 
         if (centers->len)  c0 = strtok_r(centers->str, DW_DELIM, &token_ptr);
         else               c0 = strtok_r(wwls->str,    DW_DELIM, &token_ptr);
@@ -401,7 +351,6 @@ int load_cty_from_mem(struct dw *dw, const char *file, const long int len){
     gchar *dxc, *dxcname;
     char wwl4[6];
 	long int pos;
-    struct dw_item *dxci;
 	double latitude, longitude;
     
     
@@ -413,11 +362,11 @@ int load_cty_from_mem(struct dw *dw, const char *file, const long int len){
         if (gs_next->len>0){
             g_string_assign(gs, gs_next->str);
         }else{
-            if (!safe_mgets(gs,  file, &pos, len, 0)) break;
+            if (!zfile_mgets(gs,  file, &pos, len, 0)) break;
         }
         
         while(1){
-            if (!safe_mgets(gs_next,  file, &pos, len, 0)){
+            if (!zfile_mgets(gs_next,  file, &pos, len, 0)){
                 g_string_truncate(gs_next,0); /* next safe_mgets breaks loop */
                 break;
             }
@@ -465,7 +414,7 @@ int load_cty_from_mem(struct dw *dw, const char *file, const long int len){
             *slash = '\0';
             slash++;;
         }         */
-        uc(dxc);
+        z_str_uc(dxc);
 
         
         //dbg("dxc='%s'\n", dxc);
@@ -499,7 +448,7 @@ int load_cty_from_mem(struct dw *dw, const char *file, const long int len){
         compute_wwl4(wwl4, -atof(line[5]), atof(line[4]));
 
         /*dbg("h=%s, w=%s, d=%s, wwl4=%s \n", line[4], line[5], line[7], wwl4);*/
-        dxci = add_dxc(dw, dxc, wwl4, NULL, waz, itu, continent, dxcname, latitude, longitude); 
+        add_dxc(dw, dxc, wwl4, NULL, waz, itu, continent, dxcname, latitude, longitude); 
         
         g_strfreev(line);
         
@@ -514,14 +463,20 @@ void read_dw_files(struct dw *dw){
     gchar *s;
 	int ret;
     
-    s = g_strconcat(getenv("HOME"), "/tucnak/cty.dat", NULL);
+	progress(VTEXT(T_LOAD_DWDB));		
+
+    s = g_strconcat(tucnak_dir, "/cty.dat", NULL);
     ret=load_cty_from_file(dw, s);
     g_free(s);
 	if (ret<0){
-    	load_cty_from_mem(dw, txt_cty, sizeof(txt_cty));
+		GString *gs = g_string_new("");
+		g_string_append(gs, txt_cty);
+		g_string_append(gs, txt_cty1);
+    	load_cty_from_mem(dw, gs->str, gs->len);
+		g_string_free(gs, TRUE);
 	}
     
-    s = g_strconcat(getenv("HOME"), "/tucnak/tucnakdw", NULL);
+    s = g_strconcat(tucnak_dir, "/tucnakdw", NULL);
     ret=load_dw_from_file(dw, s);
     g_free(s);
 	if (ret<0){
@@ -530,11 +485,11 @@ void read_dw_files(struct dw *dw){
 }
 
 
-struct dw_item *get_dw_item_by_call(struct dw *dw, gchar *call){
+struct dw_item *get_dw_item_by_call(struct dw *dw, const char *call){
     char dxcc[25];
 	
     get_dxcc(dw, dxcc, call);
-    return g_hash_table_lookup(dw->dw, dxcc);
+    return (struct dw_item *)g_hash_table_lookup(dw->dw, dxcc);
 }
 
 
@@ -609,7 +564,7 @@ int get_susp_call(struct cw *cw, struct dw *dw, gchar *call, gchar *wwl){
     susp=get_susp(cw, dw, call, wwl, 1);
     if (susp) return 1; /* get_susp muze vratit 0x100 a co bysme s tim delali */
 
-    get_raw_call(s, call);
+    z_get_raw_call(s, call);
     if (strcmp(s, call)==0){ /* without /p */
         strcat(s, "/P");
         //dbg("without %s", s);
@@ -643,7 +598,7 @@ int get_susp_loc(struct cw *cw, struct dw *dw, gchar *call, gchar *wwl){
         g_string_sprintfa(gs, "%s %s\n", qso->callsign?qso->callsign:"?", qso->locator?qso->locator:"?"); \
     }err=1;}
 
-static enum suspcall qso_info(struct qso *qso, GString *gs, GString *gs2){
+enum suspcall qso_info(struct qso *qso, GString *gs, GString *gs2){
     char s[40],s1[40], raw[40];
     int err, susp, k;
     enum suspcall ret;
@@ -654,15 +609,15 @@ static enum suspcall qso_info(struct qso *qso, GString *gs, GString *gs2){
 
 
     ret=SUSP_NONE;
-    if (!qso) return -1;
-    if (qso->error) return -1;
-    if (qso->dupe) return -1;
+    if (!qso) return (enum suspcall)-1;
+    if (qso->error) return (enum suspcall)-1;
+    if (qso->dupe) return (enum suspcall)-1;
 
     err=0;
 
-    if (qso->band && qso->band->stats && qso->band->stats->nqsos > 0){
+    /*if (qso->band && qso->band->stats && qso->band->stats->nqsos > 0){
         if (qso->qsop > qso->band->stats->nqsop/qso->band->stats->nqsos) if (ret < SUSP_QRB) ret = SUSP_QRB;
-    }
+    } */
 
     if (qso->remark && *qso->remark) {
         if (ret < SUSP_REM) ret = SUSP_REM;
@@ -670,28 +625,36 @@ static enum suspcall qso_info(struct qso *qso, GString *gs, GString *gs2){
 
     if (!find_name_by_call(namedb, qso->callsign) &&
         !find_wwl_by_call(cw, qso->callsign)){
-        ADDQSO;
-        g_string_sprintfa(gs, "Callsign unknown in namedb and C_W\n");
+		if (gs){
+			ADDQSO;
+			g_string_append(gs, VTEXT(T_CALLSIGN_UNKNOWN));
+		}
         if (ret < SUSP_WARN) ret = SUSP_WARN;
     }
 
     if (qso->mode == MOD_SSB_CW || qso->mode == MOD_CW_SSB){
-        ADDQSO;
-        g_string_sprintfa(gs, "Crossmode QSO ?\n");
+		if (gs){
+			ADDQSO;
+			g_string_append(gs, VTEXT(T_CROSSMODE_QSO_N));
+		}
         if (ret < SUSP_WARN) ret = SUSP_WARN;
     }
 
     if (!get_dw_item_by_call(dw, qso->callsign)){
-        ADDQSO;
-        g_string_sprintfa(gs, "Unknown DXCC prefix\n");
+		if (gs){
+			ADDQSO;
+			g_string_append(gs, VTEXT(T_UNKNOWN_DXCC_PREFIXN));
+		}
         if (ret < SUSP_ERR) ret = SUSP_ERR;
     }
 
 
     if (qso->locator && *qso->locator){
         if (!wwl_is_ok_by_call(dw, qso->locator, qso->callsign)){
-            ADDQSO;
-            g_string_sprintfa(gs, "Prefix and big wwl don't match\n");
+			if (gs){
+				ADDQSO;
+				g_string_append(gs, VTEXT(T_PREFIX_AND_BIG_WWL));
+			}
             if (ret < SUSP_ERR) ret = SUSP_ERR;
         }
     }
@@ -700,28 +663,32 @@ static enum suspcall qso_info(struct qso *qso, GString *gs, GString *gs2){
     wwl1=find_wwl_by_call(cw, NULL); 
 
     susp=0;
-    if (wwl0 != NULL && qso->locator != NULL){ /* we don't know this call, cannot be suspicious */
+    if (wwl0!=NULL){ /* we don't know this call, cannot be suspicious */
         susp = 1;
-        if (wwl0 && strcasecmp(wwl0, qso->locator)==0) susp = 0; 
-        if (wwl1 && strcasecmp(wwl1, qso->locator)==0) susp = 0; 
+        if (wwl0 && qso->locator && strcasecmp(wwl0, qso->locator)==0) susp = 0; 
+        if (wwl1 && qso->locator && strcasecmp(wwl1, qso->locator)==0) susp = 0; 
     }
 
     if (qso->locator && *qso->locator){
         if (susp){
-            ADDQSO;
-            g_string_sprintfa(gs, "Known call from unknown locator\n");
+			if (gs){
+				ADDQSO;
+				g_string_append(gs, VTEXT(T_KNOWN_CALL_UNK_LOC));
+			}
             if (ret < SUSP_WARN) ret = SUSP_WARN;
         }
     }
     
-    get_raw_call(raw, qso->callsign);
+    z_get_raw_call(raw, qso->callsign);
     strcpy(s, raw);
     if (strcmp(s, qso->callsign)==0){ /* without /p */
         strcpy(s1, s);
         strcat(s, "/P");
         if (find_wwl_by_call_newer(cw, s, cw->minstamp)) {
-            ADDQSO;
-            g_string_sprintfa(gs, "Call is without /P and in C_W exists call with /P\n");
+			if (gs){
+				ADDQSO;
+				g_string_append(gs, VTEXT(T_CALL_WITHOUT_P));
+			}
             if (ret < SUSP_WARN) ret = SUSP_WARN;
         }
         strcat(s1, "/");
@@ -730,26 +697,32 @@ static enum suspcall qso_info(struct qso *qso, GString *gs, GString *gs2){
             s[strlen(s)+1]='\0';
             s[strlen(s)]='0'+k;
             if (find_wwl_by_call_newer(cw, s, cw->minstamp)) {
-                ADDQSO;
-                g_string_sprintfa(gs, "Call is without /%d and in C_W exists call with /%d\n", k, k);
+				if (gs){
+					ADDQSO;
+					g_string_sprintfa(gs, VTEXT(T_CALL_WITHOUT_D), k, k);
+				}
                 if (ret < SUSP_WARN) ret = SUSP_WARN;
             }
         }
 
     }else{        /* call is with /p */
         if (find_wwl_by_call_newer(cw, s, cw->minstamp)) {
-            ADDQSO;
-            g_string_sprintfa(gs, "Call is with /P and in C_W exists call without /P or CEPT prefix\n");
+			if (gs){
+				ADDQSO;
+				g_string_append(gs, VTEXT(T_CALL_IS_WITH_P_AND));
+			}
             if (ret < SUSP_WARN) ret = SUSP_WARN;
         }
     }
 
-    if (qso->locator != NULL){
+    if (qso->locator != NULL && gs2){
         g_string_truncate(gs2, 0);
         get_susp_ambiguous_call(cw, qso->band, qso->callsign, qso->locator, gs2, 1);
         if (gs2->len>0){
-            ADDQSO;
-            g_string_sprintfa(gs, "Other similar call known from this locator. Maybe %s?\n", gs2->str);
+			if (gs){
+				ADDQSO;
+				g_string_sprintfa(gs, VTEXT(T_OTHER_SIMILAR_CALL_KNOWN), gs2->str);
+			}
             if (ret < SUSP_WARN) ret = SUSP_WARN;
         }
     }
@@ -762,7 +735,7 @@ static enum suspcall qso_info(struct qso *qso, GString *gs, GString *gs2){
                 q = (struct qso *)g_ptr_array_index(b->qsos, j);
                 if (q->error || q->dupe) continue;
 
-                get_raw_call(s, q->callsign);
+                z_get_raw_call(s, q->callsign);
 
                 /*if (q->callsign[0]=='D' && q->callsign[1]=='M' && q->callsign[2]=='7'){
                      dbg("call %s %s %s %d\n", q->callsign, s, raw, strcmp(s, raw));
@@ -770,14 +743,18 @@ static enum suspcall qso_info(struct qso *qso, GString *gs, GString *gs2){
                 if (strcmp(s, raw) != 0) continue; /* not same station */
                  /*dbg("RAW match %s %s\n", s, raw);*/
                 if (strcmp(q->callsign, qso->callsign) != 0){ 
-                    ADDQSO;
-                    g_string_sprintfa(gs, "Callsign on band %c is %s\n", toupper(b->bandchar), q->callsign);
+					if (gs){
+						ADDQSO;
+						g_string_sprintfa(gs, VTEXT(T_CALL_ON_BAND_S), b->bandname, q->callsign);
+					}
                     if (ret < SUSP_WARN) ret = SUSP_WARN;
                 }
                 
                 if (qso->locator != NULL && strcmp(q->locator, qso->locator) != 0){
-                    ADDQSO;
-                    g_string_sprintfa(gs, "Locator on band %c is %s\n", toupper(b->bandchar), q->locator);
+					if (gs){
+						ADDQSO;
+						g_string_sprintfa(gs, VTEXT(T_LOCATOR_ON_BAND), b->bandname, q->locator);
+					}
                     if (ret < SUSP_WARN) ret = SUSP_WARN;
                 }
 
@@ -788,7 +765,7 @@ static enum suspcall qso_info(struct qso *qso, GString *gs, GString *gs2){
     return ret;
 }
 
-void menu_qso_check(cba_t cba){
+void menu_qso_check(void *arg){
     GString *gs, *gs2;
     struct band *band;
     struct qso *qso;
@@ -800,29 +777,31 @@ void menu_qso_check(cba_t cba){
 
     gs=g_string_sized_new(10000);
     gs2=g_string_sized_new(1000);
-    g_string_sprintf(gs,  "   Tucnak QSO check list\n");
+    g_string_sprintf(gs,  "%s", VTEXT(T_QSO_CHECK_LIST));
     g_string_sprintfa(gs, " *************************\n");
     g_string_sprintfa(gs, "\n\n");
-    g_string_sprintfa(gs, "   Call: %s\n", ctest->pcall);
-    g_string_sprintfa(gs, "Contest: %s\n", ctest->tname);
-    g_string_sprintfa(gs, "   Date: %s\n", ctest->tdate);
+    g_string_sprintfa(gs, VTEXT(T_CALL_SN), ctest->pcall);
+    g_string_sprintfa(gs, VTEXT(T_CONTEST_SN), ctest->tname);
+    g_string_sprintfa(gs, VTEXT(T_DATE_SN), ctest->tdate);
     g_string_sprintfa(gs, "\n\n");
 
     for (i=0;i<ctest->bands->len;i++){
         band = (struct band *)g_ptr_array_index(ctest->bands, i);
-        g_string_sprintfa(gs, "   Band %c %s\n", toupper(band->bandchar), band->pband);
+		progress(VTEXT(T_CHECKING_S), band->bandname);
+        g_string_sprintfa(gs, VTEXT(T_BAND_C_SN), band->bandchar, band->pband);
         g_string_sprintfa(gs, " ********************\n");
         for (j=0;j<band->qsos->len;j++){
             qso = (struct qso *)g_ptr_array_index(band->qsos, j);
             ret=qso_info(qso, gs, gs2);
             if (ret) g_string_sprintfa(gs, "\n");
-            if (ret>=0) qso->suspcall1=ret;
+            if (ret>=0) qso->suspcall1=(enum suspcall)ret;
         }
     }
 
     g_string_sprintfa(gs, "\n\nEOF\n");
 
     filename = g_strdup_printf("%s/qsocheck.txt", ctest->directory);
+	z_wokna(filename);
     f=fopen(filename,"wt");
     if (!f) {
         log_addf(VTEXT(T_CANT_WRITE_S), filename);
@@ -830,6 +809,7 @@ void menu_qso_check(cba_t cba){
         g_free(filename);
         g_string_free(gs, TRUE);
         g_string_free(gs2, TRUE);
+		progress(NULL);
         return;
     }
     ret=fwrite(gs->str, 1, gs->len, f);
@@ -840,33 +820,54 @@ void menu_qso_check(cba_t cba){
         g_free(filename);
         g_string_free(gs, TRUE);
         g_string_free(gs2, TRUE);
+		progress(NULL);
         return;
     }
     log_addf(VTEXT(T_SAVED_S), filename);
     g_free(filename);
     g_string_free(gs, TRUE);
     g_string_free(gs2, TRUE);
+	progress(NULL);
     return;
 }
 
-void call_info(cba_t cba){
+
+void call_info_map(struct dialog_data *dlg, struct dialog_item_data *di){
+	GString *gs = g_string_new("http://qthlocator.free.fr/index.php?locator=");
+	g_string_append(gs, di->item->text);
+	z_browser(gs->str);
+	g_string_free(gs, TRUE);
+}
+
+struct memory_list *call_info_ml = NULL;
+void call_info_wwl(gpointer key, gpointer value, gpointer data){
+	add_to_ml(&call_info_ml, g_strdup((char*)key), NULL);
+}
+
+void call_info(void *arg){
     struct qso *qso=NULL;
     GString *gs, *gs2;
     static char ss[1024];
     int err,freeqso,i;
-    char *title, *c;
-    char raw[30];
-    struct qrv_item *qi;
+    char *title, *c, *txt;
+    char raw[30];    
+	struct qrv_item *qi;
+	struct dw_item *dwi;
+	GHashTable *wwls;
+	int wwlcnt;
 
     gs=g_string_sized_new(500);
     gs2=g_string_sized_new(500);
+	wwls = g_hash_table_new(g_str_hash, g_str_equal);
+
+	    
 
 	err=0;
 	freeqso=0;
 /*    dbg("TMPQ.callsign=%s %p\n", TMPQ.callsign, aband);*/
 
-    if (GETCBA(cba,qso) != NULL){
-		qso = (struct qso *)GETCBA(cba, qso);
+    if (arg != NULL){
+		qso = (struct qso *)arg;
         qso_info(qso, gs, gs2);
     }else if (strlen(INPUTLN(aband)->cdata) > 0){
         qso=g_new0(struct qso, 1);
@@ -901,7 +902,7 @@ void call_info(cba_t cba){
         title=VTEXT(T_CALL_INFO);
     }else{
 		ADDQSO;
-        g_string_sprintfa(gs, "Seems to be O.K.\n");
+        g_string_append(gs, VTEXT(T_SEEMS_TO_BE_OK));
         safe_strncpy(ss, gs->str, sizeof(ss));
         title=VTEXT(T_OK);
     }
@@ -910,19 +911,19 @@ void call_info(cba_t cba){
     g_string_append(gs, "\n");
 
     /* C_W */
-    get_raw_call(raw, qso->callsign);
+    z_get_raw_call(raw, qso->callsign);
     g_string_truncate(gs2, 0);
-    cwdb_call_info(gs2, qso->callsign, "");
-    cwdb_call_info(gs2, qso->callsign, "/P");
+    cwdb_call_info(gs2, qso->callsign, "", wwls);
+    cwdb_call_info(gs2, qso->callsign, "/P", wwls);
     for (i=0;i<=9;i++){
         char stroke[10];
         sprintf(stroke, "/%d", i);
-        cwdb_call_info(gs2, qso->callsign, stroke);
+        cwdb_call_info(gs2, qso->callsign, stroke, wwls);
     }
     if (gs2->len>0){
         g_string_append(gs, gs2->str);
     }else{
-        g_string_sprintfa(gs, "C_W: %s not found\n", qso->callsign);
+        g_string_sprintfa(gs, VTEXT(T_CW_NOT_FOUND), qso->callsign);
     }
 
     /* wkd */
@@ -936,7 +937,7 @@ void call_info(cba_t cba){
     }
 
     /* qrv */
-    qi = qrv_get(qrvdb, qso->callsign);
+    qi = qrv_get(qrv, qso->callsign);
     g_string_sprintfa(gs, "QRV: ");
     if (qi){
 //        g_string_sprintfa(gs, " %s", qi->wwl);
@@ -949,23 +950,50 @@ void call_info(cba_t cba){
             g_string_sprintfa(gs, " %2d:%02d", utc.tm_hour, utc.tm_min);
         }
     }else{
-        g_string_append(gs, " 0x (not in database)");
+        g_string_append(gs, VTEXT(T_0X_NOT_IN_DB));
     }
     g_string_append_c(gs, '\n');
 
     /* namedb */
     c=find_name_by_call(namedb, qso->callsign);
     if (c){
-        g_string_sprintfa(gs, "Name: %s's name is %s\n", qso->callsign, c);
+        g_string_sprintfa(gs, VTEXT(T_NAME_IS), qso->callsign, c);
     }else{
-        g_string_sprintfa(gs, "Name: %s's name is unknown\n", qso->callsign);
+        g_string_sprintfa(gs, VTEXT(T_NAME_UNKNOWN), qso->callsign);
     }
 
-    safe_strncpy(ss, gs->str, sizeof(ss));
-    msg_box(NULL, title, AL_LEFT, ss, NULL, 1, VTEXT(T_OK), NULL, B_ENTER | B_ESC);
+	/* dxcc */
+	dwi = get_dw_item_by_call(dw, qso->callsign);
+	if (dwi){
+		char dxcc[32];
+		get_dxcc(dw, dxcc, qso->callsign);
+		g_string_sprintfa(gs, "DXCC: %s %s\n", dxcc, dwi->dxcname);
+	}
+
+	txt = g_strdup(gs->str);
+	call_info_ml = getml(txt, NULL);
+
+	/* locator itself */
+	if (regcmp(qso->callsign,"^[A-R]{2}[0-9]{2}[A-X]{2}$") == 0){
+		char *loc = qso->callsign;
+		if (g_hash_table_lookup(wwls, loc) == NULL) {
+			loc = g_strdup(loc);
+			g_hash_table_insert(wwls, loc, NULL);
+			add_to_ml(&call_info_ml, loc, NULL);
+		}
+	}
+    
+	g_hash_table_foreach(wwls, call_info_wwl, NULL);
+	wwlcnt = g_hash_table_size(wwls);
+	if (wwlcnt > 3) wwlcnt = 3;
+	msg_box(call_info_ml, title, AL_LEFT, txt, NULL, 1 + wwlcnt, VTEXT(T_OK), NULL, B_ENTER | B_ESC, 
+		call_info_ml->p[1 + 0], call_info_map, 0,
+		call_info_ml->p[1 + 1], call_info_map, 0,
+		call_info_ml->p[1 + 2], call_info_map, 0);
 x:;    
     g_string_free(gs, TRUE);
     g_string_free(gs2, TRUE);
+	g_hash_table_destroy(wwls);
     if (freeqso) g_free(qso);
 }
 
@@ -976,7 +1004,7 @@ gchar *find_wwl_by_dxc(struct dw *dw,  gchar *dxc){
     if (!dxc) return dw_next;
     dw_next = NULL;
     
-    dxci = g_hash_table_lookup(dw->dw, dxc);
+    dxci = (struct dw_item *)g_hash_table_lookup(dw->dw, dxc);
     if (!dxci) return NULL;
 
     dw_next = dxci->wwl1;
@@ -995,7 +1023,7 @@ gchar *find_dxc_by_wwl(struct dw *dw, gchar *wwl){
     safe_strncpy(s, wwl, 10);
     if (strlen(s)>4) s[4]='\0';
 
-    wdi = g_hash_table_lookup(dw->wd, s);
+    wdi = (struct wd_item *)g_hash_table_lookup(dw->wd, s);
     if (!wdi) return NULL;
     
     wd_next = wdi->dxc1;
@@ -1004,7 +1032,7 @@ gchar *find_dxc_by_wwl(struct dw *dw, gchar *wwl){
 }
 
 
-char *get_dxcc(struct dw *dw, char *buf, char *call){ 
+char *get_dxcc(struct dw *dw, char *buf, const char *call){ 
     char *c0, *c1, *c2, *dxc;
     char s[100], *stroke = NULL;
     
@@ -1022,7 +1050,7 @@ char *get_dxcc(struct dw *dw, char *buf, char *call){
             safe_strncpy0(s, buf, 20);
             if (stroke) strcat(s, stroke);
         //    dbg("trying1 '%s'\n", s);
-            dxc = g_hash_table_lookup(dw->pd, s);
+            dxc = (gchar *)g_hash_table_lookup(dw->pd, s);
             if (dxc) {
           //      dbg("get_dxcc(%s)='%s'\n", call, s); 
                 safe_strncpy0(buf, dxc, 20);
@@ -1035,7 +1063,7 @@ char *get_dxcc(struct dw *dw, char *buf, char *call){
         while (*buf!='\0'){
             safe_strncpy0(s, buf, 20);
            // dbg("trying2 '%s'\n", s);
-            dxc = g_hash_table_lookup(dw->pd, s);
+            dxc = (gchar *)g_hash_table_lookup(dw->pd, s);
             if (dxc) {
              //   dbg("get_dxcc(%s)='%s'\n", call, s); 
                 safe_strncpy0(buf, dxc, 20);
@@ -1046,10 +1074,14 @@ char *get_dxcc(struct dw *dw, char *buf, char *call){
         safe_strncpy0(buf, call, 20);
     }
 found:;    
-    if (c0) mem_free(c0);
-    if (c1) mem_free(c1);
-    if (c2) mem_free(c2);
+    if (c0) g_free(c0);
+    if (c1) g_free(c1);
+    if (c2) g_free(c2);
+
+    dxc = (gchar *)g_hash_table_lookup(dw->pd, buf);
+    if (dxc) safe_strncpy0(buf, dxc, 20);
 //    dbg("get_dxcc(%s)='%s'\n", call, buf);
+
     return buf;  
 }
 
@@ -1061,8 +1093,8 @@ char *get_pref(char *buf, char *call){
     if (regmatch(call, "^(.*[0-9])[A-Z]*", &c0, &c1, NULL)==0){
         safe_strncpy0(buf, c1, 20);
     }
-    if (c0) mem_free(c0);
-    if (c1) mem_free(c1);
+    if (c0) g_free(c0);
+    if (c1) g_free(c1);
     dbg("get_pref(%s)='%s'\n", call, buf);
     return buf;
 }
