@@ -1,15 +1,35 @@
 /*
     Tucnak - VHF contest log
-    Copyright (C) 2002-2006  Ladislav Vaiz <ok1zia@nagano.cz>
+    Copyright (C) 2002-2015  Ladislav Vaiz <ok1zia@nagano.cz>
 
     This program is free software; you can redistribute it and/or
     modify it under the terms of the GNU General Public License
     version 2 as published by the Free Software Foundation.
 
 */
-
 #include "header.h"
 
+
+#include "bfu.h"
+#include "chart.h"
+#include "control.h"
+#include "cwdb.h"
+#include "edi.h"
+#include "excdb.h"
+#include "fifo.h"
+#include "main.h"
+#include "menu.h"
+#include "map.h"
+#include "namedb.h"
+#include "net.h"
+#include "qrvdb.h"
+#include "qsodb.h"
+#include "session.h"
+#include "sked.h"
+#include "stats.h"
+#include "tregex.h"
+#include "tsdl.h"
+
 static int edi_serid_used;
 
 #define dumpvar(var) { int dvi; dbg("%s=",#var); for (dvi=0; dvi<sizeof(var); dvi++) dbg("%02x ", ((char *)&var)[dvi]); dbg("\n");}
@@ -29,28 +49,28 @@ void add_qso_str1(GString *gs, struct qso *q, struct band *b){
     if (q->error)
         g_string_append(gs, "ERROR");
     else
-        g_string_append(gs, uc(safe_strncpy0(s, q->callsign, 20)));
+        g_string_append(gs, z_str_uc(safe_strncpy0(s, q->callsign, 20)));
     g_string_append_c(gs, ';');
    
     g_string_sprintfa(gs, "%d", q->mode);
     g_string_append_c(gs, ';');
     
-    g_string_append(gs, uc(safe_strncpy0(s, q->rsts, 6)));
+    g_string_append(gs, z_str_uc(safe_strncpy0(s, q->rsts, 6)));
     g_string_append_c(gs, ';');
    
-    g_string_append(gs, uc(safe_strncpy0(s, q->qsonrs, 5)));
+    g_string_append(gs, z_str_uc(safe_strncpy0(s, q->qsonrs, 5)));
     g_string_append_c(gs, ';');
    
-    g_string_append(gs, uc(safe_strncpy0(s, q->rstr, 6)));
+    g_string_append(gs, z_str_uc(safe_strncpy0(s, q->rstr, 6)));
     g_string_append_c(gs, ';');
    
-    g_string_append(gs, uc(safe_strncpy0(s, q->qsonrr, 5)));
+    g_string_append(gs, z_str_uc(safe_strncpy0(s, q->qsonrr, 5)));
     g_string_append_c(gs, ';');
    
-    g_string_append(gs, uc(safe_strncpy0(s, q->exc, 7)));
+    g_string_append(gs, z_str_uc(safe_strncpy0(s, q->exc, 7)));
     g_string_append_c(gs, ';');
     
-    g_string_append(gs, uc(safe_strncpy0(s, q->locator, 7)));
+    g_string_append(gs, z_str_uc(safe_strncpy0(s, q->locator, 7)));
     g_string_append_c(gs, ';');
 }
     
@@ -115,16 +135,29 @@ static int write_edi_header(struct band *b, struct config_band *confb, FILE *f){
         g_string_sprintfa(gs, "TDate=%s;%s\r\n", ctest->cdate, ctest->cdate); 
     else
         g_string_sprintfa(gs, "TDate=%d;%d\r\n", b->stats->first_date, b->stats->last_date); 
-    g_string_sprintfa(gs, "PCall=%s\r\n", uc(safe_strncpy0(s, ctest->pcall, 69)));
-    g_string_sprintfa(gs, "PWWLo=%s\r\n", uc(safe_strncpy0(s, ctest->pwwlo, 69)));
-    g_string_sprintfa(gs, "PExch=%s\r\n", uc(safe_strncpy0(s, ctest->pexch, 69)));
+    g_string_sprintfa(gs, "PCall=%s\r\n", z_str_uc(safe_strncpy0(s, ctest->pcall, 69)));
+    g_string_sprintfa(gs, "PWWLo=%s\r\n", z_str_uc(safe_strncpy0(s, ctest->pwwlo, 69)));
+    g_string_sprintfa(gs, "PExch=%s\r\n", z_str_uc(safe_strncpy0(s, ctest->pexch, 69)));
     g_string_sprintfa(gs, "PAdr1=%s\r\n",    safe_strncpy0(s, ctest->padr1, 69));
     g_string_sprintfa(gs, "PAdr2=%s\r\n",    safe_strncpy0(s, ctest->padr2, 69));
-    g_string_sprintfa(gs, "PSect=%s\r\n", b->psect?"Single":"Multi");
+	switch(b->psect){
+		case 0:
+			g_string_sprintfa(gs, "PSect=Multi\r\n");
+			break;
+		case 1:
+			g_string_sprintfa(gs, "PSect=Single\r\n");
+			break;
+		case 2:
+			g_string_sprintfa(gs, "PSect=Check\r\n");
+			break;
+		default:
+			g_string_sprintfa(gs, "PSect=%s\r\n", b->opsect);
+			break;
+	}
     g_string_sprintfa(gs, "PBand=%s\r\n", b->pband);
-    g_string_sprintfa(gs, "PClub=%s\r\n", uc(safe_strncpy0(s, ctest->pclub, 69)));
+    g_string_sprintfa(gs, "PClub=%s\r\n", z_str_uc(safe_strncpy0(s, ctest->pclub, 69)));
     g_string_sprintfa(gs, "RName=%s\r\n",    safe_strncpy0(s, ctest->rname, 69));
-    g_string_sprintfa(gs, "RCall=%s\r\n", uc(safe_strncpy0(s, ctest->rcall, 69)));
+    g_string_sprintfa(gs, "RCall=%s\r\n", z_str_uc(safe_strncpy0(s, ctest->rcall, 69)));
     g_string_sprintfa(gs, "RAdr1=%s\r\n",    safe_strncpy0(s, ctest->radr1, 69));
     g_string_sprintfa(gs, "RAdr2=%s\r\n",    safe_strncpy0(s, ctest->radr2, 69));
     g_string_sprintfa(gs, "RPoCo=%s\r\n",    safe_strncpy0(s, ctest->rpoco, 69));
@@ -132,8 +165,8 @@ static int write_edi_header(struct band *b, struct config_band *confb, FILE *f){
     g_string_sprintfa(gs, "RCoun=%s\r\n",    safe_strncpy0(s, ctest->rcoun, 69));
     g_string_sprintfa(gs, "RPhon=%s\r\n",    safe_strncpy0(s, ctest->rphon, 69));
     g_string_sprintfa(gs, "RHBBS=%s\r\n",    safe_strncpy0(s, ctest->rhbbs, 69));
-    g_string_sprintfa(gs, "MOpe1=%s\r\n", uc(safe_strncpy0(s, b->mope1, 69)));
-    g_string_sprintfa(gs, "MOpe2=%s\r\n", uc(safe_strncpy0(s, b->mope2, 69)));
+    g_string_sprintfa(gs, "MOpe1=%s\r\n", z_str_uc(safe_strncpy0(s, b->mope1, 69)));
+    g_string_sprintfa(gs, "MOpe2=%s\r\n", z_str_uc(safe_strncpy0(s, b->mope2, 69)));
     g_string_sprintfa(gs, "STXEq=%s\r\n",    safe_strncpy0(s, b->stxeq, 69));
     g_string_sprintfa(gs, "SPowe=%s\r\n",    safe_strncpy0(s, b->spowe, 69));
     g_string_sprintfa(gs, "SRXEq=%s\r\n",    safe_strncpy0(s, b->srxeq, 69));
@@ -141,7 +174,7 @@ static int write_edi_header(struct band *b, struct config_band *confb, FILE *f){
     g_string_sprintfa(gs, "SAntH=%s\r\n",    safe_strncpy0(s, b->santh, 69));
     qsomult = ctest->qsomult;
     if (ctest->qsop_method==16) qsomult = qsomult16(b);
-    g_string_sprintfa(gs, "CQSOs=%d;%d\r\n",    b->stats->nqsos, qsomult);         
+	g_string_sprintfa(gs, "CQSOs=%d;%d\r\n",    b->stats->nqsos, qsomult * b->qsomultb);         
     g_string_sprintfa(gs, "CQSOP=%d\r\n",       b->stats->nqsop);            
     g_string_sprintfa(gs, "CWWLs=%d;%d;%d\r\n", g_hash_table_size(b->stats->wwls),  ctest->wwlbonu, ctest->wwlmult==0 ? 1 : ctest->wwlmult);  
     g_string_sprintfa(gs, "CWWLB=%d\r\n",       g_hash_table_size(b->stats->wwls) * ctest->wwlbonu);
@@ -150,8 +183,8 @@ static int write_edi_header(struct band *b, struct config_band *confb, FILE *f){
     g_string_sprintfa(gs, "CDXCs=%d;%d;%d\r\n", g_hash_table_size(b->stats->dxcs),  ctest->dxcbonu, ctest->dxcmult==0 ? 1 : ctest->dxcmult); 
     g_string_sprintfa(gs, "CDXCB=%d\r\n",       g_hash_table_size(b->stats->dxcs) * ctest->dxcbonu);
     g_string_sprintfa(gs, "CToSc=%d\r\n",       b->stats->ntotal); 
-    g_string_sprintfa(gs, "CODXC=%s;",  uc(safe_strncpy0(s, b->stats->odxcall, 20)));
-    g_string_sprintfa(gs, "%s;%d\r\n",    uc(safe_strncpy0(s, b->stats->odxwwl,  20)), b->stats->odxqrb_int);  
+    g_string_sprintfa(gs, "CODXC=%s;",  z_str_uc(safe_strncpy0(s, b->stats->odxcall, 20)));
+    g_string_sprintfa(gs, "%s;%d\r\n",    z_str_uc(safe_strncpy0(s, b->stats->odxwwl,  20)), b->stats->odxqrb_int);  
     
     ret=fprintf(f, "%s", gs->str) != gs->len;
     g_string_free(gs, TRUE);
@@ -206,6 +239,7 @@ static int write_edi_remarks(struct band *b, struct config_band *confb, FILE *f)
     g_string_sprintfa(gs, "exccfm=%d\r\n",  ctest->exccfm);
     g_string_sprintfa(gs, "qsoused=%d\r\n", ctest->qsoused);
     g_string_sprintfa(gs, "qsomult=%d\r\n", ctest->qsomult);
+    g_string_sprintfa(gs, "qsomultb=%d\r\n", b->qsomultb);
     g_string_sprintfa(gs, "qsoglob=%d\r\n", ctest->qsoglob);
     g_string_sprintfa(gs, "qsop_method=%d\r\n",  ctest->qsop_method);
     g_string_sprintfa(gs, "rstused=%d\r\n", ctest->rstused);
@@ -224,6 +258,7 @@ static int write_edi_remarks(struct band *b, struct config_band *confb, FILE *f)
     g_string_sprintfa(gs, "mode=%d\r\n", gses->mode);
     g_string_sprintfa(gs, "operator=%s\r\n", b->operator_);
     g_string_sprintfa(gs, "psect=%d\r\n", b->psect);
+    g_string_sprintfa(gs, "opsect=%s\r\n", b->opsect);
     g_string_sprintfa(gs, "readonly=%d\r\n", b->readonly);
     g_string_sprintfa(gs, "saveid=%d\r\n", b->saveid);
 	g_string_sprintfa(gs, "skedqrg=%s\r\n", b->skedqrg);
@@ -265,7 +300,7 @@ static int write_edi_qsos(struct band *b, struct config_band *confb, FILE *f){
         /*g_string_sprintfa(gs, "%s\r\n", gs->str);*/
     }
     
-    g_string_sprintfa(gs, "[END;TUCNAK ver. "VERSION_STRING"]\r\n");
+    g_string_sprintfa(gs, "[END;TUCNAK-%s ver. %s]\r\n", Z_PLATFORM, VERSION);
     ret=fprintf(f, "%s", gs->str) != gs->len;
     g_string_free(gs, TRUE);
     
@@ -317,39 +352,47 @@ int export_all_bands_edi(void){
     struct band *band;
     struct config_band *confb;
     int i, err;
-    gchar *filename;
     FILE *f;
     char callbuf[20];
     int ignoreerror=0;
     
     if (!ctest) return -1;
     
+	menu_fillop(GINT_TO_POINTER(0));
+
     /*dbg("export_all_bands_edi()\n");*/
     for (i=0; i<ctest->bands->len; i++){
-        char *section;
+        char *section = NULL;
         
-        band = g_ptr_array_index(ctest->bands, i);
-        confb = get_config_band_by_bandchar(band->bandchar);
+        band = (struct band *)g_ptr_array_index(ctest->bands, i);
+        confb = (struct config_band *)get_config_band_by_bandchar(band->bandchar);
 
+		progress(VTEXT(T_EXPORTING), band->bandname);
+		zg_free0(band->edifile);
         stats_thread_join(band);
         if (band->stats->nqsos <=0) continue;
 
-        section = band->psect ? confb->ok_section_single : confb->ok_section_multi;
+		switch (band->psect){
+			case 0:	section = confb->ok_section_multi; break;
+			case 1:	section = confb->ok_section_single; break;
+			case 2:	section = "CH"; break;
+			case 3:	section = confb->opsect; break;
+		}
         if (section && *section)
-            filename = g_strdup_printf("%s/%s%s.edi",
+            band->edifile = g_strdup_printf("%s/%s%s.edi",
                             ctest->directory,
                             section,
-                            uc(get_raw_call(callbuf,ctest->pcall)));
+                            z_get_raw_call(callbuf,ctest->pcall));
         else
-            filename = g_strdup_printf("%s/%s_%c.edi",
+            band->edifile = g_strdup_printf("%s/%s_%c.edi",
                             ctest->directory,
-                            uc(get_raw_call(callbuf,ctest->pcall)),
-                            toupper(band->bandchar));
-        
-        f=fopen(filename,"wb"); /* must be b for windoze */
+                            z_get_raw_call(callbuf,ctest->pcall),
+                            band->bandchar);
+		z_wokna(band->edifile);
+        f=fopen(band->edifile, "wb"); /* must be b for windoze */
         if (!f) {
             if (!ignoreerror) { errbox(VTEXT(T_CANT_WRITE), errno); ignoreerror=1;}
-            g_free(filename);
+			progress(NULL);
             return -1;
         }
         err = 0;
@@ -359,43 +402,42 @@ int export_all_bands_edi(void){
         err|=fclose(f);
         if (err) {
             if (!ignoreerror) { errbox(VTEXT(T_CANT_WRITE), 0); ignoreerror=1; }
-            g_free(filename);
+			progress(NULL);
             return -1;
         }
-        log_addf("Saved '%s'", filename);
-        g_free(filename);
+        log_addf(VTEXT(T_SAVED), band->edifile);
         
         fprintf(band->swap, "SE %d\n", ++band->saveid);
         fflush(band->swap);
         
     }
+	progress(NULL);
     return 0;
 }
 
 static gchar *get_ctest_directory(int floppy){
     gchar *dir;
+    const char *c;
     
     if (floppy==0){
         dir = g_strdup(ctest->directory);
     }else{
-        char *c1, *c2;
+        //char *c1, *c2;
         
         if (!cfg->as_floppy_path || strlen(cfg->as_floppy_path)==0) return NULL;
         if (!cfg->as_floppy_aq) return NULL;
         
-        c1=c2=NULL;
-        if (regmatch(ctest->directory, ".*\\/(.*)", &c1, &c2, NULL)==0){
-            dir = g_strdup_printf("%s/%s",
-                  cfg->as_floppy_path,
-                  c2);
+        //c1=c2=NULL;
+		c = z_filename(ctest->directory);
+		if (c != NULL){
+        //if (regmatch(ctest->directory, ".*\\/(.*)", &c1, &c2, NULL)==0){
+            dir = g_strdup_printf("%s/%s", cfg->as_floppy_path, c);
         }else{
-            dir = g_strdup_printf("%s/%s",
-                  cfg->as_floppy_path,
-                  ctest->directory);
+            dir = g_strdup_printf("%s/%s", cfg->as_floppy_path, ctest->directory);
         }
         
-        if (c1) mem_free(c1);
-        if (c2) mem_free(c2);
+        //if (c1) g_free(c1);
+        //if (c2) g_free(c2);
     }
     return dir;
 }
@@ -403,24 +445,36 @@ static gchar *get_ctest_directory(int floppy){
 int save_all_bands_txt(int autosave){
     struct band *band;
     struct config_band *confb;
-    int j, i, err;
+    int j, i, err, gerr;
     gchar *filename, *dir;
     FILE *f;
     int ignoreerror=0;
     int floppy_mounted,nm_showed;
     
-    dbg("save_all_bands_txt(%d)\n", autosave);
+    //dbg("save_all_bands_txt(%d)\n", autosave);
     
     if (!ctest) return -1;
     
-    floppy_mounted = is_in_mtab(cfg->as_floppy_path);
     nm_showed=0;
+	floppy_mounted = is_in_mtab(cfg->as_floppy_path);
+	if (cfg->as_floppy_path && !*cfg->as_floppy_path){
+		nm_showed = 1;	 // dont show error when no floppy is defined
+	}
+
+
+    
+	gerr = 0;
     for (i=0; i<ctest->bands->len; i++){
         for (j=0; j<2; j++){
-            band = g_ptr_array_index(ctest->bands, i);
-            confb = get_config_band_by_bandchar(band->bandchar);
+            band = (struct band *)g_ptr_array_index(ctest->bands, i);
+            confb = (struct config_band *)get_config_band_by_bandchar(band->bandchar);
 /*            dbg("  %c dirty=%d\n", band->bandchar, band->dirty_save); */
-            if (autosave && !band->dirty_save) continue;
+
+            dir = get_ctest_directory(j);
+            if (!dir) continue;
+            z_mkdir_p(dir, 0775);
+
+            if (autosave == 1 && !band->dirty_save) continue;
             
             if (j==0){
                 if (autosave && autosave!=1) continue;
@@ -435,14 +489,12 @@ int save_all_bands_txt(int autosave){
             }
             
             stats_thread_join(band);
-            dir=get_ctest_directory(j);
-            if (!dir) continue;
-            mkdir_p(dir, 0775);
             
-            filename = g_strdup_printf("%s/%c.txt", dir, band->bandchar);
+            filename = g_strdup_printf("%s/%c.txt", dir, z_char_lc(band->bandchar));
+			z_wokna(filename);
             g_free(dir);
             
-            dbg("  saving filename '%s'\n", filename);
+            //dbg("  saving filename '%s'\n", filename);
             f=fopen(filename,"wb"); /* must be b for windoze */
             if (!f) {
                 gchar *c;
@@ -473,15 +525,18 @@ int save_all_bands_txt(int autosave){
                 g_free(filename);
                 continue;
             }
+			gerr |= err;
             band->dirty_save = 0;
             fprintf(band->swap, "ST %d\n", band->saveid);
             fflush(band->swap);
-            if (!autosave) log_addf(VTEXT(T_SAVED_S), filename);
             g_free(filename);
         }
         band->saveid++;
     }
 
+	if (!gerr){
+		if (!autosave) log_addf(VTEXT(T_SAVED_S), "contest");
+	}
     if (!autosave){
         ctest->as_disk_qsonr = cfg->as_disk_aq;
         ctest->as_floppy_qsonr = cfg->as_floppy_aq;
@@ -501,50 +556,55 @@ int save_all_bands_txt(int autosave){
             
         dir = get_ctest_directory(j);
         if (!dir) continue;
-        
+      
         
         if (j==1){ /* floppy */
 		  	if (nm_showed) continue;
             filename = g_strdup_printf("%s/desc",dir);
+			z_wokna(filename);
             save_desc_to_file(filename);
             g_free(filename);
         }
         
-        if (i==1){ /* save only contents of fifo to ctest/log on floppy */
+        if (j==1){ /* save only contents of fifo to ctest/log on floppy */
             filename = g_strdup_printf("%s/log",dir);
+			z_wokna(filename);
             save_fifo_to_file(glog, filename);
             g_free(filename);
         }
     
         filename = g_strdup_printf("%s/talk",dir);
+		z_wokna(filename);
         save_fifo_to_file(gtalk, filename);
         g_free(filename);
     
-/*        filename = g_strdup_printf("%s/sked",dir);
-        save_fifo_to_file(gsked, filename);
-        g_free(filename);*/
-
         filename = g_strdup_printf("%s/spy", dir);  
+		z_wokna(filename);
         save_spypeers_to_file(filename); /* called also in control.c */
         g_free(filename);
         
         filename = g_strdup_printf("%s/hicalls", dir);  
+		z_wokna(filename);
         save_hicalls_to_file(filename);
         g_free(filename);
 
         filename = g_strdup_printf("%s/qrv", dir);  
+		z_wokna(filename);
         save_qrv_to_file(filename);
         g_free(filename);
 
         filename = g_strdup_printf("%s/chart", dir);  
+		z_wokna(filename);
         save_chart_to_file(filename);
         g_free(filename);
 
-        
-
         g_free(dir);
     }
     
+	save_cw(cw, 0);
+	save_namedb(namedb, 0);
+
+
     return 0;
 }
 
@@ -560,7 +620,7 @@ static GHashTable *load_edi_until(FILE *f, GHashTable *hash, gchar *until){
     
     if (!hash) hash=g_hash_table_new(g_str_hash, g_str_equal);
  
-    while( (c = safe_fgets(gs, f, 0)) != NULL){
+    while( (c = zfile_fgets(gs, f, 0)) != NULL){
 /*        dbg("load_edi_until   read '%s'\n", c);*/
         if (regcmpi(c, until)==0) break; 
         
@@ -574,7 +634,7 @@ static GHashTable *load_edi_until(FILE *f, GHashTable *hash, gchar *until){
         g_strstrip(a[0]);
         g_strstrip(a[1]); 
 
-        k=lc(g_strdup(a[0]));
+        k=z_str_lc(g_strdup(a[0]));
         v=g_strdup(a[1]);
 
         if (g_hash_table_lookup_extended(hash, (gpointer)k, &orig_key, &orig_value)){
@@ -601,7 +661,7 @@ static gchar *load_edi_remarks_until(FILE *f, gchar *until){
     gs=g_string_new("");
     
     ret=NULL;
-    while( (c = safe_fgets(gs, f, 0)) != NULL){
+    while( (c = zfile_fgets(gs, f, 0)) != NULL){
 /*        dbg("load_edi_remarks_until   read '%s'\n", c);*/
         if (regcmpi(c, until)==0) break; 
         if (ret) {
@@ -628,7 +688,7 @@ static GPtrArray *load_edi_qsos_until(FILE *f, gchar *until){
     
     array=g_ptr_array_new();
  
-    while( (c = safe_fgets(gs, f, 0)) != NULL){
+    while( (c = zfile_fgets(gs, f, 0)) != NULL){
 /*        dbg("load_edi_qsos_unti: '%s'\n", c);*/
         if (regcmpi(c, until)==0) break; 
         
@@ -657,7 +717,7 @@ static void skip_edi_until(FILE *f, gchar *until){
     
     gs=g_string_new("");
     
-    while( (c = safe_fgets(gs, f, 0)) != NULL){
+    while( (c = zfile_fgets(gs, f, 0)) != NULL){
  /*       dbg("skip_edi_until: read '%s'\n", c);*/
         if (regcmpi(c, until)==0) break; 
     }
@@ -678,7 +738,7 @@ static void destroy_string_hash(GHashTable *hash){
 }
 
 
-static void add_qsos_from_mem(struct band *b, GHashTable *hash, GPtrArray *qsos, gchar *date){
+static void add_qsos_from_mem(struct band *b, GHashTable *hash, GPtrArray *qsos, const gchar *date){
     gchar **items;
     int i, j;
     gchar *c, *add;
@@ -712,14 +772,14 @@ static void add_qsos_from_mem(struct band *b, GHashTable *hash, GPtrArray *qsos,
         g_strfreev(items);
 
         c = g_strdup_printf("qso%d", i);
-        add = g_hash_table_lookup(hash, c);
+        add = (char *)g_hash_table_lookup(hash, c);
         g_free(c);
         if (!add){
             char century[3];
             safe_strncpy0(century, q->date_str, 3);
             add=g_strdup_printf("%s;%s;%d;%s;"
                                 "%s;%s;%d;",    
-                    net->myid, fixsemi(b->operator_), (int)now, "",
+                    gnet->myid, fixsemi(b->operator_), (int)now, "",
                     century, "", -1);
             items = g_strsplit(add, ";", 0);
             g_free(add);
@@ -794,7 +854,7 @@ static void add_unfi_from_mem(struct band *b, GHashTable *hash){
     b->unfi->withouttime=1;
     for (i=0;;i++){
         c = g_strdup_printf("unfinished%d", i);
-        add = g_hash_table_lookup(hash, c);
+        add = (char *)g_hash_table_lookup(hash, c);
         g_free(c);
         if (!add) break;
 /*        dbg("unfinished=''\n", add);*/
@@ -812,7 +872,7 @@ static void add_sked_from_mem(struct band *b, GHashTable *hash){
     
     for (i=0;;i++){
         c = g_strdup_printf("sked%d", i);
-        add = g_hash_table_lookup(hash, c);
+        add = (char *)g_hash_table_lookup(hash, c);
         g_free(c);
         if (!add) break;
 //        dbg("sked=''\n", add);
@@ -829,26 +889,26 @@ static void add_sked_from_mem(struct band *b, GHashTable *hash){
 }
 
 
-static int select_enable_all(const struct dirent *de){
+static int select_enable_all(const char *dir, const struct dirent *de){
     return 1;
 }
 
 
 #define REPLACE(from,to) \
-    c = g_hash_table_lookup(hash, from);\
+    c = (char *)g_hash_table_lookup(hash, from);\
     if (c && !g_hash_table_lookup(hash, to)){\
         g_hash_table_insert(hash, g_strdup(to), g_strdup(c));\
         dbg("replacing %s to %s with %s\n", from, to, c); \
     }
 
 
-static int convert_bo_s_hash(GHashTable *hash, char *dir){
+static int convert_bo_s_hash(GHashTable *hash, const char *dir){
     struct config_band *tmpcb;
     gchar *c, *date;
 
-    c = g_hash_table_lookup(hash, "band");
+    c = (char *)g_hash_table_lookup(hash, "band");
     if (c){
-        tmpcb = get_config_band_by_bandchar(lowcase(c[0]));
+        tmpcb = get_config_band_by_bandchar(z_char_lc(c[0]));
         if (tmpcb){
             if (tmpcb->pband && !g_hash_table_lookup(hash, "pband"))
                 g_hash_table_insert(hash, g_strdup("pband"), g_strdup(tmpcb->pband));
@@ -882,14 +942,14 @@ static int convert_bo_s_hash(GHashTable *hash, char *dir){
 
 //int *pnn = NULL;
 /* loads edi and txt */
-void load_contest_edi(gchar *date, int edi){
+void load_contest_edi(const gchar *date, int edi){
     struct dirent **namelist=NULL;
     int nn=0; // probably buggy register optimisation in gcc 4.3.2 amd64
     int i;
     gchar *datedir, *file, *pband;
     struct stat st;
     char *ext, *c;
-    FILE *f, *descfile,*logfile;
+    FILE *f, *lockfile,*logfile;
     GHashTable *hash;
     GPtrArray *qsos;
     struct band *b;
@@ -903,17 +963,20 @@ void load_contest_edi(gchar *date, int edi){
     file = NULL;
     datedir = NULL;
     
-    c=g_strdup_printf("%s/tucnak/%s/desc", getenv("HOME"), date);
-    descfile = fopen(c, "rt+");
-    if (!descfile){
+	progress(VTEXT(T_LOADING_CONTEST), date);
+
+    c=g_strdup_printf("%s/%s/desc.lock", tucnak_dir, date);
+	z_wokna(c);
+    lockfile = fopen(c, "w");
+    if (!lockfile){
         g_snprintf(ss,1000, VTEXT(T_CANT_OPEN_S),c);
         g_free(c);
         msg_box(NULL, VTEXT(T_ERROR), AL_CENTER, ss, NULL, 1, VTEXT(T_OK), NULL, B_ENTER | B_ESC);
         goto x;
     }
     
-    if (lockf(fileno(descfile), F_TLOCK, 0)){ 
-        fclose(descfile); descfile=NULL;
+    if (z_lockf(fileno(lockfile), F_TLOCK, 0, 1)){ 	 
+        fclose(lockfile); lockfile=NULL;
         g_snprintf(ss,1000, VTEXT(T_CANT_LOCK_S),c);
         g_free(c);
         msg_box(NULL, VTEXT(T_ERROR), AL_CENTER, ss, NULL, 1, VTEXT(T_OK), NULL, B_ENTER | B_ESC);
@@ -923,15 +986,15 @@ void load_contest_edi(gchar *date, int edi){
                 
     
     
-    datedir = g_strdup_printf("%s/tucnak/%s", 
-            getenv("HOME"), date);
+    datedir = g_strdup_printf("%s/%s", tucnak_dir, date);
    
     file = g_strdup_printf("%s/log", datedir);
     load_fifo_from_file(glog, file,0);
     fifo_addfq(glog, VTEXT(T_LOADING_CTEST_S), date); 
-    CONDGFREE(file);
+    zg_free0(file);
 
-    c=g_strdup_printf("%s/tucnak/%s/log", getenv("HOME"), date);
+	progress(VTEXT(T_LOADING_LOGFILE));
+    c=g_strdup_printf("%s/%s/log", tucnak_dir, date);
     logfile = fopen(c, "at");
     if (!logfile){
         g_snprintf(ss,1000, VTEXT(T_CANT_OPEN_S),c);
@@ -944,17 +1007,17 @@ void load_contest_edi(gchar *date, int edi){
     fprintf(logfile, VTEXT(T_LOADING_CTEST_S), date); fprintf(logfile, "\n");
     
     
+	progress(VTEXT(T_LOADING_TALK));
     file = g_strdup_printf("%s/talk", datedir);
     load_fifo_from_file(gtalk, file,1);
     fifo_addfq(gtalk, VTEXT(T_LOADING_CTEST_S), date);
-    CONDGFREE(file);
+    zg_free0(file);
 
 
-    nn = tucnak_scandir(datedir, &namelist, select_enable_all, tucnak_alphasort);
-    //printf("%lx\n", &nn);
+    nn = z_scandir(datedir, &namelist, select_enable_all, z_scandir_alphasort);
     if (nn<=0) {
         errbox(VTEXT(T_NO_FILES), 0);
-        fclose(descfile); 
+        fclose(lockfile); 
         fclose(logfile); 
         goto x;
     }
@@ -963,11 +1026,12 @@ void load_contest_edi(gchar *date, int edi){
     for (i=0; i<nn; i++){
         GString *gs;
 
+		progress(VTEXT(T_SEARCH_FILES));
         ext = edi ? "[a-z]\\.edi$" : "[a-z]\\.txt$";
     
         if (regcmp(namelist[i]->d_name, ext)) continue;
 
-        CONDGFREE(file);
+        zg_free0(file);
         file = g_strconcat(datedir, "/", namelist[i]->d_name, NULL); 
         /*dbg("file='%s'\n", file);*/
 
@@ -991,19 +1055,19 @@ void load_contest_edi(gchar *date, int edi){
         
         convert_bo_s_hash(hash, date);
 
-        pband = g_hash_table_lookup(hash, "pband"); 
+        pband = (char *)g_hash_table_lookup(hash, "pband"); 
         /*dbg("pband='%s'\n", pband);*/
         if (!pband) {
             struct config_band *tmpcb;
             gchar *c;
-            c = g_hash_table_lookup(hash, "band");
+            c = (char *)g_hash_table_lookup(hash, "band");
             if (!c) goto destroy;
-            tmpcb = get_config_band_by_bandchar(lowcase(c[0]));
+            tmpcb = get_config_band_by_bandchar(z_char_lc(c[0]));
             if (!tmpcb) goto destroy;
             pband = tmpcb->pband;
             if (!pband){
 destroy:;         
-                CONDGFREE(remarks);       
+                zg_free0(remarks);       
                 destroy_string_hash(hash);
                 continue;
             }
@@ -1021,12 +1085,13 @@ destroy:;
             chart_reload();
         }
         
-        b = init_band(NULL, hash, NULL);
+        b = init_band(NULL, hash, NULL, NULL);
         if (!b) {
           /*  dbg("load_contest_edi: b=NULL!\n");*/
             goto destroy;
             
         }
+		progress(VTEXT(T_LOADING_BAND), b->bandname);
         /* edi_serid_used=0; must be assigned but before band loop */
         add_qsos_from_mem(b, hash, qsos, date);
         add_unfi_from_mem(b, hash);
@@ -1034,11 +1099,11 @@ destroy:;
         clear_tmpqsos(b, 1);
         destroy_string_hash(hash);  
         if (remarks){
-            CONDGFREE(b->remarks);
+            zg_free0(b->remarks);
             b->remarks=remarks;
         }      
 
-        CONDGFREE(file);
+        zg_free0(file);
         
         file = g_strconcat(datedir, "/", namelist[i]->d_name, NULL); 
         c = strstr(file,"txt");
@@ -1049,7 +1114,7 @@ destroy:;
         f = fopen(file, "rt");
         if (!f) continue;
         gs = g_string_sized_new(100);
-        while(safe_fgets(gs, f, 0)!=NULL){
+        while(zfile_fgets(gs, f, 0)!=NULL){
           /*      dbg("read '%s'\n", gs->str);*/
             if (strncmp(gs->str,"ST ",3)!=0) continue;
             saveid=atoi(gs->str+3);
@@ -1060,8 +1125,8 @@ destroy:;
         if (saveid == b->saveid-1){
             b->ignoreswap = 1;
    /*         dbg("begin is %d\n", saveid);*/
-            while(safe_fgets(gs, f, 0)!=NULL){
-/*                dbg("read '%s'\n", gs->str);*/
+            while(zfile_fgets(gs, f, 0)!=NULL){
+//                dbg("read '%s'\n", gs->str);
                         
                 if (strncmp(gs->str,"# ",2)==0){
                     process_input(b, gs->str+2, 0);
@@ -1091,7 +1156,7 @@ destroy:;
     
     if (!ctest) {
         errbox(VTEXT(T_NO_BANDS), 0);
-        fclose(descfile); 
+        fclose(lockfile); 
         fclose(logfile); 
         goto x;
     }
@@ -1102,27 +1167,37 @@ destroy:;
     }
     
     
-    CONDGFREE(file);
+    zg_free0(file);
     
+	progress(VTEXT(T_LOADING_SPY));
     file = g_strdup_printf("%s/spy", datedir);
+	z_wokna(file);
     load_spypeers_from_file(file);
-    CONDGFREE(file);
+    zg_free0(file);
 
+	progress(VTEXT(T_LOADING_HICALLS));
     file = g_strdup_printf("%s/hicalls", datedir);
+	z_wokna(file);
     load_hicalls_from_file(file);
-    CONDGFREE(file);
+    zg_free0(file);
 
+	progress(VTEXT(T_LOADING_QRV));
     file = g_strdup_printf("%s/qrv", datedir);
-    load_qrv_from_file(qrvdb, file);
-    CONDGFREE(file);
+	z_wokna(file);
+    load_qrv_from_file(qrv, file);
+    zg_free0(file);
     
+	progress(VTEXT(T_LOADING_CHART));
     file = g_strdup_printf("%s/chart", datedir);
+	z_wokna(file);
     load_chart_from_file(file);
-    CONDGFREE(file);
+    zg_free0(file);
     
+	progress(VTEXT(T_SAVING_CONTEST));
     check_autosave();
 
-    ctest->descfile = descfile;
+	progress(VTEXT(T_UPD_INT_DATA));
+    ctest->lockfile = lockfile;
     ctest->logfile = logfile;
     ctest->cdate = g_strdup(date);
     c = strchr(ctest->cdate, '.');
@@ -1145,13 +1220,14 @@ destroy:;
     }
 
     g_hash_table_foreach(ctest->bystamp, foreach_source_qsort_by_stamp, NULL);
-    g_index_array_qsort(ctest->allqsos, compare_date_time_qsonrs);
+    z_ptr_array_qsort(ctest->allqsos, compare_date_time_qsonrs);
 
     
     set_ctest_title();
     clear_exc(excdb);
     read_exc_files(excdb, ctest->exctype, ctest->excname);
     
+	progress(VTEXT(T_CALC_STATS));
     recalc_allb_stats();
     for (i=0; i<ctest->bands->len; i++){
         struct band *b = (struct band *)g_ptr_array_index(ctest->bands, i);
@@ -1159,34 +1235,47 @@ destroy:;
         recalc_statsfifo(b);
     }
     
-    dump_all_sources(ctest);
+    //dump_all_sources(ctest);
+	progress(VTEXT(T_ANN_OVER_NET));
     net_send_ac();
-        
+	sw_shake_tmo = ztimeout_init(100000);        
+
     if (cfg->startband){
         struct band *b;
         b=find_band_by_pband(cfg->startband);
-    //  pnn = &nn;
-        activate_band(b); /* b==NULL is handled by function */
+        if (b){
+            progress(VTEXT(T_ACTIVATING_BAND), b->bandname);
+            activate_band(b); /* b==NULL is handled by function */
+        }
     }
-/*#ifdef HAVE_SDL		
+
+/*#ifdef Z_HAVE_SDL		
     maps_reload(); dela se pri activate_band 
 #endif*/
-    qrv_recalc_wkd(qrvdb);
-    qrv_recalc_qrbqtf(qrvdb);
-    qrv_recalc_gst(qrvdb);
+	progress(VTEXT(T_UPDATING_WKD));
+    qrv_recalc_wkd(qrv);
+	progress(VTEXT(T_UPDATING_QRBQTF));
+	qrv_recalc_qrbqtf(qrv);
+	progress(VTEXT(T_UPDATING_GAIN));
+    qrv_recalc_gst(qrv);
+	progress(VTEXT(T_SEN_OVER_NET));
     net_send_operator();
     net_send_read_write_bands();
+	net_send_talk();
+	spypeer_send_requests();
 
 x: 
-    dbg("nnx=%d\n", nn);
+	sw_shake_tmo = -1;
+    //dbg("nnx=%d\n", nn);
 //     dumpvar(nn);
 //     dumpvar(i);
     if (namelist){
         for (i=0; i<nn; i++) free(namelist[i]);
         free(namelist);
     }
-    CONDGFREE(datedir);   
-    CONDGFREE(file);   
+    zg_free0(datedir);   
+    zg_free0(file);   
+	progress(NULL);
 }
 
 void import_edi(void *xxx, char *filename){
@@ -1206,7 +1295,7 @@ void import_edi(void *xxx, char *filename){
     qsos = load_edi_qsos_until(f, "^\\[end;");
     fclose(f);
 
-    pband = g_hash_table_lookup(hash, "pband"); 
+    pband = (char *)g_hash_table_lookup(hash, "pband"); 
     /*dbg("import_edi: pband='%s'\n", pband);*/
     
     band=find_band_by_pband(pband);
@@ -1222,5 +1311,33 @@ destroy:;
     redraw_later();
 }
 
+
 void import_swap(void *xxx, char *filename){
+	FILE *f;
+	f = fopen(filename, "rt");
+	if (!f){
+		char *c = g_strdup_printf(VTEXT(T_CANT_READ_S), filename);
+        msg_box(getml(c, NULL), VTEXT(T_ERROR), AL_CENTER, c, NULL, 1, VTEXT(T_OK), NULL, B_ENTER | B_ESC);
+	}
+	add_qsos_from_swap(aband, f);
+	fclose(f);
+}
+
+int save_desc_to_file(gchar *filename){
+    gchar *pcall, *s;
+    FILE *f;
+    int ret;
+
+    z_wokna(filename);
+	f = fopen(filename, "wt");
+    if (!f) return 1;
+    
+    pcall=g_strdup(ctest->pcall);
+    s = g_strdup_printf("%s %s %s\n", ctest->cdate, z_str_uc(pcall), ctest->tname);    
+    ret = fprintf(f, "%s", s) != strlen(s);
+    g_free(pcall);
+    g_free(s);
+    fclose(f);
+    return ret;
+    
 }
