@@ -1,6 +1,6 @@
 /*
     Tucnak - VHF contest log
-    Copyright (C) 2002-2008  Ladislav Vaiz <ok1zia@nagano.cz>
+    Copyright (C) 2002-2015  Ladislav Vaiz <ok1zia@nagano.cz>
 
     This program is free software; you can redistribute it and/or
     modify it under the terms of the GNU General Public License
@@ -10,7 +10,16 @@
 
 #include "header.h"
 
-#include <string.h>
+#include "excdb.h"
+#include "dwdb.h"
+#include "language2.h"
+#include "main.h"
+#include "stats.h"
+#include "tsdl.h"
+
+
+//#include <string.h>
+
 struct excdb *excdb;
 static gchar *exc_next;
 static gchar *cxe_next;
@@ -18,12 +27,14 @@ static gchar *cxe_next;
 struct excdb *init_exc(void){
     struct excdb *excdb;
 
+	progress(VTEXT(T_INIT_EXCDB));		
+
     excdb = g_new0(struct excdb, 1);
     
     excdb->exc = g_hash_table_new(g_str_hash, g_str_equal);
     excdb->cxe = g_hash_table_new(g_str_hash, g_str_equal);
     excdb->vexc = g_hash_table_new(g_str_hash, g_str_equal);
-    excdb->vexcia = g_index_array_new();
+    excdb->vexcia = z_ptr_array_new();
     
     exc_next = NULL;
     cxe_next = NULL;
@@ -63,18 +74,20 @@ gboolean free_vexc_item(gpointer key, gpointer value, gpointer user_data){
 
 void clear_exc(struct excdb *excdb){
 //    dbg("clear_exc()\n");
-    g_index_array_free_all(excdb->vexcia);
-    excdb->vexcia = g_index_array_new();
+    z_ptr_array_free_all(excdb->vexcia);
+    excdb->vexcia = z_ptr_array_new();
     g_hash_table_foreach_remove(excdb->vexc, free_vexc_item, NULL);
     g_hash_table_foreach_remove(excdb->cxe, free_cxe_item, NULL);
     g_hash_table_foreach_remove(excdb->exc, free_exc_item, NULL);
 
-    CONDGFREE(excdb->excname);
+    zg_free0(excdb->excname);
 }
 
 void free_exc(struct excdb *excdb){
+	progress(VTEXT(T_FREE_EXCDB));		
+
     clear_exc(excdb);
-    g_index_array_free_all(excdb->vexcia);
+    z_ptr_array_free_all(excdb->vexcia);
     g_hash_table_destroy(excdb->vexc);
     g_hash_table_destroy(excdb->cxe);
     g_hash_table_destroy(excdb->exc);
@@ -101,7 +114,7 @@ void load_one_exc(struct excdb *excdb, gchar *s){
     gchar *call, *exc, *stamp_str;
     char *token_ptr;
     
-    uc(s);
+    z_str_uc(s);
     call = strtok_r(s, EXC_DELIM, &token_ptr);
     if (!call) return;
 
@@ -125,7 +138,7 @@ int load_exc_from_file(struct excdb *excdb, gchar *filename){
     gs = g_string_sized_new(256);
 
     while(1){
-        if (!safe_fgets(gs, f, 1)) break;
+        if (!zfile_fgets(gs, f, 1)) break;
         g_strstrip(gs->str);
 
         load_one_exc(excdb,gs->str);
@@ -139,14 +152,14 @@ void load_one_vexc(struct excdb *excdb, gchar *s){
     gchar *vexc;
     char *token_ptr;
     
-    uc(s);
+    z_str_uc(s);
     for (vexc = strtok_r(s, EXC_DELIM, &token_ptr);
          vexc != NULL;
          vexc = strtok_r(NULL, EXC_DELIM, &token_ptr)){
          
          if (!*vexc) continue;
          if (g_hash_table_lookup(excdb->vexc, vexc) != NULL) continue;
-         uc(vexc);
+         z_str_uc(vexc);
          g_hash_table_insert(excdb->vexc, g_strdup(vexc), (void*)1);
     }
 }
@@ -161,7 +174,7 @@ int load_vexc_from_file(struct excdb *excdb, gchar *filename){
     gs = g_string_sized_new(256);
 
     while(1){
-        if (!safe_fgets(gs, f, 1)) break;
+        if (!zfile_fgets(gs, f, 1)) break;
         g_strstrip(gs->str);
 
         load_one_vexc(excdb, gs->str);
@@ -179,7 +192,7 @@ int load_vexc_from_mem(struct excdb *excdb, const char *file, const long int len
     pos = 0;
     
     while(1){
-        if (!safe_mgets(gs,  file, &pos, len, 0)) break;
+        if (!zfile_mgets(gs,  file, &pos, len, 0)) break;
         g_strstrip(gs->str);
 
         load_one_vexc(excdb, gs->str);
@@ -193,22 +206,22 @@ void read_exc_files(struct excdb *excdb, enum exctype exctype, gchar *excname){
     gchar *s, *ename;
     int ret, i;
 
-    dbg("read_exc_files(%d, %s)\n", exctype, excname);
+    //dbg("read_exc_files(%d, %s)\n", exctype, excname);
     if (!excname || !*excname) return;
 
     excdb->exctype = exctype;
     ename = g_strdup(excname);
-    uc(ename);
-    CONDGFREE(excdb->excname);
+    z_str_uc(ename);
+    zg_free0(excdb->excname);
     excdb->excname = g_strdup(ename);
-    lc(ename);
+    z_str_lc(ename);
     
-    s = g_strconcat(getenv("HOME"), "/tucnak/tucnakexc", ename, NULL); 
+    s = g_strconcat(tucnak_dir, "/tucnakexc", ename, NULL); 
     ret = load_exc_from_file(excdb, s);
     dbg("    %s %d\n", s, ret);
     g_free(s);
     
-    s = g_strconcat(getenv("HOME"), "/tucnak/tucnakvexc", ename, NULL); 
+    s = g_strconcat(tucnak_dir, "/tucnakvexc", ename, NULL); 
     ret = load_vexc_from_file(excdb, s);
     dbg("    %s %d\n", s, ret);
     g_free(s);
@@ -240,23 +253,23 @@ void read_exc_files(struct excdb *excdb, enum exctype exctype, gchar *excname){
             break;
         case EXC_WAZ:  
 //            g_hash_table_foreach(dw->dw, exc_ia_waz, (gpointer) excdb->vexcia);
-            for (i=1; i<=40; i++) g_index_array_add(excdb->vexcia, g_strdup_printf("%02d", i));
+            for (i=1; i<=40; i++) z_ptr_array_add(excdb->vexcia, g_strdup_printf("%02d", i));
             break;
         case EXC_ITU:
 //            g_hash_table_foreach(dw->dw, exc_ia_itu, (gpointer) excdb->vexcia);
-            for (i=1; i<=90; i++) g_index_array_add(excdb->vexcia, g_strdup_printf("%02d", i));
+            for (i=1; i<=90; i++) z_ptr_array_add(excdb->vexcia, g_strdup_printf("%02d", i));
             break;
         default:
             break;
     }
-    g_index_array_qsort(excdb->vexcia, compare_string);
-    g_index_array_uniq(excdb->vexcia, compare_string, TRUE);
+    z_ptr_array_qsort(excdb->vexcia, z_compare_string);
+    z_ptr_array_uniq(excdb->vexcia, z_compare_string, TRUE);
 
 }
 
 void exc_ia_verified(gpointer key, gpointer value, gpointer user_data){
     gchar *exc          = (gchar *) key;
-    GIndexArray *vexcia = (GIndexArray *) user_data;
+    ZPtrArray *vexcia = (ZPtrArray *) user_data;
     char s[10];
 
     if (strcasecmp(ctest->excname, "USACA")==0){
@@ -264,7 +277,7 @@ void exc_ia_verified(gpointer key, gpointer value, gpointer user_data){
         usaca_multiple_exc(s);
         if (strcmp(s, exc) != 0) return; // QC, MAN, ALB, ALT, YU
     }
-    g_index_array_add(vexcia, g_strdup(exc));
+    z_ptr_array_add(vexcia, g_strdup(exc));
 }
 
 
@@ -272,14 +285,14 @@ void exc_ia_verified(gpointer key, gpointer value, gpointer user_data){
     struct dw_item *dwi = (struct dw_item*) value;
     GIndexArray *vexcia = (GIndexArray *) user_data;
 
-    g_index_array_add(vexcia, g_strdup_printf("%02d", dwi->waz));
+    z_ptr_array_add(vexcia, g_strdup_printf("%02d", dwi->waz));
 }
 
 void exc_ia_itu(gpointer key, gpointer value, gpointer user_data){
     struct dw_item *dwi = (struct dw_item*) value;
     GIndexArray *vexcia = (GIndexArray *) user_data;
 
-    g_index_array_add(vexcia, g_strdup_printf("%02d", dwi->itu));
+    z_ptr_array_add(vexcia, g_strdup_printf("%02d", dwi->itu));
 } */
 
 
@@ -400,7 +413,7 @@ void add_cxe(struct excdb *excdb, gchar *exc, gchar *call, gint stamp){
 }
 
 
-gchar *find_exc_by_call(struct excdb *excdb, gchar *call){
+gchar *find_exc_by_call(struct excdb *excdb, const gchar *call){
     struct exc_item *exci;
     struct dw_item *dwi;
     static char s[20];
@@ -469,7 +482,7 @@ int is_valid_vexc(struct excdb *excdb, gchar *vexc){
             
         case EXC_VERIFIED:
             s = g_strdup(vexc);
-            uc(s);
+            z_str_uc(s);
             ret = g_hash_table_lookup(excdb->vexc, s) != NULL;
             g_free(s);
             return ret;
