@@ -11,9 +11,21 @@
 
 #include "header.h"
 
-#define FREE_Cx if (c1) {mem_free(c1); c1=NULL;}\
-                if (c2) {mem_free(c2); c2=NULL;}\
-                if (c3) {mem_free(c3); c3=NULL;}
+#include "cwdaemon.h"
+#include "inputln.h"
+#include "kbdbind.h"
+#include "kbd.h"
+#include "qsodb.h"
+#include "rc.h"
+#include "session.h"
+#include "subwin.h"
+#include "terminal.h"
+#include "tregex.h"
+#include "zosk.h"
+
+#define FREE_Cx if (c1) {g_free(c1); c1=NULL;}\
+                if (c2) {g_free(c2); c2=NULL;}\
+                if (c3) {g_free(c3); c3=NULL;}
 
 void draw_inputln(struct inputln *il, int sel){
 
@@ -24,6 +36,11 @@ void draw_inputln(struct inputln *il, int sel){
     fill_area(il->x, il->y, il->l, 1, COL_INV);
     
     print_text(il->x, il->y, strlen(il->cdata + il->vpos) <= il->l ? strlen(il->cdata + il->vpos) : il->l, il->cdata + il->vpos, COL_INV);
+#ifdef Z_HAVE_SDL
+	if (sdl && cfg->usetouch) print_text(il->x + il->l, il->y, 1, "\x19", COL_INV);
+#endif
+
+
 
     if (il->focused) {
 /*        dbg("draw_inputln: set_cursor %d %d \n", il->x + il->cpos - il->vpos, il->y);*/
@@ -42,9 +59,9 @@ static void il_hist_check_bounds(struct inputln *il){
     if ((signed)il->hist_i >= (signed)il->history->len) il->hist_i = il->history->len - 1;
 }
 
-static void il_add_to_history(struct inputln *il){
-    if (strlen(il->cdata)==0) return;
-    g_ptr_array_add(il->history, g_strdup(il->cdata));
+void il_add_to_history(struct inputln *il, char *s){
+    if (strlen(s)==0) return;
+    g_ptr_array_add(il->history, g_strdup(s));
     if (il->history->len == IL_HIST_LEN){
         gpointer p;
         p = g_ptr_array_index(il->history, 0);
@@ -55,11 +72,11 @@ static void il_add_to_history(struct inputln *il){
 
 static struct inputln *gil;
 
-void il_history_func (cba_t cba, cba_t unused){
+void il_history_func (void *arg, void *unused){
     int active;
     struct inputln *il;
 
-    active=(int)GETCBA(cba, int_);
+    active=GPOINTER_TO_INT(arg);
     il=gil;
     
     if (active<0 || active>il->history->len) return;
@@ -84,12 +101,15 @@ int inputln_func(struct inputln *il, struct event *ev){
 /*            dbg("inputln_func EV_INIT\n");*/
             il->x = 20;
             il->y = term->y - cfg->loglines - 1;
-            if (il->band) il->y -= ctest->spypeers->len;
+			if (il->band && !ctest->oldcontest) il->y -= ctest->spypeers->len;
             
             il->l = 40;
+#ifdef Z_HAVE_SDL
+			if (cfg->usetouch) il->l--;
+#endif
             il->dlen = 256;
             if (!il->cdata)
-               il->cdata=(char*)mem_alloc(il->dlen);
+               il->cdata=(char*)g_malloc(il->dlen);
 
             memset(il->cdata,0,il->dlen);
             il->cpos = strlen(il->cdata);
@@ -99,17 +119,20 @@ int inputln_func(struct inputln *il, struct event *ev){
             il_readonly(il, il->readonly);
             draw_inputln(il,1);/*FIXME*/
             
-            break;
         case EV_RESIZE:
             if (il->sw){ 
                 il->x = il->sw->x;
                 il->y = il->sw->y + il->sw->h;
                 il->l = il->sw->w;
+#ifdef Z_HAVE_SDL
+				if (cfg->usetouch) il->l--;
+#endif
             }else{
                 il->x = 20;
                 il->y = term->y - cfg->loglines - 1;
-                if (il->band) il->y -= ctest->spypeers->len;
+				if (il->band && !ctest->oldcontest) il->y -= ctest->spypeers->len;
             }
+
             /*dbg("inputln_func RESIZE %p term->y=%d il->x=%d il->y=%d il->band=%p\n", il, term->y, il->x, il->y, il->band);*/
         
             
@@ -119,7 +142,7 @@ int inputln_func(struct inputln *il, struct event *ev){
             break;
         case EV_KBD:
             if ((ev->y & KBD_CTRL) &&
-                upcase(ev->x) == 'V' && il->allow_ctrlv){
+                z_char_uc(ev->x) == 'V' && il->allow_ctrlv){
                 if (il->readonly) break;
                 il->wasctrlv=1;
                 goto dsp_f;
@@ -182,6 +205,11 @@ int inputln_func(struct inputln *il, struct event *ev){
                 case ACT_KILL_LINE:
                     if (il->readonly) break;
                     clear_inputline(il);
+#ifdef Z_HAVE_SDL
+					if (gses->osk){
+						zosk_clear(gses->osk);
+					}
+#endif
                     goto dsp_f;    
                 case ACT_KILL_TO_BOL:
                     if (il->readonly) break;
@@ -202,6 +230,13 @@ int inputln_func(struct inputln *il, struct event *ev){
                                CTEXT(T_OK), CTEXT(T_CANCEL), il, 
                                NULL, 20, callkst, 0, 0, NULL,
                                (void (*)(void *, char *)) newkst, NULL, 0);
+#ifdef Z_HAVE_SDL
+					if (gses && cfg->usetouch){
+						if (gses->osk) zosk_free(gses->osk);
+						gses->osk = zosk_init(sdl->screen, ZOSK_ENTERCLOSE | ZOSK_SENDENTER, il->cdata);
+						zosk_portrait(gses->osk);
+					}
+#endif
                     goto dsp_f;
                     break;
                 case ACT_LASTCALL_KST:
@@ -212,7 +247,7 @@ int inputln_func(struct inputln *il, struct event *ev){
                     
                     if (il->band) break;
                     if (il->hist_i==-1 && strlen(il->cdata)>0){
-                        il_add_to_history(il);
+                        il_add_to_history(il, il->cdata);
                         il->hist_i++;
                     }
                     for (il->hist_i++; (signed)il->hist_i < (signed)il->history->len; il->hist_i++){
@@ -225,7 +260,7 @@ int inputln_func(struct inputln *il, struct event *ev){
                             dbg("c1=%s c2=%s c3=%s\n", c1, c2, c3);
                             
                             safe_strncpy(s, c2, sizeof(s));
-                            uc(s);
+                            z_str_uc(s);
                             g_snprintf(il->cdata, il->dlen, "/cq %s ", s);
                             FREE_Cx;
                             break;
@@ -233,12 +268,19 @@ int inputln_func(struct inputln *il, struct event *ev){
                         FREE_Cx;
                     }
                     il->cpos = strlen(il->cdata);
+#ifdef Z_HAVE_SDL
+					if (gses && cfg->usetouch){
+						if (gses->osk) zosk_free(gses->osk);
+						gses->osk = zosk_init(sdl->screen, ZOSK_ENTERCLOSE | ZOSK_SENDENTER, il->cdata);
+						zosk_portrait(gses->osk);
+					}
+#endif
                     goto dsp_f;
                     break;
                 case ACT_PREV_HISTORY:    
                     if (il->readonly) break;
                     if (il->hist_i==-1 && strlen(il->cdata)>0){
-                        il_add_to_history(il);
+                        il_add_to_history(il, il->cdata);
                         il->hist_i++;
                     }
                     il->hist_i++;
@@ -250,6 +292,13 @@ int inputln_func(struct inputln *il, struct event *ev){
                     else
                         strcpy(il->cdata, ""); /* FIXME */        
                     il->cpos = strlen(il->cdata);
+#ifdef Z_HAVE_SDL
+					if (gses && cfg->usetouch){
+						if (gses->osk) zosk_free(gses->osk);
+						gses->osk = zosk_init(sdl->screen, ZOSK_ENTERCLOSE | ZOSK_SENDENTER, il->cdata);
+						zosk_portrait(gses->osk);
+					}
+#endif
                     goto dsp_f;
                     break;
                 case ACT_NEXT_HISTORY:
@@ -263,9 +312,17 @@ int inputln_func(struct inputln *il, struct event *ev){
                     else
                         strcpy(il->cdata, ""); /* FIXME */        
                     il->cpos = strlen(il->cdata);
+#ifdef Z_HAVE_SDL
+					if (gses && cfg->usetouch){
+						if (gses->osk) zosk_free(gses->osk);
+						gses->osk = zosk_init(sdl->screen, ZOSK_ENTERCLOSE | ZOSK_SENDENTER, il->cdata);
+						zosk_portrait(gses->osk);
+					}
+#endif
                     goto dsp_f;
                     break;
                 case ACT_SHOW_HISTORY:{
+show_history:
                     if (il->readonly) break;
 
                     
@@ -273,13 +330,11 @@ int inputln_func(struct inputln *il, struct event *ev){
                     if (il->history->len==0) break;
                     if (!(mi = new_menu(1))) break;
                     for (i = 0; i < il->history->len; i++) {
-                        cba_t cba2;
-                        SETCBA(cba2, int_, i);
                         add_to_menu(&mi, (char *)g_ptr_array_index(il->history, i),
-                                "", "", il_history_func, cba2, 0);
+                                "", "", il_history_func, GINT_TO_POINTER(i), 0);
                     }
                     set_window_ptr(gses->win,il->x,il->y-2-i);
-                    do_menu_selected(mi, (cba_t)0, il->history->len-1);
+                    do_menu_selected(mi, GINT_TO_POINTER(0), il->history->len-1);
                     
                     }goto dsp_f;
                 case ACT_COPY_CLIPBOARD:
@@ -301,7 +356,7 @@ int inputln_func(struct inputln *il, struct event *ev){
                     strncpy(il->cdata , clipboard, il->item->dlen);
                     il->cdata[il->item->dlen - 1] = 0;
                     il->cpos = strlen(il->cdata);
-                    mem_free(clipboard);*/
+                    g_free(clipboard);*/
                     goto dsp_f;
                 }
                 default:
@@ -315,7 +370,7 @@ int inputln_func(struct inputln *il, struct event *ev){
                         if (strlen(il->cdata) < il->dlen - 1) {
                             memmove(il->cdata + il->cpos + 1, il->cdata + il->cpos, strlen(il->cdata) - il->cpos + 1);
                             c = ev->x;
-                            if (il->upconvert) c = upcase(c);
+                            if (il->upconvert) c = z_char_uc(c);
                             if (!il->valid_chars || strchr(il->valid_chars, c)!=NULL){
                                 il->cdata[il->cpos++] = c;
                                 il->cdata[il->dlen-1] = '\0';
@@ -358,11 +413,11 @@ int inputln_func(struct inputln *il, struct event *ev){
                 if (!il->enter) return 1;
                 /*dbg("inputln_func: KBD_ENTER '%s' \n",il->cdata);*/
                 
-                il_add_to_history(il);
+                il_add_to_history(il, il->cdata);
                 il->hist_i=-1;
                 
                 il->enter(il->enterdata, il->cdata, 1);
-                clear_inputline(il);
+				if (il->enter != process_input_no_contest) clear_inputline(il);
                 draw_inputln(il,1);
                 return 1;
             }
@@ -370,14 +425,28 @@ int inputln_func(struct inputln *il, struct event *ev){
             break;
         case EV_MOUSE:
             if (ev->b & B_MOVE) break;
-            if (ev->y != il->y || ev->x < il->x || ev->x >= il->x + il->l) return 0;
-            if ((il->cpos = il->vpos + ev->x - il->x) > strlen(il->cdata)) il->cpos = strlen(il->cdata);
+#ifdef Z_HAVE_SDL
+			if (gses && cfg->usetouch && ev->b & B_CLICK){
+				int flg = ZOSK_SENDENTER;
+				if (abs(il->x + il->l - ev->x) < 2) goto show_history;
+				if (gses->osk) zosk_free(gses->osk);
+				if (!il->band) flg |= ZOSK_ENTERCLOSE;
+				else flg |= ZOSK_CLEARENTER;
+				if (il->upconvert) flg |= ZOSK_UPCONVERT;
+				gses->osk = zosk_init(sdl->screen, flg, il->cdata);
+				zosk_portrait(gses->osk);
+			}
+#endif
+			if (!cfg->usetouch){
+				if (ev->y != il->y || ev->x < il->x || ev->x >= il->x + il->l) return 0;
+				if ((il->cpos = il->vpos + ev->x - il->x) > strlen(il->cdata)) il->cpos = strlen(il->cdata);
+			}
             draw_inputln(il,1);
             break;
 
         case EV_ABORT:
-            if (il->cdata) mem_free(il->cdata);
-            g_ptr_array_free_all(il->history);
+            if (il->cdata) g_free(il->cdata);
+            zg_ptr_array_free_all(il->history);
             break;
     }
     return 0;
@@ -385,11 +454,13 @@ int inputln_func(struct inputln *il, struct event *ev){
 
 void il_set_focus(struct inputln *il){
     if (!il) return;  /* subwins having no inputline */
+	//dbg("il_set_focus(%c %p)\n", il->band ? il->band->bandchar : ' ', il->sw);
     il->focused=1;            
 }
 
 void il_unset_focus(struct inputln *il){
     if (!il) return;
+	//dbg("il_unset_focus(%c %p)\n", il->band ? il->band->bandchar : ' ', il->sw);
     il->focused=0;
 }
 
@@ -405,10 +476,15 @@ void il_readonly(struct inputln *il, int ro){
 void newkst(struct inputln *il, char *callkst){
 
     if (!il || !callkst) return;
-    uc(callkst);
+    z_str_uc(callkst);
     //g_snprintf(il->cdata, il->dlen, "/cq %s (%s) ", callkst, callkst);
     g_snprintf(il->cdata, il->dlen, "/cq %s ", callkst);  // new format since abt. jun 2011
     il->cpos = strlen(il->cdata);
 
 }
 
+void askkst(struct inputln *il, char *s){
+    if (!il) return;
+    g_snprintf(il->cdata, il->dlen, "%s", s);
+    il->cpos = strlen(il->cdata);
+}
