@@ -413,7 +413,7 @@ void sw_map_redraw(struct subwin *sw, struct band *band, int flags){
 //    plot_info_qrv(sw, sw->screen, band, qrv_get(qrv, sw->minqrvcall));
     plot_qrb_qth(sw, sw->screen, sdl->mouse_x, sdl->mouse_y);
 //	ST_STOP; 
-    
+	//plot_hgts(sw, &sw->map);
 }
 
 void sw_map_check_bounds(struct subwin *sw){
@@ -1330,6 +1330,60 @@ void map_recalc_gst(struct subwin *sw, struct band *band){
     for (i=0; i<360; i++) sw->gst[i] = sw->gst[i] / max; 
 } 
 
+int plot_hgt(struct subwin *sw, SDL_Rect *area, int xx, int yy, char *file)
+{
+	char s[256];
+	FILE *f;
+	int x, y, max = -10000, min = 10000, c;
+
+	strcpy(s, "../src/");
+	strcat(s, file);
+	z_wokna(s);
+	f = fopen(s, "rb");
+	if (f){
+		signed short u;
+		for (x = 0; x <= 1201; x++){
+			for (y = 0; y <= 1201; y++){
+				if (fread(&u, sizeof(u), 1, f) != 1) 
+					break;
+				u = ntohs(u);
+				if (u == -32768) continue;
+				if (u < min) min = u;
+				if (u > max) max = u;
+			}
+		}
+		fclose(f);
+	}
+	
+    SDL_SetClipRect(sw->screen, &sw->map);
+
+	f = fopen(s, "rb");
+	if (f){
+		signed short u;
+		for (y = 0; y <= 1200; y++){
+			for (x = 0; x <= 1200; x++){
+				if (fread(&u, sizeof(u), 1, f) != 1) 
+					break;
+				u = ntohs(u);
+				if (u == -32768) {
+					z_putpixel(sw->screen, xx + x/2 + area->x, yy + y/2 + area->y, z_makecol(0, 50, 100));
+				}else{
+					c = (u - min) * 255 / (max - min);
+					z_putpixel(sw->screen, xx + x/2 + area->x, yy + y/2 + area->y, z_makecol(c, c, c));
+				}
+			}
+		}
+		fclose(f);
+	}
+}
+
+void plot_hgts(struct subwin *sw, SDL_Rect *area){
+	plot_hgt(sw, area, 0, 0, "N50E012.hgt");
+	plot_hgt(sw, area, 600, 0, "N50E013.hgt");
+	plot_hgt(sw, area, 0, 600, "N49E012.hgt");
+	plot_hgt(sw, area, 600, 600, "N49E013.hgt");
+}
+
 #else
 int sw_map_kbd_func(struct subwin *sw, struct event *ev, int fw){
     return 0;
