@@ -1043,8 +1043,8 @@ static struct menu_item file_menu22[] = {
 #endif
     {CTEXT(T_MEMORY_INFO), "", CTEXT(T_HK_MEMORY_INFO), MENU_FUNC memory_info, NULL, 0, 0},
     {CTEXT(T_NINFO), "", CTEXT(T_HK_NINFO), MENU_FUNC net_info, NULL, 0, 0},
-	{CTEXT(T_CRASH_TEST), "", CTEXT(T_HK_CRASH_TEST), crash_test, NULL, 0, 0},
-	{CTEXT(T_INTERROR_TEST), "", CTEXT(T_HK_INTERNAL_TEST), internal_test, NULL, 0, 0},
+//	{CTEXT(T_CRASH_TEST), "", CTEXT(T_HK_CRASH_TEST), crash_test, NULL, 0, 0},
+//	{CTEXT(T_INTERROR_TEST), "", CTEXT(T_HK_INTERNAL_TEST), internal_test, NULL, 0, 0},
 	{CTEXT(T_DUMP_RW_BANDS), "", CTEXT(T_HK_DUMP_RW_BANDS), dump_rw, NULL, 0, 0},
 	{CTEXT(T_UPDATE), "", CTEXT(T_HK_UPDATE), update_tucnak, NULL, 0, 0},
     {"", "", M_BAR, NULL, NULL, 0, 0}
