@@ -10,6 +10,20 @@
 
 #include "header.h"
 
+#include "bfu.h"
+#include "chart.h"
+#include "edi.h"
+#include "map.h"
+#include "menu.h"
+#include "net.h"
+#include "qrvdb.h"
+#include "qsodb.h"
+#include "session.h"
+#include "sked.h"
+#include "stats.h"
+#include "terminal.h"
+
+
 /******************** MODES *********************************/
 
 #define MAX_MODES 9
@@ -17,10 +31,10 @@
 gint mode;
 char mode_str[EQSO_LEN];
 
-void mode_func (cba_t cba){
+void mode_func (void *arg){
     int active;
 
-    active=(int)GETCBA(cba, int_);
+    active=GPOINTER_TO_INT(arg);
     if (active<0 || active>MAX_MODES) return;
     mode = active;
     safe_strncpy0(mode_str,mode_msg[mode],EQSO_LEN);
@@ -34,12 +48,12 @@ int dlg_mode(struct dialog_data *dlg, struct dialog_item_data *di){
     
     if (!(mi = new_menu(1))) return 0;
     for (i = 1; i <= MAX_MODES; i++) {
-        add_to_menu(&mi, mode_msg[i], "", "", MENU_FUNC mode_func, (cba_t)i, 0);
+        add_to_menu(&mi, mode_msg[i], "", "", MENU_FUNC mode_func, GINT_TO_POINTER(i), 0);
     }
     sel = mode-1;
     if (sel < 0) sel = 0;
     if (sel>=MAX_MODES) sel=0;
-    do_menu_selected(mi, (cba_t)mode, sel);
+    do_menu_selected(mi, GINT_TO_POINTER(mode), sel);
     return 0;
 }
 
@@ -65,12 +79,19 @@ int pbandi=0;
 void refresh_edit_qso(struct qso *q)
 {
     char *token_ptr, *h, *m;
+	char raw[30];
+	struct qso *qq;
 
 
+	
     
 /*    dbg("refresh_edit_qso\n");*/
     dump_qso(q, "refresh_edit_qso before");
 
+	z_get_raw_call(raw, q->callsign);
+	g_hash_table_remove(q->band->rawqsoshash, raw);
+	
+
     if ((strcmp(rstr,oorstr) || strcmp(rsts,oorsts)) &&
          !strcmp(mode_str,oomode_str)){
 
@@ -117,7 +138,7 @@ void refresh_edit_qso(struct qso *q)
 
     q->stamp = time(NULL);
     g_hash_table_foreach(ctest->bystamp, foreach_source_qsort_by_stamp, NULL);
-    g_index_array_qsort(ctest->allqsos, compare_date_time_qsonrs);
+    z_ptr_array_qsort(ctest->allqsos, compare_date_time_qsonrs);
    
     compute_qrbqtf(q);
     dirty_band(aband);
@@ -125,11 +146,18 @@ void refresh_edit_qso(struct qso *q)
     replicate_qso(NULL, q);
     recalc_stats(aband);
     recalc_worked_skeds(aband);
-    qrv_recalc_wkd(qrvdb);
-    qrv_recalc_qrbqtf(qrvdb);
-    qrv_recalc_wkd(qrvdb);
+    qrv_recalc_wkd(qrv);
+    qrv_recalc_qrbqtf(qrv);
+    qrv_recalc_gst(qrv);
+	
+	if (!q->dupe && !q->error){
+		z_get_raw_call(raw, q->callsign);
+		qq = (struct qso *)g_hash_table_lookup(q->band->rawqsoshash, raw);
+		if (!qq) g_hash_table_insert(q->band->rawqsoshash, g_strdup(raw), q);
+	}
+
     /*check_autosave();*/
-#ifdef HAVE_SDL
+#ifdef Z_HAVE_SDL
 	maps_reload();
 #endif
     chart_reload();    
@@ -263,11 +291,11 @@ void edit_qso(struct qso *qso)
     dump_qso(qso, "edit_qso");
     dump_all_sources(ctest);
     
-    safe_strncpy0(callsign,  qso->callsign,  EQSO_LEN); uc(callsign);
+    safe_strncpy0(callsign,  qso->callsign,  EQSO_LEN); z_str_uc(callsign);
     safe_strncpy0(rstr,      qso->rstr,      EQSO_LEN);
     safe_strncpy0(qsonrr,    qso->qsonrr,    EQSO_LEN);
     safe_strncpy0(exc,       qso->exc,       EQSO_LEN); 
-    safe_strncpy0(locator,   qso->locator,   EQSO_LEN); uc(locator);
+    safe_strncpy0(locator,   qso->locator,   EQSO_LEN); z_str_uc(locator);
     mode=qso->mode;
     safe_strncpy0(mode_str,  mode_msg[mode], EQSO_LEN);
     safe_strncpy0(rsts,      qso->rsts,      EQSO_LEN);
@@ -309,10 +337,10 @@ void edit_qso(struct qso *qso)
     g_snprintf(oqrg, QRG_LEN, "%s %s Hz", VTEXT(T_QRG), s);
     
     
-    if (!(d = (struct dialog*)mem_alloc(sizeof(struct dialog) + 55 * sizeof(struct dialog_item)))) return;
+    if (!(d = (struct dialog*)g_malloc(sizeof(struct dialog) + 55 * sizeof(struct dialog_item)))) return;
     memset(d, 0, sizeof(struct dialog) + 55 * sizeof(struct dialog_item));
     /*d->title = "Edit QSO";*/
-    g_snprintf(ss,100,VTEXT(T_EDIT_QSO_CD), upcase(aband->bandchar), atoi(qso->qsonrs));
+    g_snprintf(ss,100,VTEXT(T_EDIT_QSO_CD), aband->bandchar, atoi(qso->qsonrs));
     d->title = ss;
     d->fn = edit_qso_fn;
     d->refresh = (void (*)(void *))refresh_edit_qso;
