@@ -1,6 +1,6 @@
 /*
     Tucnak - VHF contest log
-    Copyright (C) 2002-2006  Ladislav Vaiz <ok1zia@nagano.cz>
+    Copyright (C) 2002-2015  Ladislav Vaiz <ok1zia@nagano.cz>
 
     This program is free software; you can redistribute it and/or
     modify it under the terms of the GNU General Public License
@@ -8,8 +8,32 @@
 
 */
 
+
 #include "header.h"
 
+#include "bfu.h"
+#include "control.h"
+#include "cwdb.h"
+#include "ebw.h"
+#include "edi.h"
+#include "excdb.h"
+#include "fifo.h"
+#include "hf.h"
+#include "inputln.h"
+#include "kbd.h"
+#include "main.h"
+#include "menu.h"
+#include "namedb.h"
+#include "net.h"
+#include "qrvdb.h"
+#include "qsodb.h"
+#include "session.h"
+#include "stats.h"
+#include "subwin.h"
+#include "terminal.h"
+#include "tregex.h"
+#include "trig.h"
+#include "zosk.h"
 
 /*********************** DUPLICATE CALLSIGN *********************************/
 
@@ -121,8 +145,15 @@ void duplicate_callsign(struct qso *qso)
     struct dialog *d;
     int i;
 
+#ifdef Z_HAVE_SDL
+	if (gses->osk){
+		zosk_free(gses->osk);
+		gses->osk = NULL;
+		sdl_force_redraw();
+	}
+#endif
 
-    if (!(d = mem_alloc(sizeof(struct dialog) + 55 * sizeof(struct dialog_item)))) return;
+    if (!(d = g_malloc(sizeof(struct dialog) + 55 * sizeof(struct dialog_item)))) return;
     memset(d, 0, sizeof(struct dialog) + 55 * sizeof(struct dialog_item));
     d->title = VTEXT(T_DUPE_QSO);
     d->fn = duplicate_callsign_fn;
@@ -157,7 +188,7 @@ void duplicate_callsign(struct qso *qso)
 
 /**************************** EDIT ***********************************/
 
-void menu_add_error(cba_t cba){
+void menu_add_error(void *arg){
     add_error(aband, "");
 } 
 
@@ -187,11 +218,11 @@ void chop(struct band *band, char *op){
 void refresh_chop(void *xxx){
     int i;
 
-    uc(op);
-    if (!regcmp(op,"^([0-9][A-Z]|[A-Z]{1,2}[0-9]?)[0-9]{1,4}[A-Z]{1,4}$")==0){
+    z_str_uc(op);
+    /*if (!regcmp(op,"^([0-9][A-Z]|[A-Z]{1,2}[0-9]?)[0-9]{1,4}[A-Z]{1,4}$")==0){
         errbox(VTEXT(T_BAD_CALL), 0);
         return;
-    }
+    } */
     cfg->global_operator = global_operator;
     
     if (global_operator){
@@ -256,7 +287,7 @@ void chop_fn(struct dialog_data *dlg)
 }
 
 
-void menu_chop(cba_t cba){
+void menu_chop(void *arg){
     struct dialog *d;
     int i;
     
@@ -266,12 +297,8 @@ void menu_chop(cba_t cba){
     safe_strncpy0(op, aband->operator_, EQSO_LEN);
     global_operator=cfg->global_operator;
 
-/*    input_field(NULL, VTEXT(T_CHOP), VTEXT(T_N_CALLSIGN), 
-               VTEXT(T_OK), VTEXT(T_CANCEL), NULL, 
-               NULL, 20, op, 0, 0, NULL,
-               (void (*)(void *, char *)) chop, NULL);*/
     
-    if (!(d = mem_alloc(sizeof(struct dialog) + 10 * sizeof(struct dialog_item)))) return;
+    if (!(d = (struct dialog *)g_malloc(sizeof(struct dialog) + 10 * sizeof(struct dialog_item)))) return;
     memset(d, 0, sizeof(struct dialog) + 10 * sizeof(struct dialog_item));
     d->title = VTEXT(T_CHOP);
     d->fn = chop_fn;
@@ -285,7 +312,7 @@ void menu_chop(cba_t cba){
     d->items[i].gid = 0;
     d->items[i].gnum = 1;
     d->items[i].dlen = sizeof(int);
-    d->items[i].data = (void *)&global_operator;
+    d->items[i].data = (char *)&global_operator;
     
 
     d->items[++i].type = D_BUTTON;
@@ -321,7 +348,7 @@ void do_skedqrg(struct session *ses, char *qrg){
     redraw_later();
 }
 
-void menu_skedqrg(cba_t cba){
+void menu_skedqrg(void *arg){
     static char qrg[20];
     
     if (!ctest || !aband) return;
@@ -337,8 +364,10 @@ void menu_skedqrg(cba_t cba){
 }
 
 /******************** FILL OPERATORS **************************/
+// arg==0 -> fill only when empty, don't save (from edi export)
+// arg==1 -> overwrite ops, save (from menu)
 
-void menu_fillop(cba_t cba){
+void menu_fillop(void *arg){
     struct band *b;
     struct qso *q;
     int i,j,k;
@@ -349,25 +378,27 @@ void menu_fillop(cba_t cba){
     if (!ctest) return;
     for (i=0; i<ctest->bands->len; i++){
         b = (struct band *)g_ptr_array_index(ctest->bands,i);
-        
+
+		if (GPOINTER_TO_INT(arg) == 0 && strlen(b->mope1) != 0) continue;
+		
         calls=g_ptr_array_new();
         for (j=0;j<b->qsos->len;j++){
             q=(struct qso *)g_ptr_array_index(b->qsos,j);
 
             for (k=0;k<calls->len;k++){
-                if (strcmp(q->operator_,g_ptr_array_index(calls,k))==0) break;
+                if (strcmp(q->operator_,(char*)g_ptr_array_index(calls,k))==0) break;
             }
             if (k==calls->len)
                 g_ptr_array_add(calls, q->operator_);
             
         }
 
-        g_ptr_array_qsort(calls, compare_gstring);
+        zg_ptr_array_qsort(calls, z_compare_string);
         gs1=g_string_sized_new(100);
         gs2=g_string_sized_new(100);
         gs=gs1;
         for (k=0; k<calls->len;k++){
-            c=g_ptr_array_index(calls, k);
+            c=(char*)g_ptr_array_index(calls, k);
             if (gs->len+2+strlen(c) >= 50) gs=gs2;
             g_string_append(gs, ";");
             g_string_append(gs, c);
@@ -385,26 +416,29 @@ void menu_fillop(cba_t cba){
         b->mope2=g_strdup(gs2->str);
         g_string_free(gs2,TRUE);
     }
-    save_all_bands_txt(0);
+	if (GPOINTER_TO_INT(arg) != 0){
+		save_all_bands_txt(0);
+	}
 }
 
 /******************** RECALC ALL QRB and QTF  **************************/
 
-void menu_recalc_qrb(cba_t cba){
+void menu_recalc_qrb(void *arg){
 	if (ctest) recalc_all_qrbqtf(ctest);
-    qrv_recalc_wkd(qrvdb);
-    qrv_recalc_qrbqtf(qrvdb);
-    qrv_recalc_gst(qrvdb);
+    qrv_recalc_wkd(qrv);
+    qrv_recalc_qrbqtf(qrv);
+    qrv_recalc_gst(qrv);
 }
 
 /******************** UPDATE C_W ******************************/
 
-void menu_cw_update_contest(cba_t cba){
+void menu_cw_update_contest(void *arg){
     gchar *s;
     int ret;
     
     update_cw_from_ctest(cw, ctest);
-    s = g_strconcat(getenv("HOME"), "/tucnak/tucnakcw", NULL);
+    s = g_strconcat(tucnak_dir, "/tucnakcw", NULL);
+	z_wokna(s);
     ret=save_cw_into_file(cw, s);        
     if (ret){
         errbox(VTEXT(T_CANT_WRITE), ret);
@@ -414,12 +448,13 @@ void menu_cw_update_contest(cba_t cba){
     g_free(s);
 }
 
-void menu_cw_update_band(cba_t cba){
+void menu_cw_update_band(void *arg){
     gchar *s;
     int ret;
     
     update_cw_from_band(cw, aband);
-    s = g_strconcat(getenv("HOME"), "/tucnak/tucnakcw", NULL);
+    s = g_strconcat(tucnak_dir, "/tucnakcw", NULL);
+	z_wokna(s);
     ret=save_cw_into_file(cw, s);        
     if (ret){
         errbox(VTEXT(T_CANT_WRITE), ret);
@@ -431,7 +466,7 @@ void menu_cw_update_band(cba_t cba){
 
 /******************** UPDATE EXC ******************************/
 
-void menu_exc_update_contest(cba_t cba){
+void menu_exc_update_contest(void *arg){
     gchar *s, *ename;
     int ret;
     
@@ -441,10 +476,11 @@ void menu_exc_update_contest(cba_t cba){
     if (!ctest->excname || !*ctest->excname) return;
 
     ename = g_strdup(ctest->excname);
-    lc(ename);
+    z_str_lc(ename);
     
     update_exc_from_ctest(excdb, ctest);
-    s = g_strconcat(getenv("HOME"), "/tucnak/tucnakexc", ename, NULL);
+    s = g_strconcat(tucnak_dir, "/tucnakexc", ename, NULL);
+	z_wokna(s);
     ret=save_exc_into_file(excdb, s);        
     if (ret){
         errbox(VTEXT(T_CANT_WRITE), ret);
@@ -454,7 +490,7 @@ void menu_exc_update_contest(cba_t cba){
     g_free(s);
 }
 
-void menu_exc_update_band(cba_t cba){
+void menu_exc_update_band(void *arg){
     gchar *s, *ename;
     int ret;
     
@@ -463,9 +499,10 @@ void menu_exc_update_band(cba_t cba){
     if (!ctest->excname || !*ctest->excname) return;
 
     ename = g_strdup(ctest->excname);
-    lc(ename);
+    z_str_lc(ename);
     update_exc_from_band(excdb, aband);
-    s = g_strconcat(getenv("HOME"), "/tucnak/tucnakexc", ename, NULL);
+    s = g_strconcat(tucnak_dir, "/tucnakexc", ename, NULL);
+	z_wokna(s);
     ret=save_exc_into_file(excdb, s);        
     if (ret){
         errbox(VTEXT(T_CANT_WRITE), ret);
@@ -475,30 +512,14 @@ void menu_exc_update_band(cba_t cba){
     g_free(s);
 }
 
-void menu_import_ebw(cba_t cba){
-    gchar *s;
+void menu_import_ebw(void *arg){
     int ret;
     
     ret=read_ebw_files(cw, namedb);
 	if (ret!=0) return;
     
-    s = g_strconcat(getenv("HOME"), "/tucnak/tucnakcw", NULL);
-    ret=save_cw_into_file(cw, s);        
-    if (ret){
-        errbox(VTEXT(T_CANT_WRITE), ret);
-    }else{
-        log_addf(VTEXT(T_SAVED_S), s);
-    }
-    g_free(s);
-
-    s = g_strconcat(getenv("HOME"), "/tucnak/tucnaknames", NULL);
-    ret=save_namedb_into_file(namedb, s);        
-    if (ret){
-        errbox(VTEXT(T_CANT_WRITE), ret);
-    }else{
-        log_addf(VTEXT(T_SAVED_S), s);
-    }
-    g_free(s);
+	save_cw(cw, 1);
+    save_namedb(namedb, 1);
 }
 
 /******************** AUTOSAVE *******************************/
@@ -625,7 +646,7 @@ void autosave_fn(struct dialog_data *dlg)
     
 }
 
-void menu_autosave(cba_t cba)
+void menu_autosave(void *arg)
 {
     struct dialog *d;
     int i;
@@ -634,15 +655,15 @@ void menu_autosave(cba_t cba)
  /*   snprint(as_disk_aq_str,AS_LEN, cfg->as_disk_aq);
     snprint(as_disk_am_str, AS_LEN, cfg->as_disk_am);
     as_disk_fsync = cfg->as_disk_fsync;*/
-    snprint(as_floppy_aq_str, AS_LEN, cfg->as_floppy_aq);
-    snprint(as_floppy_am_str, AS_LEN, cfg->as_floppy_am);
+    g_snprintf(as_floppy_aq_str, AS_LEN, "%d", cfg->as_floppy_aq);
+    g_snprintf(as_floppy_am_str, AS_LEN, "%d", cfg->as_floppy_am);
     as_floppy_fsync = cfg->as_floppy_fsync;
     as_mount = cfg->as_mount;
     safe_strncpy0(as_floppy_path, cfg->as_floppy_path, MAX_STR_LEN); 
     safe_strncpy0(as_mount_cmd, cfg->as_mount_cmd, MAX_STR_LEN); 
     
 
-    if (!(d = mem_alloc(sizeof(struct dialog) + 55 * sizeof(struct dialog_item)))) return;
+    if (!(d = (struct dialog *)g_malloc(sizeof(struct dialog) + 55 * sizeof(struct dialog_item)))) return;
     memset(d, 0, sizeof(struct dialog) + 55 * sizeof(struct dialog_item));
     d->title = VTEXT(T_AUTOSAVE_OPTS);
     d->fn = autosave_fn;
@@ -682,14 +703,14 @@ void menu_autosave(cba_t cba)
     d->items[i].gid = 0;
     d->items[i].gnum = 1;
     d->items[i].dlen = sizeof(int);
-    d->items[i].data = (void *)&as_floppy_fsync;
+    d->items[i].data = (char *)&as_floppy_fsync;
     
 
     d->items[++i].type = D_CHECKBOX;
     d->items[i].gid = 0;
     d->items[i].gnum = 1;
     d->items[i].dlen = sizeof(int);
-    d->items[i].data = (void *)&as_mount;
+    d->items[i].data = (char *)&as_mount;
     
     d->items[++i].type = D_FIELD;
     d->items[i].dlen = MAX_STR_LEN;
@@ -724,178 +745,110 @@ char asw_cmd[MAX_STR_LEN];
 char asw_respawn_str[MAX_STR_LEN];
 
                                                    
-void refresh_add_sw(struct session *ses){
-    struct subwin *sw;
+void refresh_add_sw(void *arg){
+	int where_ = -1;
+    struct subwin *asw = (struct subwin *)arg;
+	struct subwin *newsw;
+
 
     dbg("refresh_add_sw\n");
-    
+	{
+		zg_ptr_array_foreach(struct subwin *, subwin, gses->subwins)
+		{
+			if (subwin != asw) continue;
+			where_ = subwin_i;
+			break;
+		}
+	}
+	
+	newsw = new_subwin((enum sw_type)asw_type, where_);
+	if (newsw){
+    	struct event ev = { EV_RESIZE, 0, 0, 0, 0, 0};
 
-    switch(asw_type){
-        case SWT_PIPE:/* shell */
-            sw = new_subwin(SWT_PIPE, VTEXT(T_SW_SHELL), sw_pipe_enter);
-            break;
-        case SWT_KST3:/* KST in tucnak3, Shell in tucnak2 */
-            sw = new_subwin(SWT_KST3, VTEXT(T_SW_SHELL), sw_pipe_enter);
-            break;
-        case SWT_QSOS: 
-            sw = new_subwin(SWT_QSOS, VTEXT(T_SW_QSOS), NULL);
-            break;
-        case SWT_LOG: 
-            sw = new_subwin(SWT_LOG, VTEXT(T_SW_LOG), NULL);
-            break;
-        case SWT_TALK: 
-            sw = new_subwin(SWT_TALK, VTEXT(T_SW_TALK), NULL);
-            break;
-        case SWT_SKED: 
-            sw = new_subwin(SWT_SKED, VTEXT(T_SW_SKED), NULL);
-            break;
-        case SWT_UNFI: 
-            sw = new_subwin(SWT_UNFI, VTEXT(T_SW_UNFI), NULL);
-            break;
-        case SWT_DXC: 
-            sw = new_subwin(SWT_DXC, VTEXT(T_SW_DXC), NULL);
-            break;
-        case SWT_STAT: 
-            sw = new_subwin(SWT_STAT, VTEXT(T_SW_STAT), NULL);
-            break;
-        case SWT_SWAP: 
-/*            sw = new_subwin(SWT_SWAP, VTEXT(T_SW_STAT), NULL);*/
-            break;
-        case SWT_MAP:
-            sw = new_subwin(SWT_MAP, VTEXT(T_SW_MAP), NULL);
-            break;
-        case SWT_SCOPE:
-            sw = new_subwin(SWT_SCOPE, VTEXT(T_SW_SCOPE), NULL);
-            break;
-        case SWT_QRV:
-            sw = new_subwin(SWT_QRV, VTEXT(T_SW_QRV), NULL);
-            break;
-        case SWT_HF:
-            sw = new_subwin(SWT_HF, VTEXT(T_SW_HF), NULL);
-            break;
-        case SWT_PLAYER:
-            sw = new_subwin(SWT_PLAYER, VTEXT(T_SW_PLAYER), NULL);
-            break;
-        case SWT_CHART:
-            sw = new_subwin(SWT_CHART, VTEXT(T_SW_CHART), NULL);
-            break;
-            
-    }
+		if (gses->ontop == asw) gses->ontop = newsw;
+		if (asw){
+			newsw->ontop = asw->ontop;
+			newsw->ontop2 = asw->ontop2;
+			newsw->focused = asw->focused;
+		}
+		sw_default_func(newsw, &ev, 1);
+		
+	}
+	if (asw) free_subwin(asw);
     
 }
 
 
-void menu_add_subwin(cba_t cba){
-
+void menu_add_subwin(void *arg){
     struct dialog *d;
-    int i;
+    int i, change;
 
-    dbg("menu_add_sw\n");
-    asw_type=SWT_PIPE;
+	change = GPOINTER_TO_INT(arg);
+	
+	if (!change){
+		asw_type = SWT_SHELL;
+	}
     safe_strncpy0(asw_cmd, "/bin/sh", MAX_STR_LEN);
-    snprint(asw_respawn_str, MAX_STR_LEN, 10);
+    g_snprintf(asw_respawn_str, MAX_STR_LEN, "%d", 10);
     
-    if (!(d = mem_alloc(sizeof(struct dialog) + 20 * sizeof(struct dialog_item)))) return;
-    memset(d, 0, sizeof(struct dialog) + 20 * sizeof(struct dialog_item));
-    d->title = VTEXT(T_ADD_SUBWIN);
+    d = g_new0(struct dialog, 20);
+	if (!change){
+		d->title = VTEXT(T_ADD_SUBWIN);
+	}else{
+		d->title = VTEXT(T_CHANGE_TYPE);
+		d->refresh_data = gses->ontop;
+	}
     d->fn = dlg_pf_fn;
-    d->refresh = (void (*)(void *))refresh_add_sw;
-    d->refresh_data = (void *)gses; /*todo */
+    d->refresh = refresh_add_sw;
+    
+	i = -1;
+	d->y0 = 1;
 
-    
-    d->items[i=0].type = D_CHECKBOX;
-    d->items[i].gid = 1;
-    d->items[i].gnum = SWT_QSOS;
-    d->items[i].dlen = sizeof(int);
-    d->items[i].data = (void *)&asw_type;
-    d->items[i].msg = CTEXT(T_QSOS);
-    d->items[i].wrap = 1;
-    
     d->items[++i].type = D_CHECKBOX;
     d->items[i].gid = 1;
-    d->items[i].gnum = SWT_SKED;
+    d->items[i].gnum = SWT_KST;
     d->items[i].dlen = sizeof(int);
-    d->items[i].data = (void *)&asw_type;
-    d->items[i].msg = CTEXT(T_SKEDS);
+    d->items[i].data = (char *)&asw_type;
+    d->items[i].msg = CTEXT(T_KST);
     d->items[i].wrap = 1;
-    
-    d->items[++i].type = D_CHECKBOX;
+
+    d->items[++i].type = D_CHECKBOX;  
     d->items[i].gid = 1;
-    d->items[i].gnum = SWT_TALK;
+    d->items[i].gnum = SWT_SHELL;
     d->items[i].dlen = sizeof(int);
-    d->items[i].data = (void *)&asw_type;
-    d->items[i].msg = CTEXT(T_TALK);
-    d->items[i].wrap = 1;
-    
-    d->items[++i].type = D_CHECKBOX;
-    d->items[i].gid = 1;
-    d->items[i].gnum = SWT_DXC;
-    d->items[i].dlen = sizeof(int);
-    d->items[i].data = (void *)&asw_type;
-    d->items[i].msg = CTEXT(T_DXC);
-    d->items[i].wrap = 1;
-    
-    d->items[++i].type = D_CHECKBOX; /* 5 */
-    d->items[i].gid = 1;
-    d->items[i].gnum = SWT_UNFI;
-    d->items[i].dlen = sizeof(int);
-    d->items[i].data = (void *)&asw_type;
-    d->items[i].msg = CTEXT(T_UNFI);
-    d->items[i].wrap = 1;
-    
-    d->items[++i].type = D_CHECKBOX;
-    d->items[i].gid = 1;
-    d->items[i].gnum = SWT_STAT;
-    d->items[i].dlen = sizeof(int);
-    d->items[i].data = (void *)&asw_type;
-    d->items[i].msg = CTEXT(T_STAT);
-    d->items[i].wrap = 1;
-    
-    d->items[++i].type = D_CHECKBOX;  /* 0 */
-    d->items[i].gid = 1;
-    d->items[i].gnum = SWT_PIPE;
-    d->items[i].dlen = sizeof(int);
-    d->items[i].data = (void *)&asw_type;
+    d->items[i].data = (char *)&asw_type;
     d->items[i].msg = CTEXT(T_SHELL);
     d->items[i].wrap = 1;
     
     d->items[++i].type = D_CHECKBOX;
     d->items[i].gid = 1;
-    d->items[i].gnum = SWT_MAP;
+    d->items[i].gnum = SWT_QSOS;
     d->items[i].dlen = sizeof(int);
-    d->items[i].data = (void *)&asw_type;
-    d->items[i].msg = CTEXT(T_MAP);
-    d->items[i].wrap = 1;
-    
-    d->items[++i].type = D_CHECKBOX;
-    d->items[i].gid = 1;
-    d->items[i].gnum = SWT_SCOPE;
-    d->items[i].dlen = sizeof(int);
-    d->items[i].data = (void *)&asw_type;
-    d->items[i].msg = CTEXT(T_SCOPE);
-    d->items[i].wrap = 1;
-    
-    d->items[++i].type = D_CHECKBOX;
-    d->items[i].gid = 1;
-    d->items[i].gnum = SWT_QRV;
-    d->items[i].dlen = sizeof(int);
-    d->items[i].data = (void *)&asw_type;
-    d->items[i].msg = CTEXT(T_QRV);
+    d->items[i].data = (char *)&asw_type;
+    d->items[i].msg = CTEXT(T_QSOS);
     d->items[i].wrap = 1;
     
     d->items[++i].type = D_CHECKBOX;
     d->items[i].gid = 1;
     d->items[i].gnum = SWT_HF;
     d->items[i].dlen = sizeof(int);
-    d->items[i].data = (void *)&asw_type;
+    d->items[i].data = (char *)&asw_type;
     d->items[i].msg = CTEXT(T_HF);
     d->items[i].wrap = 1;
     
+    d->items[++i].type = D_CHECKBOX;
+    d->items[i].gid = 1;
+    d->items[i].gnum = SWT_QRV;
+    d->items[i].dlen = sizeof(int);
+    d->items[i].data = (char *)&asw_type;
+    d->items[i].msg = CTEXT(T_QRV);
+    d->items[i].wrap = 1;
+    
     d->items[++i].type = D_CHECKBOX;
     d->items[i].gid = 1;
     d->items[i].gnum = SWT_LOG;
     d->items[i].dlen = sizeof(int);
-    d->items[i].data = (void *)&asw_type;
+    d->items[i].data = (char *)&asw_type;
     d->items[i].msg = CTEXT(T_LOG);
     d->items[i].wrap = 1;
     
@@ -903,7 +856,7 @@ void menu_add_subwin(cba_t cba){
     d->items[i].gid = 1;
     d->items[i].gnum = SWT_PLAYER;
     d->items[i].dlen = sizeof(int);
-    d->items[i].data = (void *)&asw_type;
+    d->items[i].data = (char *)&asw_type;
     d->items[i].msg = CTEXT(T_SW_PLAYER);
     d->items[i].wrap = 1;
     
@@ -911,10 +864,75 @@ void menu_add_subwin(cba_t cba){
     d->items[i].gid = 1;
     d->items[i].gnum = SWT_CHART;
     d->items[i].dlen = sizeof(int);
-    d->items[i].data = (void *)&asw_type;
+    d->items[i].data = (char *)&asw_type;
     d->items[i].msg = CTEXT(T_SW_CHART);
     d->items[i].wrap = 1;
     
+
+    d->items[++i].type = D_CHECKBOX;
+    d->items[i].gid = 1;
+    d->items[i].gnum = SWT_SKED;
+    d->items[i].dlen = sizeof(int);
+    d->items[i].data = (char *)&asw_type;
+    d->items[i].msg = CTEXT(T_SKEDS);
+    d->items[i].wrap = 1;
+    
+    d->items[++i].type = D_CHECKBOX;
+    d->items[i].gid = 1;
+    d->items[i].gnum = SWT_TALK;
+    d->items[i].dlen = sizeof(int);
+    d->items[i].data = (char *)&asw_type;
+    d->items[i].msg = CTEXT(T_TALK);
+    d->items[i].wrap = 1;
+    
+    d->items[++i].type = D_CHECKBOX;
+    d->items[i].gid = 1;
+    d->items[i].gnum = SWT_DXC;
+    d->items[i].dlen = sizeof(int);
+    d->items[i].data = (char *)&asw_type;
+    d->items[i].msg = CTEXT(T_DXC);
+    d->items[i].wrap = 1;
+    
+    d->items[++i].type = D_CHECKBOX; 
+    d->items[i].gid = 1;
+    d->items[i].gnum = SWT_UNFI;
+    d->items[i].dlen = sizeof(int);
+    d->items[i].data = (char *)&asw_type;
+    d->items[i].msg = CTEXT(T_UNFI);
+    d->items[i].wrap = 1;
+    
+    d->items[++i].type = D_CHECKBOX;
+    d->items[i].gid = 1;
+    d->items[i].gnum = SWT_STAT;
+    d->items[i].dlen = sizeof(int);
+    d->items[i].data = (char *)&asw_type;
+    d->items[i].msg = CTEXT(T_STAT);
+    d->items[i].wrap = 1;
+    
+    d->items[++i].type = D_CHECKBOX;
+    d->items[i].gid = 1;
+    d->items[i].gnum = SWT_MAP;
+    d->items[i].dlen = sizeof(int);
+    d->items[i].data = (char *)&asw_type;
+    d->items[i].msg = CTEXT(T_MAP);
+    d->items[i].wrap = 1;
+    
+    d->items[++i].type = D_CHECKBOX;
+    d->items[i].gid = 1;
+    d->items[i].gnum = SWT_SCOPE;
+    d->items[i].dlen = sizeof(int);
+    d->items[i].data = (char *)&asw_type;
+    d->items[i].msg = CTEXT(T_SCOPE);
+    d->items[i].wrap = 1;
+    
+    d->items[++i].type = D_CHECKBOX;
+    d->items[i].gid = 1;
+    d->items[i].gnum = SWT_SDR;
+    d->items[i].dlen = sizeof(int);
+    d->items[i].data = (char *)&asw_type;
+    d->items[i].msg = CTEXT(T_SDR);
+    d->items[i].wrap = 1;
+    
     /*d->items[++i].type = D_CHECKBOX;
     d->items[i].gid = 1;
     d->items[i].gnum = SWT_SWAP;
@@ -922,18 +940,23 @@ void menu_add_subwin(cba_t cba){
     d->items[i].data = (void *)&asw_type;
     d->items[i].msg = CTEXT(T_SWAP);
     d->items[i].wrap = 1;  */
-    
-    d->items[++i].type = D_FIELD;
-    d->items[i].dlen = MAX_STR_LEN;
-    d->items[i].data = asw_cmd;
-    d->items[i].msg = CTEXT(T_CMD);
-    d->items[i].wrap = 2;
+
+    if (!change){
+		d->items[++i].type = D_FIELD;
+		d->items[i].dlen = MAX_STR_LEN;
+		d->items[i].data = asw_cmd;
+		d->items[i].msg = CTEXT(T_CMD);
+		d->items[i].wrap = 1;
+	}
+
     
     /*d->items[++i].type = D_FIELD;
     d->items[i].dlen = MAX_STR_LEN;
     d->items[i].data = asw_respawn_str;
     d->items[i].msg = CTEXT(T_RESPAWN);
     d->items[i].wrap = 1; */
+
+	d->items[i].wrap++;
     
     d->items[++i].type = D_BUTTON;
     d->items[i].gid = B_ENTER;
@@ -954,13 +977,15 @@ void menu_add_subwin(cba_t cba){
 
 /********************* CLOSE SUBWIN ***********************/
 
-void menu_close_subwin(cba_t cba){
+void menu_close_subwin(void *arg){
     struct subwin *sw;
     
 
     sw = find_sw_ontop();
     if (!sw) return;
 
+	g_ptr_array_remove(gses->subwins, sw);
+
     free_subwin(sw);
 	if (sw == gses->ontop2)
 		sw_totop_next(0, 1);
@@ -971,14 +996,14 @@ void menu_close_subwin(cba_t cba){
 
 /********************* SPLIT SUBWIN ***********************/
 
-void menu_split_subwin(cba_t cba){
+void menu_split_subwin(void *arg){
 	cfg->splitheight = 10;
-	resize_terminal(CBA0);
+	resize_terminal(NULL);
 }
 
-void menu_remove_split(cba_t cba){
+void menu_remove_split(void *arg){
 	cfg->splitheight = 0;
-	resize_terminal(CBA0);
+	resize_terminal(NULL);
 }
 
 /******************* UNFINISHED REMARK *************/
@@ -1016,8 +1041,8 @@ void do_unfinished(struct session *ses, char *remark){
         g_string_sprintfa(gs, "    %-8s %7d %3d", TMPQ.locator, (int)TMPQ.qrb, TMPQ.qtf);
     }
 #ifdef HAVE_HAMLIB
-    if (trig){
-        g_string_sprintfa(gs, "  %1.1f", trig->qrg/1000.0);
+	if (gtrigs->trigs->len > 0){
+        g_string_sprintfa(gs, "  %1.1f", gtrigs->qrg/1000.0);
     }
 #endif
 
@@ -1037,7 +1062,7 @@ void do_unfinished(struct session *ses, char *remark){
 }
 
     
-void menu_unfinished(cba_t cba){
+void menu_unfinished(void *arg){
     static char remark[40];
     int max;
 
