@@ -1,6 +1,6 @@
 /*
     Tucnak - VHF contest log
-    Copyright (C) 2002-2006  Ladislav Vaiz <ok1zia@nagano.cz>
+    Copyright (C) 2002-2015  Ladislav Vaiz <ok1zia@nagano.cz>
 
     This program is free software; you can redistribute it and/or
     modify it under the terms of the GNU General Public License
@@ -10,12 +10,24 @@
 
 #include "header.h"
 
+#include "bfu.h"
+#include "fifo.h"
+#include "kbd.h"
+#include "menu.h"
+#include "qsodb.h"
+#include "rc.h"
+#include "rotar.h"
+#include "tsdl.h"
+#include "terminal.h"
+#include "trig.h"
+
 /******************** MISC OPTIONS *******************************/
 
 
-char loglines_str[MAX_STR_LEN],skedshift_str[MAX_STR_LEN],skedcount_str[MAX_STR_LEN],startbandchar_str[MAX_STR_LEN];
+char loglines_str[MAX_STR_LEN],skedcount_str[MAX_STR_LEN],startbandchar_str[MAX_STR_LEN];
 char gfx_x_ch_str[MAX_STR_LEN],gfx_y_ch_str[MAX_STR_LEN];
 char gfx_x_px_str[MAX_STR_LEN],gfx_y_px_str[MAX_STR_LEN];
+int fullscreen, usetouch, altsyms, nolocks, portrait, reverse;
 char fontheight_str[EQSO_LEN];
 char slashkey[3];
 int dssaver;
@@ -24,10 +36,12 @@ void refresh_misc_opts(void *xxx)
 {
     struct config_band *confb;
     int gfx_x_ch, gfx_y_ch, gfx_x_px, gfx_y_px;
-    int new_x, new_y, resize;
+    int new_w, new_h, resize, new_fh;
+
+	resize=0;
+	if (cfg->loglines != atoi(loglines_str)) resize++;
 
     STORE_SINT(cfg,loglines);
-    STORE_SINT(cfg,skedshift);
     STORE_SINT(cfg,skedcount);
 
     confb=get_config_band_by_bandchar(startbandchar_str[0]);
@@ -41,70 +55,91 @@ void refresh_misc_opts(void *xxx)
     gfx_y_px=atoi(gfx_y_px_str);
     STORE_STR(cfg, slashkey);
     STORE_INT(cfg, dssaver);
-    
-    new_x=800;
-    new_y=600;
-    resize=0;
-    
-    if (atoi(fontheight_str)!=cfg->fontheight) {
-        cfg->fontheight = atoi(fontheight_str);
-        resize++;
+	STORE_INT(cfg, usetouch);
+	STORE_INT(cfg, altsyms);
+	STORE_INT(cfg, nolocks);
+    if (portrait != cfg->portrait || reverse != cfg->reverse){
+#ifdef Z_ANDROID
+        log_addf(VTEXT(T_SAVE_RESTART));
+#endif
+	    STORE_INT(cfg, portrait);
+	    STORE_INT(cfg, reverse);
     }
+    
+    new_w=800;
+    new_h=600;
 
-#ifdef HAVE_SDL    
+    new_fh = atoi(fontheight_str);
+
+#ifdef Z_HAVE_SDL    
     if (sdl){
+		if (new_fh !=cfg->fontheight) {
+			cfg->fontheight = new_fh;
+			new_w = sdl->screen->w;
+			new_h = sdl->screen->h;
+			resize++;
+		}
+		if (fullscreen != cfg->fullscreen){
+			cfg->fullscreen = fullscreen;
+			new_w = sdl->screen->w;
+			new_h = sdl->screen->h;
+			resize++;
+		}
         if (gfx_x_ch>10 && gfx_y_ch>10){
-            if (gfx_x_ch*sdl->font_w!=cfg->gfx_x ||
-                gfx_y_ch*sdl->font_h!=cfg->gfx_y){
-                new_x=gfx_x_ch*sdl->font_w;
-                new_y=gfx_y_ch*sdl->font_h;
+            if (gfx_x_ch * zsdl->font_w != cfg->gfx_x ||
+                gfx_y_ch * zsdl->font_h != cfg->gfx_y){
+                new_w = gfx_x_ch * zsdl->font_w;
+                new_h = gfx_y_ch * zsdl->font_h;
                 resize++;
             }
         }
-        if (gfx_x_px>100 && gfx_y_px>100){
-            if (gfx_x_px!=cfg->gfx_x ||
-                gfx_y_px!=cfg->gfx_y){
-                new_x=gfx_x_px;
-                new_y=gfx_y_px;
+        if (gfx_x_px > 100 && gfx_y_px > 100){
+            if (gfx_x_px != cfg->gfx_x ||
+                gfx_y_px != cfg->gfx_y){
+                new_w = gfx_x_px;
+                new_h = gfx_y_px;
                 resize++;
             }
         }
 
-        if (resize && new_x<=2048 && new_y<=2048) {
-            cfg->gfx_x=new_x;
-            cfg->gfx_y=new_y;
-            sdl_setvideomode(new_x, new_y, 0);
-            resize_terminal(CBA0);
+        if (resize && new_w <= 2048 && new_h <= 2048) {
+            cfg->gfx_x = new_w;
+            cfg->gfx_y = new_h;
+            sdl_setvideomode(new_w, new_h, 0);
+            resize_terminal(NULL);
         }
     }
-    
+    else
 #endif  
+    {
+        if (resize) resize_terminal(NULL);
+    }
+
 }
 
-void misc_opts(cba_t cba)
+void misc_opts(void *arg)
 {
     struct dialog *d;
     int i;
     struct config_band *confb;
 
     g_snprintf(loglines_str, MAX_STR_LEN, "%d", cfg->loglines);
-    g_snprintf(skedshift_str, MAX_STR_LEN, "%d", cfg->skedshift);
     g_snprintf(skedcount_str, MAX_STR_LEN, "%d", cfg->skedcount);
     
     confb=get_config_band_by_pband(cfg->startband);
     if (!confb) confb=(struct config_band *)g_ptr_array_index(cfg->bands,0);
-    startbandchar_str[0]=toupper(confb->bandchar);
+    startbandchar_str[0]=confb->bandchar;
     startbandchar_str[1]='\0';
 
 //    dbg("gfx_x=%d gfx_x=%d %%=%d %d\n", cfg->gfx_x, cfg->gfx_y, cfg->gfx_x%sdl->font_w, cfg->gfx_y%font_h);
-#ifdef HAVE_SDL        
+#ifdef Z_HAVE_SDL        
     if (sdl){
-        if (cfg->gfx_x%sdl->font_w!=0 || cfg->gfx_y%sdl->font_h!=0){
+        if (cfg->gfx_x % zsdl->font_w!=0 || cfg->gfx_y % zsdl->font_h!=0){
             strcpy(gfx_x_ch_str, "");
             strcpy(gfx_y_ch_str, "");
         }else{
-            g_snprintf(gfx_x_ch_str, MAX_STR_LEN, "%d", cfg->gfx_x/sdl->font_w);
-            g_snprintf(gfx_y_ch_str, MAX_STR_LEN, "%d", cfg->gfx_y/sdl->font_h);
+            g_snprintf(gfx_x_ch_str, MAX_STR_LEN, "%d", cfg->gfx_x/zsdl->font_w);
+            g_snprintf(gfx_y_ch_str, MAX_STR_LEN, "%d", cfg->gfx_y/zsdl->font_h);
         }
     }
     else
@@ -119,46 +154,86 @@ void misc_opts(cba_t cba)
     g_snprintf(fontheight_str, EQSO_LEN, "%d", cfg->fontheight);
     safe_strncpy0(slashkey, cfg->slashkey, sizeof(slashkey));
     dssaver = cfg->dssaver;
+	fullscreen = cfg->fullscreen;
+	usetouch = cfg->usetouch;
+	altsyms = cfg->altsyms;
+	nolocks = cfg->nolocks;
+	portrait = cfg->portrait;
+	reverse = cfg->reverse;
 
 
-    if (!(d = mem_alloc(sizeof(struct dialog) + 20 * sizeof(struct dialog_item)))) return;
-    memset(d, 0, sizeof(struct dialog) + 20 * sizeof(struct dialog_item));
+    d = g_new0(struct dialog, 20);
     d->title = VTEXT(T_MISCOPTS);
     d->fn = dlg_pf_fn;
     d->refresh = (void (*)(void *))refresh_misc_opts;
     d->y0 = 1;
+	i = -1;
     
 
-    d->items[i=0].type = D_FIELD;
-    d->items[i].dlen = MAX_STR_LEN;
-    d->items[i].data = loglines_str;
-    d->items[i].maxl = 3;
-    d->items[i].fn   = check_number;
-    d->items[i].gid  = 1;
-    d->items[i].gnum = 30;
-    d->items[i].msg = CTEXT(T_LOGLINES);
-    d->items[i].wrap = 1;
+#ifdef Z_HAVE_SDL
+	if (sdl){
+		d->items[++i].type = D_FIELD;
+		d->items[i].gid  = 13;
+		d->items[i].gnum = 32;
+		d->items[i].maxl = 5;
+		d->items[i].dlen = EQSO_LEN;
+		d->items[i].data = fontheight_str;
+		d->items[i].msg = CTEXT(T_FONTHEIGHT);
+		d->items[i].wrap = 1;
     
-    d->items[++i].type = D_FIELD;
-    d->items[i].dlen = MAX_STR_LEN;
-    d->items[i].data = skedshift_str;
-    d->items[i].maxl = 3;
-    d->items[i].fn   = check_number;
-    d->items[i].gid  = 0;
-    d->items[i].gnum = 10;
-    d->items[i].msg = CTEXT(T_SKEDSHIFT);
-    d->items[i].wrap = 1;
+		d->items[++i].type = D_FIELD;
+		d->items[i].dlen = MAX_STR_LEN;
+		d->items[i].data = gfx_x_ch_str;
+		d->items[i].maxl = 5;
+		d->items[i].msg = CTEXT(T_GFX_X_CH);
     
-    d->items[++i].type = D_FIELD;
-    d->items[i].dlen = MAX_STR_LEN;
-    d->items[i].data = skedcount_str;
-    d->items[i].maxl = 3;
-    d->items[i].fn   = check_number;
-    d->items[i].gid  = 0;
-    d->items[i].gnum = 10;
-    d->items[i].msg = CTEXT(T_SKEDCOUNT);
-    d->items[i].wrap = 1;
+		d->items[++i].type = D_FIELD;
+		d->items[i].dlen = MAX_STR_LEN;
+		d->items[i].data = gfx_y_ch_str;
+		d->items[i].maxl = 5;
+		d->items[i].msg = CTEXT(T_GFX_Y_CH);
+		d->items[i].wrap = 1;
     
+		d->items[++i].type = D_FIELD;
+		d->items[i].dlen = MAX_STR_LEN;
+		d->items[i].data = gfx_x_px_str;
+		d->items[i].maxl = 5;
+		d->items[i].msg = CTEXT(T_GFX_X_PX);
+    
+		d->items[++i].type = D_FIELD;
+		d->items[i].dlen = MAX_STR_LEN;
+		d->items[i].data = gfx_y_px_str;
+		d->items[i].maxl = 5;
+		d->items[i].msg = CTEXT(T_GFX_Y_PX);
+		d->items[i].wrap = 1;
+
+		d->items[++i].type = D_CHECKBOX;
+		d->items[i].gid  = 0;
+		d->items[i].gnum = 1;
+		d->items[i].dlen = sizeof(int);
+		d->items[i].data = (char *)&fullscreen;
+		d->items[i].msg = CTEXT(T_FULLSCREEN);
+		d->items[i].wrap = 1;
+
+		d->items[++i].type = D_CHECKBOX;
+		d->items[i].gid  = 0;
+		d->items[i].gnum = 1;
+		d->items[i].dlen = sizeof(int);
+		d->items[i].data = (char *)&usetouch;
+		d->items[i].msg = CTEXT(T_TOUCH);
+		d->items[i].wrap = 1;
+        
+		d->items[++i].type = D_CHECKBOX;
+		d->items[i].gid  = 0;
+		d->items[i].gnum = 1;
+		d->items[i].dlen = sizeof(int);
+		d->items[i].data = (char *)&altsyms;
+		d->items[i].msg = CTEXT(T_ALT_QWER);
+		d->items[i].wrap = 1;
+
+	}
+#endif
+
     d->items[++i].type = D_FIELD;
     d->items[i].dlen = MAX_STR_LEN;
     d->items[i].data = startbandchar_str;
@@ -168,37 +243,23 @@ void misc_opts(cba_t cba)
     
     d->items[++i].type = D_FIELD;
     d->items[i].dlen = MAX_STR_LEN;
-    d->items[i].data = gfx_x_ch_str;
-    d->items[i].maxl = 5;
-    d->items[i].msg = CTEXT(T_GFX_X_CH);
-    
-    d->items[++i].type = D_FIELD;
-    d->items[i].dlen = MAX_STR_LEN;
-    d->items[i].data = gfx_y_ch_str;
-    d->items[i].maxl = 5;
-    d->items[i].msg = CTEXT(T_GFX_Y_CH);
+    d->items[i].data = loglines_str;
+    d->items[i].maxl = 3;
+    d->items[i].fn   = check_number;
+    d->items[i].gid  = 1;
+    d->items[i].gnum = 30;
+    d->items[i].msg = CTEXT(T_LOGLINES);
     d->items[i].wrap = 1;
     
+  
     d->items[++i].type = D_FIELD;
     d->items[i].dlen = MAX_STR_LEN;
-    d->items[i].data = gfx_x_px_str;
-    d->items[i].maxl = 5;
-    d->items[i].msg = CTEXT(T_GFX_X_PX);
-    
-    d->items[++i].type = D_FIELD;
-    d->items[i].dlen = MAX_STR_LEN;
-    d->items[i].data = gfx_y_px_str;
-    d->items[i].maxl = 5;
-    d->items[i].msg = CTEXT(T_GFX_Y_PX);
-    d->items[i].wrap = 1;
-    
-    d->items[++i].type = D_FIELD;
-    d->items[i].gid  = 16;
-    d->items[i].gnum = 32;
-    d->items[i].maxl = 5;
-    d->items[i].dlen = EQSO_LEN;
-    d->items[i].data = fontheight_str;
-    d->items[i].msg = CTEXT(T_FONTHEIGHT);
+    d->items[i].data = skedcount_str;
+    d->items[i].maxl = 3;
+    d->items[i].fn   = check_number;
+    d->items[i].gid  = 0;
+    d->items[i].gnum = 10;
+    d->items[i].msg = CTEXT(T_SKEDCOUNT);
     d->items[i].wrap = 1;
     
     d->items[++i].type = D_FIELD;
@@ -214,10 +275,38 @@ void misc_opts(cba_t cba)
     d->items[i].dlen = sizeof(int);
     d->items[i].data = (char *)&dssaver;
     d->items[i].msg = CTEXT(T_DSSAVER);
-    d->items[i].wrap = 2;
+	d->items[i].wrap = 1;
+
+//#ifdef Z_UNIX_ANDROID
+	d->items[++i].type = D_CHECKBOX;
+	d->items[i].gid  = 0;
+	d->items[i].gnum = 1;
+	d->items[i].dlen = sizeof(int);
+	d->items[i].data = (char *)&nolocks;
+	d->items[i].msg = CTEXT(T_NO_LOCKS);
+	d->items[i].wrap = 1;
+//#endif
     
+	d->items[++i].type = D_CHECKBOX;
+	d->items[i].gid  = 0;
+	d->items[i].gnum = 1;
+	d->items[i].dlen = sizeof(int);
+	d->items[i].data = (char *)&portrait;
+	d->items[i].msg = CTEXT(T_PORTRAIT_ORIENTATION);
+
+#ifdef Z_ANDROID
+	d->items[++i].type = D_CHECKBOX;
+	d->items[i].gid  = 0;
+	d->items[i].gnum = 1;
+	d->items[i].dlen = sizeof(int);
+	d->items[i].data = (char *)&reverse;
+	d->items[i].msg = CTEXT(T_REVERSE);
+#endif
+	d->items[i].wrap = 1;
     
-    d->items[++i].type = D_BUTTON;   /* 9 */
+
+	d->items[i].wrap++;
+    d->items[++i].type = D_BUTTON;   
     d->items[i].gid = B_ENTER;
     d->items[i].fn = ok_dialog;
     d->items[i].text = VTEXT(T_OK);
@@ -233,40 +322,67 @@ void misc_opts(cba_t cba)
     do_dialog(d, getml(d, NULL));
 }
 
-/*********************** FIXQRG - Change operator *************************/
+/*********************** FIXQRG - Fix rig frequency *************************/
 
 #ifdef HAVE_HAMLIB
-void fixqrg(struct session *ses, char *qrgstr){
+void fixqrg(void *xxx, char *qrgstr){
     gdouble newqrg;
+	int rignr = GPOINTER_TO_INT(xxx);
 
     if (!qrgstr) return;
 
-    
-    newqrg = atof(qrgstr);
+	     
+    //newqrg = atof(qrgstr);
+	newqrg = z_qrg_parse(qrgstr);
     if (newqrg<=0){
         errbox(CTEXT(T_BAD_QRG), 0);
         return;
     }
 
-    set_rig_lo(aband, get_rig_lo(aband) + newqrg - trig->qrg);
+    set_rig_lo(aband, rignr, get_rig_lo(aband, rignr) + newqrg - gtrigs->qrg);
     
-    trig_resend_freq(trig);
-//    redraw_later();
+    trigs_resend_freq(gtrigs, rignr);
 }
 
-void menu_fixqrg(struct terminal *term, void *d, struct session *ses){
-    static char qrgstr[EQSO_LEN];
+
+void do_fixqrg(void *itdata, void *menudata){
+    static char qrgstr[256];
+	//int rignr = GPOINTER_TO_INT(itdata);
     
-    if (!ctest || !aband || !trig) return;
-    
-    g_snprintf(qrgstr, EQSO_LEN, "%1.0f", trig->qrg);
+    //g_snprintf(qrgstr, EQSO_LEN, "%1.0f", gtrigs->qrg);
+	z_qrg_format(qrgstr, sizeof(qrgstr), gtrigs->qrg); 
 
     input_field(NULL, CTEXT(T_FIXQRG), CTEXT(T_ENTER_QRG) ,
-               CTEXT(T_OK), CTEXT(T_CANCEL), ses, 
+               CTEXT(T_OK), CTEXT(T_CANCEL), itdata, 
                NULL, 20, qrgstr, 0, 0, NULL,
-               (void (*)(void *, char *)) fixqrg, NULL, 0);
+               /*(void (*)(void *, char *))*/ fixqrg, NULL, 0);
 
 }
+
+void menu_fixqrg(void *itdata, void *menudata){
+	int i, enabled = 0;
+
+	for (i = 0; i < gtrigs->trigs->len; i++){
+		struct config_rig *crig = (struct config_rig *)g_ptr_array_index(cfg->crigs, i);
+		if (crig->rig_enabled) enabled++;
+	}
+
+	if (enabled > 1){
+		int i;
+		struct menu_item *mi = new_menu(3);
+		mi->rtext = VTEXT(T_CHOOSE_RIG);
+		for (i = 0; i < gtrigs->trigs->len; i++){
+			struct config_rig *crig = (struct config_rig *)g_ptr_array_index(cfg->crigs, i);
+			if (!crig->rig_enabled) continue;
+			add_to_menu(&mi, 
+				g_strdup_printf(VTEXT(T_RIG_D), i), "", "", MENU_FUNC do_fixqrg, GINT_TO_POINTER(i), 0);
+		}
+		set_window_ptr(gses->win, (term->x - strlen(mi->rtext) - 6) / 2, (term->y - 2 - gtrigs->trigs->len) / 2);
+		do_menu(mi, NULL);
+		return;
+	}
+	do_fixqrg(itdata, menudata);
+}
 #endif
 
 
@@ -349,7 +465,7 @@ void winkey_opts_fn(struct dialog_data *dlg)
     dlg_format_buttons(term, term, dlg->items +8, 2, dlg->x + DIALOG_LB, &y, w, NULL, AL_LEFT);
 }
 
-void winkey_opts(cba_t cba)
+void winkey_opts(void *arg)
 {
     struct dialog *d;
     int i;
@@ -361,7 +477,7 @@ void winkey_opts(cba_t cba)
     wk_swap = cfg->wk_swap;
 
 
-    if (!(d = mem_alloc(sizeof(struct dialog) + 20 * sizeof(struct dialog_item)))) return;
+    if (!(d = g_malloc(sizeof(struct dialog) + 20 * sizeof(struct dialog_item)))) return;
     memset(d, 0, sizeof(struct dialog) + 20 * sizeof(struct dialog_item));
     d->title = VTEXT(T_WINKEYOPTS); 
     d->fn = winkey_opts_fn;
@@ -431,15 +547,18 @@ void winkey_opts(cba_t cba)
 }
 
 int dlg_winkey_opts(struct dialog_data *dd, struct dialog_item_data *did){
-    winkey_opts(CBA0);
+    winkey_opts(NULL);
     return 0;
 }
 
 
 /************************** rig ******************************/
 #ifdef HAVE_HAMLIB
+int rig_enabled;
+char rig_desc[MAX_STR_LEN];
 char rig_filename[MAX_STR_LEN];
 char rig_model_str[EQSO_LEN];
+int rig_model;
 char rig_speed_str[EQSO_LEN];
 /* choose manufacturer */
 char rig_civaddr_str[EQSO_LEN];
@@ -447,44 +566,48 @@ char rig_ssbcw_shift_str[EQSO_LEN];
 char rig_lo_str[MAX_STR_LEN];
 //int rig_poll_ms;
 char rig_poll_ms_str[MAX_STR_LEN];
-int rig_qrg_r2t, rig_qrg_t2r, rig_mode_t2r, rig_clr_rit, rig_verbose;
+int rig_qrg_r2t, rig_qrg_t2r, rig_mode_t2r, rig_clr_rit, rig_ptt_t2r, rig_verbose;
 
 char new_model_str[EQSO_LEN];
 
-void set_rig (cba_t cba){
+void set_rig (void *arg){
     struct rig_caps *mcaps;
+    int active;
     
-    if (cba.int_<0 || cba.int_>=riglist->len) return;
-    mcaps = (struct rig_caps *)g_index_array_index(riglist, cba.int_);
+    active = GPOINTER_TO_INT(arg);
+    if (active < 0 || active >= riglist->len) return;
+    mcaps = (struct rig_caps *)z_ptr_array_index(riglist, active);
     
-    dbg("%4d %4d %-20s %s\n", cba.int_, mcaps->rig_model, mcaps->mfg_name, mcaps->model_name);
+    dbg("%4d %4d %-20s %s\n", active, mcaps->rig_model, mcaps->mfg_name, mcaps->model_name);
     g_snprintf(new_model_str, EQSO_LEN, "%d", mcaps->rig_model);
     dbg("set_rig: new_model_str = '%s'\n", new_model_str);
     redraw_later();
 }
     
-void choose_rig (cba_t cba){
+void choose_rig (void *arg){
     struct rig_caps *mcaps;
     int i, sel=0, model;
     struct menu_item *mi;
+    int active;
     
-    if (cba.int_<0 || cba.int_>=riglist->len) return;
-    mcaps = (struct rig_caps *)g_index_array_index(riglist, cba.int_);
+    active = GPOINTER_TO_INT(arg);
+    if (active < 0 || active >= riglist->len) return;
+    mcaps = (struct rig_caps *)z_ptr_array_index(riglist, active);
     
-    dbg("%4d %4d %-20s %s\n", cba.int_, mcaps->rig_model, mcaps->mfg_name, mcaps->model_name);
+    dbg("%4d %4d %-20s %s\n", active, mcaps->rig_model, mcaps->mfg_name, mcaps->model_name);
     
     if (!(mi = new_menu(1))) return;
 
-    model=cfg->rig_model;
-    for (i=cba.int_; i<riglist->len; i++){
-        struct rig_caps *caps = (struct rig_caps *)g_index_array_index(riglist, i);
+    model = GPOINTER_TO_INT(arg);
+    for (i=active; i<riglist->len; i++){
+        struct rig_caps *caps = (struct rig_caps *)z_ptr_array_index(riglist, i);
         if (strcasecmp(mcaps->mfg_name, caps->mfg_name)!=0) break;
-        if (model==caps->rig_model) sel = i - cba.int_;
-        add_to_menu(&mi, (char *)caps->model_name, "", "", MENU_FUNC set_rig, (cba_t)i, 0);
+        if (model==caps->rig_model) sel = i - active;
+        add_to_menu(&mi, (char *)caps->model_name, "", "", MENU_FUNC set_rig, GINT_TO_POINTER(i), 0);
     }
     
     dbg("sel=%d\n", sel);
-    do_menu_selected(mi, CBA0, sel);
+    do_menu_selected(mi, NULL, sel);
     
 }
 
@@ -495,10 +618,10 @@ int choose_manufacturer(struct dialog_data *dlg, struct dialog_item_data *di){
     
     if (!(mi = new_menu(1))) return 0;
     
-    model=cfg->rig_model;
+	model = GPOINTER_TO_INT(di->cdata);
     dbg("rig_model=%d\n", model);
     for (i=0; i<riglist->len; i++){
-        struct rig_caps *caps = (struct rig_caps *)g_index_array_index(riglist, i);
+        struct rig_caps *caps = (struct rig_caps *)z_ptr_array_index(riglist, i);
         same=0;
         if (old && strcasecmp(old, caps->mfg_name)==0) same=1;
         
@@ -509,80 +632,138 @@ int choose_manufacturer(struct dialog_data *dlg, struct dialog_item_data *di){
         }
         if (same) continue;
         
-        add_to_menu(&mi, (char *)caps->mfg_name, "", "", MENU_FUNC choose_rig, (cba_t)i, 0);
+        add_to_menu(&mi, (char *)caps->mfg_name, "", "", MENU_FUNC choose_rig, GINT_TO_POINTER(i), 0);
         old = caps->mfg_name;
         j++;
     }
     dbg("sel=%d\n", sel);
-    do_menu_selected(mi, CBA0, sel);
+    do_menu_selected(mi, di->cdata, sel);
     return 0;
 }
 
 void refresh_rig_opts(void *xxx){
-    STORE_STR(cfg, rig_filename);
-    STORE_SINT(cfg, rig_model);
-    STORE_SINT(cfg, rig_speed);
-    STORE_SINT(cfg, rig_civaddr);
-    STORE_SINT(cfg, rig_ssbcw_shift);
-    STORE_SDBL(cfg, rig_lo);
-    STORE_SINT(cfg, rig_poll_ms);
-    STORE_INT(cfg, rig_qrg_r2t);
-    STORE_INT(cfg, rig_qrg_t2r);
-    STORE_INT(cfg, rig_mode_t2r);
-    STORE_INT(cfg, rig_clr_rit);
-    STORE_INT(cfg, rig_verbose);
-    free_trig(trig);
-    trig = init_trig();        
+    struct config_rig *crig = (struct config_rig *)xxx;
+
+    if (!crig){
+        crig = g_new0(struct config_rig, 1);
+        crig->nr = cfg->crigs->len;
+        g_ptr_array_add(cfg->crigs, crig);
+    }
+
+	STORE_INT(crig, rig_enabled);
+	STORE_STR(crig, rig_desc);
+    STORE_STR(crig, rig_filename);
+    STORE_SINT(crig, rig_model);
+    STORE_SINT(crig, rig_speed);
+	STORE_SHEX(crig, rig_civaddr);
+    STORE_SINT(crig, rig_ssbcw_shift);
+    STORE_SDBL(crig, rig_lo);
+    STORE_SINT(crig, rig_poll_ms);
+    STORE_INT(crig, rig_qrg_r2t);
+    STORE_INT(crig, rig_qrg_t2r);
+    STORE_INT(crig, rig_mode_t2r);
+    STORE_INT(crig, rig_clr_rit);
+    STORE_INT(crig, rig_ptt_t2r);
+    STORE_INT(crig, rig_verbose);
+
+    free_trigs(gtrigs);
+    gtrigs = init_trigs();        
+	progress(NULL);
 }
 
 void rig_opts_fn(struct dialog_data *dlg)
 {
     if (strlen(new_model_str)>0){
         dbg("new_model_str=%s\n", new_model_str);
-        safe_strncpy0(dlg->items[1].cdata, new_model_str, EQSO_LEN);
+        safe_strncpy0(dlg->items[3].cdata, new_model_str, EQSO_LEN);
         strcpy(new_model_str, "");
     }
 
 }
 
-void menu_rig_opts(cba_t cba)
+void menu_rig_opts(void *arg)
 {
     struct dialog *d;
-    int i;
+    int i, rigi;
+    struct config_rig *crig;
 
-    safe_strncpy0(rig_filename, cfg->rig_filename, MAX_STR_LEN);
-    g_snprintf(rig_model_str, EQSO_LEN, "%d", cfg->rig_model);
-    g_snprintf(rig_speed_str, EQSO_LEN, "%d", cfg->rig_speed);
-    g_snprintf(rig_civaddr_str, EQSO_LEN, "%d", cfg->rig_civaddr);
-    g_snprintf(rig_ssbcw_shift_str, EQSO_LEN, "%d", cfg->rig_ssbcw_shift);
-    g_snprintf(rig_lo_str, MAX_STR_LEN, "%1.1f", cfg->rig_lo);
-    g_snprintf(rig_poll_ms_str, MAX_STR_LEN, "%d", cfg->rig_poll_ms);
+    rigi = GPOINTER_TO_INT(arg);
+    dbg("menu_rig_opts(%d)\n", rigi);
+    if (rigi >= (int)cfg->crigs->len) return;
+
+    if (rigi < 0){
+        crig = NULL;
+		rig_enabled = 1;
+		strcpy(rig_desc, VTEXT(T_NEW_RIG));
+        strcpy(rig_filename, "");       
+        strcpy(rig_model_str, "1");         
+        strcpy(rig_speed_str, "0");     
+        strcpy(rig_civaddr_str, "0x58");   
+        strcpy(rig_ssbcw_shift_str, "0");
+        strcpy(rig_lo_str, "0");        
+        strcpy(rig_poll_ms_str, "10");   
+        rig_qrg_r2t = 1;
+        rig_qrg_t2r = 1;        
+        rig_mode_t2r = 1;       
+        rig_clr_rit = 0;       
+		rig_ptt_t2r = 0;
+        rig_verbose = 0;        
+        
+    }else{
+        crig = (struct config_rig *)g_ptr_array_index(cfg->crigs, rigi);
+        LOAD_INT(crig, rig_enabled);
+        LOAD_STR(crig, rig_desc);
+        LOAD_STR(crig, rig_filename);
+        LOAD_SINT(crig, rig_model);
+        LOAD_SINT(crig, rig_speed);
+		LOAD_SHEX2(crig, rig_civaddr);
+        LOAD_SINT(crig, rig_ssbcw_shift);
+        LOAD_SDBL(crig, rig_lo);
+        LOAD_SINT(crig, rig_poll_ms);
+        LOAD_INT(crig, rig_qrg_r2t);
+        LOAD_INT(crig, rig_qrg_t2r);
+        LOAD_INT(crig, rig_mode_t2r);
+        LOAD_INT(crig, rig_clr_rit);
+        LOAD_INT(crig, rig_ptt_t2r);
+        LOAD_INT(crig, rig_verbose);
+    }
 
-    rig_qrg_r2t = cfg->rig_qrg_r2t;
-    rig_qrg_t2r = cfg->rig_qrg_t2r;
-    rig_mode_t2r = cfg->rig_mode_t2r;
-    rig_clr_rit = cfg->rig_clr_rit;
-    rig_verbose = cfg->rig_verbose;
 
     strcpy(new_model_str, "");
     
 
-    if (!(d = mem_alloc(sizeof(struct dialog) + 20 * sizeof(struct dialog_item)))) return;
+    if (!(d = (struct dialog*)g_malloc(sizeof(struct dialog) + 20 * sizeof(struct dialog_item)))) return;
     memset(d, 0, sizeof(struct dialog) + 20 * sizeof(struct dialog_item));
     d->title = VTEXT(T_RIGOPTS); 
-//    d->fn = rig_opts_fn;
     d->fn = dlg_pf_fn;
     d->fn2 = rig_opts_fn;
     d->refresh = (void (*)(void *))refresh_rig_opts;
+    d->refresh_data= crig;
     d->y0 = 1;
+	i = -1;
     
-    d->items[i=0].type = D_FIELD;
+    d->items[++i].type = D_CHECKBOX;      
+    d->items[i].gid = 0;
+    d->items[i].gnum = 1;
+    d->items[i].dlen = sizeof(int);
+	d->items[i].data=(char *)&rig_enabled;
+    d->items[i].msg = CTEXT(T_ENABLED);//CTEXT(T_RIG_QRG_R2T);
+    d->items[i].wrap = 1;
+
+	d->items[++i].type = D_FIELD;
+    d->items[i].dlen = MAX_STR_LEN;
+    d->items[i].data = rig_desc;
+    d->items[i].maxl = 31;
+    d->items[i].msg  = CTEXT(T_DESC);//CTEXT(T_RIG_DEVICE);
+    d->items[i].wrap = 1;
+    
+    d->items[++i].type = D_FIELD;
     d->items[i].dlen = MAX_STR_LEN;
     d->items[i].data = rig_filename;
     d->items[i].maxl = 35;
     d->items[i].msg  = CTEXT(T_RIG_DEVICE);
     d->items[i].wrap = 1;
-    
+
     d->items[++i].type = D_FIELD;
     d->items[i].dlen = EQSO_LEN;
     d->items[i].data = rig_model_str;
@@ -592,9 +773,11 @@ void menu_rig_opts(cba_t cba)
     d->items[i].msg  = CTEXT(T_RIG_MODEL);
     d->items[i].gnum = 20000;
 
+	rig_model = atoi(rig_model_str);
     d->items[++i].type = D_BUTTON;   /* 2 */
     d->items[i].gid = 0;
     d->items[i].fn = choose_manufacturer;
+	d->items[i].data = (char *)GINT_TO_POINTER(rig_model);
     d->items[i].text = VTEXT(T_RIG_CHOOSE);
     d->items[i].wrap = 1;
     
@@ -612,10 +795,11 @@ void menu_rig_opts(cba_t cba)
     d->items[i].dlen = EQSO_LEN;
     d->items[i].data = rig_civaddr_str;
     d->items[i].maxl = 4;
-    d->items[i].fn   = check_number;
+    d->items[i].fn   = check_hex;
     d->items[i].gid  = 0;
     d->items[i].msg  = CTEXT(T_RIG_CIV);
     d->items[i].gnum = 255;
+    d->items[i].wrap = 1;
 
     d->items[++i].type = D_FIELD;
     d->items[i].dlen = EQSO_LEN;
@@ -670,6 +854,14 @@ void menu_rig_opts(cba_t cba)
     d->items[i].msg = CTEXT(T_RIG_MODE_T2R);
     d->items[i].wrap = 1;
 
+    d->items[++i].type = D_CHECKBOX;      
+    d->items[i].gid = 0;
+    d->items[i].gnum = 1;
+    d->items[i].dlen = sizeof(int);
+    d->items[i].data=(char *)&rig_ptt_t2r;
+    d->items[i].msg = CTEXT(T_SET_PTT_ON_RADIO);//CTEXT(T_RIG_CLR_RIT);
+    d->items[i].wrap = 1;
+
     d->items[++i].type = D_CHECKBOX;      
     d->items[i].gid = 0;
     d->items[i].gnum = 1;
@@ -702,6 +894,95 @@ void menu_rig_opts(cba_t cba)
     d->items[++i].type = D_END;
     do_dialog(d, getml(d, NULL));
 }
+
+void menu_rig_delete(void *arg){
+    struct config_rig *crig;
+	int rigi;
+
+	rigi = GPOINTER_TO_INT(arg);
+    dbg("menu_rig_delete(%d)\n", rigi);
+    if (rigi >= (int)cfg->crigs->len) return;
+    crig = (struct config_rig *)g_ptr_array_index(cfg->crigs, rigi);
+    free_config_rig(crig);
+    g_ptr_array_remove_index(cfg->crigs, rigi);
+	
+    free_trigs(gtrigs);
+    gtrigs = init_trigs();        
+
+}
+
+
+
+void menu_rigs_delete(void *arg)
+{
+    struct menu_item *mi = NULL;
+    static char *rotstrs[]={"0","1","2","3","4","5","6","7","8","9"};
+
+    int i;
+  
+    
+    if (!(mi = new_menu(3))) return;
+
+    for (i=0; i<cfg->crigs->len; i++){
+        struct config_rig *crig = (struct config_rig *)g_ptr_array_index(cfg->crigs, i);
+        
+        add_to_menu(&mi, 
+            //g_strdup(rot->rot_desc && *rot->rot_desc?rot->rot_desc:"Rotator"), 
+			g_strdup_printf("Rig #%d", crig->nr),
+            rotstrs[crig->nr],
+            rotstrs[crig->nr],
+            MENU_FUNC menu_rig_delete, GINT_TO_POINTER(i), 0);
+    }
+    do_menu(mi, NULL);
+}
+
+
+
+void menu_rigs(void *arg)
+{
+    struct menu_item *mi = NULL;
+    static char *rotstrs[]={"0","1","2","3","4","5","6","7","8","9"};
+    int i;
+    
+    
+    if (!(mi = new_menu(3))) return;
+
+    for (i=0; i<cfg->crigs->len; i++){
+        struct config_rig *crig = (struct config_rig *)g_ptr_array_index(cfg->crigs, i);
+		char *c;
+
+		if (crig->rig_desc && *crig->rig_desc) c = g_strdup(crig->rig_desc);
+		else c = g_strdup_printf(VTEXT(T_RIG_D), crig->nr);
+
+		if (!crig->rig_enabled){
+			char *d = c;
+			c = g_strconcat(c, VTEXT(T_DISABLED), NULL);
+			g_free(d);
+		}
+        
+        add_to_menu(&mi, 
+            //g_strdup(crig->rig_filename && *rot->rot_desc?rot->rot_desc:"Rotator"), 
+			c,
+            rotstrs[crig->nr],
+            rotstrs[crig->nr],
+            MENU_FUNC menu_rig_opts, GINT_TO_POINTER(i), 0);
+    }
+    add_to_menu(&mi, g_strdup(""), "", M_BAR, NULL, NULL, 0);
+    
+    add_to_menu(&mi, 
+        g_strdup(VTEXT(T_ADD_RIG)),
+        "A",
+        "A",
+        MENU_FUNC menu_rig_opts, GINT_TO_POINTER(-1), 0);
+    
+    add_to_menu(&mi, 
+        g_strdup(VTEXT(T_REMOVE_RIG)/*VTEXT(T_ROT_REMOVE)*/),
+        "R", //VTEXT(T_HK_ROT_REMOVE),
+        "R", //VTEXT(T_HK_ROT_REMOVE),
+        MENU_FUNC menu_rigs_delete, GINT_TO_POINTER(-1), 1);
+    do_menu(mi, NULL);
+}
+
 #endif
     
 
@@ -716,6 +997,7 @@ char rot_vid_str[EQSO_LEN];
 char rot_pid_str[EQSO_LEN];
 char rot_serial[EQSO_LEN];
 char rot_timeout_ms_str[EQSO_LEN];
+char rot_poll_ms_str[EQSO_LEN];
 char rot_beamwidth_str[EQSO_LEN];
 char rot_saddr_str[EQSO_LEN], rot_model_str[EQSO_LEN];
 char rot_rem_rotstr[EQSO_LEN];
@@ -727,11 +1009,10 @@ void refresh_rotar_opts(void *xxx){
         crot = g_new0(struct config_rotar, 1);
         crot->nr = cfg->crotars->len;
         g_ptr_array_add(cfg->crotars, crot);
-        
     }
 
     STORE_STR (crot, rot_desc);
-    STORE_INT (crot, rot_type);
+    STORE_ENUM(crot, rot_type, enum rot_type);
     STORE_STR (crot, rot_filename); 
     STORE_STR (crot, rot_hostname); 
     STORE_SINT(crot, rot_port);
@@ -739,6 +1020,7 @@ void refresh_rotar_opts(void *xxx){
     STORE_SHEX(crot, rot_pid);
     STORE_STR (crot, rot_serial);
     STORE_SINT(crot, rot_timeout_ms);
+    STORE_SINT(crot, rot_poll_ms);
     STORE_SINT(crot, rot_beamwidth);
     STORE_SINT(crot, rot_model);
     STORE_SINT(crot, rot_saddr);
@@ -746,15 +1028,16 @@ void refresh_rotar_opts(void *xxx){
 
     free_rotars();
     init_rotars();
+	progress(NULL);
 }
         
 
-void menu_rotar_opts(cba_t cba){
+void menu_rotar_opts(void *arg){
     struct dialog *d;
     int i, roti;
     struct config_rotar *crot;
 
-	roti = (int)GETCBA(cba, int_);
+	roti = GPOINTER_TO_INT(arg);
     dbg("menu_rotar_opts(%d)\n", roti);
     if (roti >= (int)cfg->crotars->len) return;
 
@@ -771,7 +1054,8 @@ void menu_rotar_opts(cba_t cba){
         strcpy(rot_pid_str, "e112");
         strcpy(rot_serial, "");
         strcpy(rot_timeout_ms_str,"400");
-        strcpy(rot_beamwidth_str, "10");
+        strcpy(rot_poll_ms_str,"500");
+        strcpy(rot_beamwidth_str, "20");
         strcpy(rot_saddr_str, "240");
         strcpy(rot_model_str, "0");
         strcpy(rot_rem_rotstr, "A");
@@ -787,13 +1071,14 @@ void menu_rotar_opts(cba_t cba){
         LOAD_SHEX4(crot, rot_pid);
         LOAD_STR  (crot, rot_serial); 
         LOAD_SINT (crot, rot_timeout_ms);
+        LOAD_SINT (crot, rot_poll_ms);
         LOAD_SINT (crot, rot_beamwidth);
         LOAD_SINT (crot, rot_saddr);
         LOAD_SINT (crot, rot_model);
         LOAD_STR  (crot, rot_rem_rotstr); 
     }
 
-    if (!(d = mem_alloc(sizeof(struct dialog) + 20 * sizeof(struct dialog_item)))) return;
+    if (!(d = (struct dialog *)g_malloc(sizeof(struct dialog) + 20 * sizeof(struct dialog_item)))) return;
     memset(d, 0, sizeof(struct dialog) + 20 * sizeof(struct dialog_item));
     d->title = VTEXT(T_ROTAROPTS);
     d->fn = dlg_pf_fn;
@@ -901,6 +1186,16 @@ void menu_rotar_opts(cba_t cba){
     d->items[i].maxl = 5;
     d->items[i].msg = CTEXT(T_TIMEOUT);
 
+    d->items[++i].type = D_FIELD;
+    d->items[i].dlen = EQSO_LEN;
+    d->items[i].data = rot_poll_ms_str;
+    d->items[i].fn   = check_number;
+    d->items[i].gid  = 1;
+    d->items[i].gnum = 2000;
+    d->items[i].maxl = 5;
+	d->items[i].msg = CTEXT(T_ROT_POLL_MS);
+    d->items[i].wrap = 1;
+
     d->items[++i].type = D_FIELD;
     d->items[i].dlen = EQSO_LEN;
     d->items[i].data = rot_beamwidth_str;
@@ -956,19 +1251,20 @@ void menu_rotar_opts(cba_t cba){
     do_dialog(d, getml(d, NULL));
 }
 
-void menu_rotar_delete(cba_t cba){
+void menu_rotar_delete(void *arg){
     struct config_rotar *crot;
 	int roti;
 
-	roti = (int)GETCBA(cba, int_);
+	roti = GPOINTER_TO_INT(arg);
     dbg("menu_rotar_delete(%d)\n", roti);
     if (roti >= (int)cfg->crotars->len) return;
-    crot = g_ptr_array_index(cfg->crotars, roti);
+    crot = (struct config_rotar *)g_ptr_array_index(cfg->crotars, roti);
     free_config_rotar(crot);
     g_ptr_array_remove_index(cfg->crotars, roti);
+
 }
 
-void menu_rotars_delete(cba_t cba)
+void menu_rotars_delete(void *arg)
 {
     struct menu_item *mi = NULL;
     static char *rotstrs[]={"A","B","C","D","E","F","G","H","I","J","K","L","M","N","O","P","Q","R","S","T","U","V","W","X","Y","Z"};
@@ -981,15 +1277,15 @@ void menu_rotars_delete(cba_t cba)
         struct config_rotar *rot = (struct config_rotar *)g_ptr_array_index(cfg->crotars, i);
         
         add_to_menu(&mi, 
-            stracpy(rot->rot_desc && *rot->rot_desc?rot->rot_desc:"Rotator"), 
+            g_strdup(rot->rot_desc && *rot->rot_desc?rot->rot_desc:"Rotator"), 
             rotstrs[rot->nr],
             rotstrs[rot->nr],
-            MENU_FUNC menu_rotar_delete, (cba_t)i, 0);
+            MENU_FUNC menu_rotar_delete, GINT_TO_POINTER(i), 0);
     }
-    do_menu(mi, CBA0);
+    do_menu(mi, NULL);
 }
 
-void menu_rotars(cba_t cba)
+void menu_rotars(void *arg)
 {
     struct menu_item *mi = NULL;
     static char *rotstrs[]={"A","B","C","D","E","F","G","H","I","J","K","L","M","N","O","P","Q","R","S","T","U","V","W","X","Y","Z"};
@@ -1000,25 +1296,40 @@ void menu_rotars(cba_t cba)
 
     for (i=0; i<cfg->crotars->len; i++){
         struct config_rotar *rot = (struct config_rotar *)g_ptr_array_index(cfg->crotars, i);
+        char *c;
+
+        if (rot->rot_desc && *rot->rot_desc)
+            c = g_strdup(rot->rot_desc);
+        else 
+            c = g_strdup_printf(VTEXT(T_ROTATOR_D), i);
+
+        if (rot->rot_type == ROT_NONE){
+			char *d = c;
+			c = g_strconcat(c, VTEXT(T_DISABLED), NULL);
+			g_free(d);
+        }
+        
         
         add_to_menu(&mi, 
-            stracpy(rot->rot_desc && *rot->rot_desc?rot->rot_desc:"Rotator"), 
+            c, 
             rotstrs[rot->nr],
             rotstrs[rot->nr],
-            MENU_FUNC menu_rotar_opts, (cba_t)i, 0);
+            MENU_FUNC menu_rotar_opts, GINT_TO_POINTER(i), 0);
     }
-    add_to_menu(&mi, stracpy(""), "", M_BAR, NULL, CBA0, 0);
+    add_to_menu(&mi, g_strdup(""), "", M_BAR, NULL, NULL, 0);
     
     add_to_menu(&mi, 
-        stracpy(VTEXT(T_ROT_ADD)),
+        g_strdup(VTEXT(T_ROT_ADD)),
         VTEXT(T_HK_ROT_ADD),
         VTEXT(T_HK_ROT_ADD),
-        MENU_FUNC menu_rotar_opts, (cba_t)-1, 0);
+        MENU_FUNC menu_rotar_opts, GINT_TO_POINTER(-1), 0);
     
     add_to_menu(&mi, 
-        stracpy(VTEXT(T_ROT_REMOVE)),
+        g_strdup(VTEXT(T_ROT_REMOVE)),
         VTEXT(T_HK_ROT_REMOVE),
         VTEXT(T_HK_ROT_REMOVE),
-        MENU_FUNC menu_rotars_delete, (cba_t)-1, 1);
-    do_menu(mi, CBA0);
+        MENU_FUNC menu_rotars_delete, GINT_TO_POINTER(-1), 1);
+
+	do_menu(mi, NULL);
 }
+
