@@ -288,6 +288,7 @@ char ac_minelev_str[DBL_LEN];
 char ac_maxelev_str[DBL_LEN];   
 char ac_mindur_str[DBL_LEN];       
 char ac_maxdelta_str[DBL_LEN];  
+char ac_minalt_str[DBL_LEN];
 
 void refresh_ac_opts(void *arg){
     STORE_INT(cfg, ac_enable);
@@ -298,6 +299,7 @@ void refresh_ac_opts(void *arg){
     STORE_SDBL(cfg, ac_maxelev);
     STORE_SINT(cfg, ac_mindur);
     STORE_SDBL(cfg, ac_maxdelta);
+    STORE_SINT(cfg, ac_minalt);
 
 
 	free_acs(gacs);
@@ -317,6 +319,7 @@ void menu_ac_opts(void *arg){
     g_snprintf(ac_maxelev_str, DBL_LEN, "%3.1f", cfg->ac_maxelev);
     g_snprintf(ac_mindur_str,  DBL_LEN, "%d", cfg->ac_mindur);
     g_snprintf(ac_maxdelta_str, DBL_LEN, "%3.1f", cfg->ac_maxdelta);
+    g_snprintf(ac_minalt_str, DBL_LEN, "%d", cfg->ac_minalt);
 
 
     d = (struct dialog *)g_malloc(sizeof(struct dialog) + 20 * sizeof(struct dialog_item));
@@ -372,13 +375,22 @@ void menu_ac_opts(void *arg){
     d->items[i].maxl = DBL_LEN - 11;
     d->items[i].data = (char *)&ac_mindur_str;
     d->items[i].msg = CTEXT(T_AC_MIN_DURATION);
-    /*d->items[i].wrap = 1;
+    d->items[i].wrap = 1;
     
-    d->items[++i].type = D_FIELD;      
+    /*d->items[++i].type = D_FIELD;      
     d->items[i].dlen = DBL_LEN;
     d->items[i].data = (char *)&ac_maxdelta_str;
     d->items[i].msg = CTEXT(T_AC_MAX_DIFF);
-    */d->items[i].wrap = 2;
+    d->items[i].wrap = 1;*/
+    
+    d->items[++i].type = D_FIELD;      
+    d->items[i].dlen = 6;
+    d->items[i].data = (char *)&ac_minalt_str;
+    d->items[i].fn   = check_number;
+    d->items[i].gid  = 0;
+    d->items[i].gnum = 20000;
+    d->items[i].msg = TRANSLATE("Minimal AC altitude [m]");
+    d->items[i].wrap = 2;
 
     
     d->items[++i].type = D_BUTTON;
