@@ -1,6 +1,6 @@
 /*
     Tucnak - VHF contest log
-    Copyright (C) 2002-2006  Ladislav Vaiz <ok1zia@nagano.cz>
+    Copyright (C) 2002-2015  Ladislav Vaiz <ok1zia@nagano.cz>
 
     This program is free software; you can redistribute it and/or
     modify it under the terms of the GNU General Public License
@@ -10,6 +10,12 @@
 
 #include "header.h"
 
+#include "cwdaemon.h"
+#include "fifo.h"
+#include "language2.h"
+#include "ppdev.h" 
+#include "rc.h"
+
 #ifdef HAVE_LINUX_PPDEV_H
 
 int parport_init(struct cwdaemon *cwda){
@@ -29,7 +35,7 @@ int parport_open(struct cwdaemon *cwda, int verbose){
 
     cwda->fd = open(cfg->cwda_device, O_RDWR|O_NONBLOCK);
     if (cwda->fd<0){
-        if (verbose) log_addf("Can't open %s %s", cfg->cwda_device, strerror_r(errno, errbuf, sizeof(errbuf)));
+        if (verbose) log_addf(VTEXT(T_CANT_OPEN), cfg->cwda_device, strerror_r(errno, errbuf, sizeof(errbuf)));
         if (errno==ENOENT) {
             int len, devno;
             len=strcspn(cfg->cwda_device, "01234567890");
@@ -37,10 +43,10 @@ int parport_open(struct cwdaemon *cwda, int verbose){
                 devno=atoi(cfg->cwda_device+len);
             else
                 devno=0;
-            if (verbose) log_addf("Tip: try to run as root \"mknod %s c 99 %d; chmod a+rw %s\"", cfg->cwda_device, devno, cfg->cwda_device); 
+            if (verbose) log_addf(VTEXT(T_TIP_MKNOD), cfg->cwda_device, devno, cfg->cwda_device); 
         }
         if (errno==EACCES && verbose) {
-            log_addf("Tip: try to run as root \"chmod a+rw %s\"", cfg->cwda_device); 
+            log_addf(VTEXT(T_TIP_CHMOD), cfg->cwda_device); 
         }
         close(cwda->fd);
         cwda->fd=-1;
@@ -49,24 +55,24 @@ int parport_open(struct cwdaemon *cwda, int verbose){
 
     mode=PARPORT_MODE_PCSPP;
     if (ioctl(cwda->fd, PPSETMODE, &mode)<0){
-        if (verbose) log_addf("Can't set %s to EPP mode %s", cfg->cwda_device, cfg->cwda_device, errno, strerror_r(errno, errbuf, sizeof(errbuf)));
+        if (verbose) log_addf(VTEXT(T_CANT_SET_EPP), cfg->cwda_device, cfg->cwda_device, errno, strerror_r(errno, errbuf, sizeof(errbuf)));
         close(cwda->fd);
         cwda->fd=-1;
     return -1;
     }
 
     if (ioctl(cwda->fd, PPEXCL, NULL)<0){
-        if (verbose) log_addf("Can't exclusive access to %s %s", cfg->cwda_device, cfg->cwda_device, errno, strerror_r(errno, errbuf, sizeof(errbuf)));
+        if (verbose) log_addf(VTEXT(T_CANT_EXCLUSIVE_ACCESS), cfg->cwda_device, cfg->cwda_device, errno, strerror_r(errno, errbuf, sizeof(errbuf)));
         close(cwda->fd);
         cwda->fd=-1;
         return -1;
     }
 
     if (ioctl(cwda->fd, PPCLAIM, NULL)<0){
-        if (verbose) log_addf("Can't claim %s %s", cfg->cwda_device, strerror_r(errno, errbuf, sizeof(errbuf)));
+        if (verbose) log_addf(VTEXT(T_CANT_CLAIM), cfg->cwda_device, strerror_r(errno, errbuf, sizeof(errbuf)));
         if (errno==ENXIO && verbose){
-            log_addf("Tip: try to run as root \"rmmod lp\"");
-            log_addf("Tip: or kill process listed in \"lsof %s\"", cfg->cwda_device);
+            log_addf(VTEXT(T_TIP_RMMOD));
+            log_addf(VTEXT(T_TIP_KILL_LSOF), cfg->cwda_device);
         }
         close(cwda->fd);
         cwda->fd=-1;
@@ -77,7 +83,7 @@ int parport_open(struct cwdaemon *cwda, int verbose){
 	frob.mask = PARPORT_CONTROL_STROBE;
 	frob.val = PARPORT_CONTROL_STROBE;
 	if (ioctl(cwda->fd, PPFCONTROL, &frob)<0){
-        if (verbose) log_addf("Can't set parport_STROBE %s", strerror_r(errno, errbuf, sizeof(errbuf))); 
+        if (verbose) log_addf(VTEXT(T_CANT_SET_STROBE), strerror_r(errno, errbuf, sizeof(errbuf))); 
         close(cwda->fd);
         cwda->fd=-1;
         return 1;
@@ -181,7 +187,6 @@ int parport_band(struct cwdaemon *cwda, int bandsw){
     if (bandsw & 0x04) cwda->data |= 0x40;
     if (bandsw & 0x08) cwda->data |= 0x80;
 
-    dbg("parport_band: %d\n", cwda->data);
     if (ioctl(cwda->fd, PPWDATA, &cwda->data)<0){ 
         dbg("Can't write data to parport\n"); 
         close(cwda->fd);
@@ -191,7 +196,7 @@ int parport_band(struct cwdaemon *cwda, int bandsw){
     return 0;
 }
 
-void parport_info(){
+void parport_info(GString *gs){
     int port;
     int fd;
     char s[1024];
@@ -199,52 +204,52 @@ void parport_info(){
     unsigned int modes;
     int mode;
 
-    printf("\n  parport_info:\n");
+    g_string_sprintfa(gs, "\n  parport_info:\n");
     for (port=0;port<4;port++){
         sprintf(s, "/dev/parport%d", port);
         fd=open(s, O_RDWR|O_NONBLOCK);
         if (fd<0){
             if (errno==ENOENT) continue;
-            printf("Found /dev/parport%d but cannot open it: %s\n", port, strerror_r(errno, errbuf, sizeof(errbuf)));
+            g_string_sprintfa(gs, "Found /dev/parport%d but cannot open it: %s\n", port, strerror_r(errno, errbuf, sizeof(errbuf)));
             continue;
         }
         if (ioctl(fd, PPGETMODES, &modes)<0){
             if (errno==ENODEV) continue;
-            printf("Can't get modes for /dev/parport%d: %s\n", port, strerror_r(errno, errbuf, sizeof(errbuf)));
+            g_string_sprintfa(gs, "Can't get modes for /dev/parport%d: %s\n", port, strerror_r(errno, errbuf, sizeof(errbuf)));
             close(fd);
             continue;
         }
-        printf("Found /dev/parport%d\n", port);
+        g_string_sprintfa(gs, "Found /dev/parport%d\n", port);
         
-        printf(" modes: 0x%x ", modes);
-        if (modes&PARPORT_MODE_PCSPP)     printf("SPP ");
-        if (modes&PARPORT_MODE_TRISTATE)  printf("TRISTATE ");
-        if (modes&PARPORT_MODE_EPP)       printf("EPP ");
-        if (modes&PARPORT_MODE_ECP)       printf("ECP ");
-        if (modes&PARPORT_MODE_COMPAT)    printf("COMPAT ");
-        if (modes&PARPORT_MODE_DMA)       printf("DMA ");
-        if (modes&PARPORT_MODE_SAFEININT) printf("SAFEINIT ");
-        printf("\n");
+        g_string_sprintfa(gs, " modes: 0x%x ", modes);
+        if (modes&PARPORT_MODE_PCSPP)     g_string_sprintfa(gs, "SPP ");
+        if (modes&PARPORT_MODE_TRISTATE)  g_string_sprintfa(gs, "TRISTATE ");
+        if (modes&PARPORT_MODE_EPP)       g_string_sprintfa(gs, "EPP ");
+        if (modes&PARPORT_MODE_ECP)       g_string_sprintfa(gs, "ECP ");
+        if (modes&PARPORT_MODE_COMPAT)    g_string_sprintfa(gs, "COMPAT ");
+        if (modes&PARPORT_MODE_DMA)       g_string_sprintfa(gs, "DMA ");
+        if (modes&PARPORT_MODE_SAFEININT) g_string_sprintfa(gs, "SAFEINIT ");
+        g_string_sprintfa(gs, "\n");
         
         if (ioctl(fd, PPGETMODE, &mode)<0){
             close(fd);
             continue;
         }
-        printf(" mode: 0x%x ", mode);
-        if (mode==0) printf("NIBBLE ");
-        if (mode&IEEE1284_MODE_BYTE)   printf("BYTE ");
-        if (mode&IEEE1284_MODE_COMPAT) printf("COMPAT ");
-        if (mode&IEEE1284_MODE_BECP)   printf("Bounded_ECP ");
-        if (mode&IEEE1284_MODE_ECP)    printf("ECP ");
-        if (mode&IEEE1284_MODE_ECPRLE) printf("ECP_RLE ");
-        if (mode&IEEE1284_MODE_ECPSWE) printf("Software_ECP ");
-        if (mode&IEEE1284_MODE_EPP)    printf("EPP ");
-        if (mode&IEEE1284_MODE_EPPSL)  printf("EPP_1.7 ");
-        if (mode&IEEE1284_MODE_EPPSWE) printf("Software_EPP ");
-        if (mode&IEEE1284_DEVICEID)    printf("DEVICEID ");
-        printf("\n");
+        g_string_sprintfa(gs, " mode: 0x%x ", mode);
+        if (mode==0) g_string_sprintfa(gs, "NIBBLE ");
+        if (mode&IEEE1284_MODE_BYTE)   g_string_sprintfa(gs, "BYTE ");
+        if (mode&IEEE1284_MODE_COMPAT) g_string_sprintfa(gs, "COMPAT ");
+        if (mode&IEEE1284_MODE_BECP)   g_string_sprintfa(gs, "Bounded_ECP ");
+        if (mode&IEEE1284_MODE_ECP)    g_string_sprintfa(gs, "ECP ");
+        if (mode&IEEE1284_MODE_ECPRLE) g_string_sprintfa(gs, "ECP_RLE ");
+        if (mode&IEEE1284_MODE_ECPSWE) g_string_sprintfa(gs, "Software_ECP ");
+        if (mode&IEEE1284_MODE_EPP)    g_string_sprintfa(gs, "EPP ");
+        if (mode&IEEE1284_MODE_EPPSL)  g_string_sprintfa(gs, "EPP_1.7 ");
+        if (mode&IEEE1284_MODE_EPPSWE) g_string_sprintfa(gs, "Software_EPP ");
+        if (mode&IEEE1284_DEVICEID)    g_string_sprintfa(gs, "DEVICEID ");
+        g_string_sprintfa(gs, "\n");
         close(fd);
     }
-    printf("\n");
+    g_string_sprintfa(gs, "\n");
 }
 #endif
