@@ -260,6 +260,7 @@ enum {
     SYMBOL_AC_MAXELEV        = G_TOKEN_LAST + 255,
     SYMBOL_AC_MINDUR         = G_TOKEN_LAST + 256,
     SYMBOL_AC_MAXDELTA       = G_TOKEN_LAST + 257,
+    SYMBOL_AC_MINALT         = G_TOKEN_LAST + 258,
     
 	SYMBOL_SDR_ENABLE        = G_TOKEN_LAST + 260,
 	SYMBOL_SDR_CW            = G_TOKEN_LAST + 261,
@@ -522,6 +523,7 @@ void add_symbols(GScanner *scanner){
     g_scanner_add_symbol(scanner,"ac_maxelev",  GINT_TO_POINTER(SYMBOL_AC_MAXELEV));
     g_scanner_add_symbol(scanner,"ac_mindur",   GINT_TO_POINTER(SYMBOL_AC_MINDUR));
     g_scanner_add_symbol(scanner,"ac_maxdelta", GINT_TO_POINTER(SYMBOL_AC_MAXDELTA));
+    g_scanner_add_symbol(scanner,"ac_minalt", GINT_TO_POINTER(SYMBOL_AC_MINALT));
 
 	g_scanner_add_symbol(scanner,"sdr_enable",   GINT_TO_POINTER(SYMBOL_SDR_ENABLE));
 	g_scanner_add_symbol(scanner,"sdr_cw",       GINT_TO_POINTER(SYMBOL_SDR_CW));
@@ -608,6 +610,7 @@ gint init_rc(void){
     cfg->ac_maxelev = +5.0;
     cfg->ac_mindur = 45;
     cfg->ac_maxdelta = 0;
+    cfg->ac_minalt = 2000;
 
 	cfg->sdr_cw = 0;
 	cfg->sdr_lsb = 1;
@@ -1191,6 +1194,7 @@ guint read_one_token(GScanner *scanner){
 		STORE_RC_DOUBLE(cfg, SYMBOL_AC_MAXELEV, ac_maxelev);
         STORE_RC_INT   (cfg, SYMBOL_AC_MINDUR, ac_mindur);
         STORE_RC_DOUBLE(cfg, SYMBOL_AC_MAXDELTA, ac_maxdelta);
+        STORE_RC_INT   (cfg, SYMBOL_AC_MINALT, ac_minalt);
 
 		STORE_RC_INT   (cfg, SYMBOL_SDR_ENABLE, sdr_enable);
 		STORE_RC_INT   (cfg, SYMBOL_SDR_CW, sdr_cw);
@@ -1693,6 +1697,7 @@ void save_rc_string(GString *gs){
     g_string_sprintfa(gs, "ac_maxelev  = %3.1f\n", cfg->ac_maxelev);
     g_string_sprintfa(gs, "ac_mindur   = %d\n", cfg->ac_mindur);
     g_string_sprintfa(gs, "ac_maxdelta = %3.1f\n", cfg->ac_maxdelta);
+    g_string_sprintfa(gs, "ac_minalt = %d\n", cfg->ac_minalt);
     g_string_sprintfa(gs, "\n");
 
 	g_string_sprintfa(gs, "sdr_enable = %d\n", cfg->sdr_enable);
