@@ -1,6 +1,6 @@
 /*
     Tucnak - VHF contest log
-    Copyright (C) 2002-2012  Ladislav Vaiz <ok1zia@nagano.cz>
+    Copyright (C) 2002-2015  Ladislav Vaiz <ok1zia@nagano.cz>
 
     This program is free software; you can redistribute it and/or
     modify it under the terms of the GNU General Public License
@@ -9,22 +9,35 @@
 */
 
 #include "header.h"         
+#include "bfu.h"
+#include "fifo.h"
+#include "hdkeyb.h"
+#include "kbd.h"
+#include "main.h"
+#include "map.h"
+#include "net.h"
+#include "rc.h"
+#include "rotar.h"
+#include "sdev.h"
+#include "tsdl.h"
+#include "session.h"
+#include "subwin.h"
+#include "zstring.h"
 
 struct rotar *rota,*rotb;
-int hdkeyb_disp_ofs[HDKEYB_ROTARS] = {0, 8, 64, 72};
 struct hdkeyb *hdkeyb;
 
 GPtrArray *rotars;
 MUTEX_DEFINE(rotars);
 struct rotar *rotar;
 
-gpointer rot_hamlib_main(gpointer xxx);
-
+#ifdef HAVE_HAMLIB
 static int rot_print_model_list(const struct rot_caps *caps, void *data)
 {
     log_addf("%-6d %-14s  %-20s  %s\n", caps->rot_model, caps->mfg_name, caps->model_name, caps->version);
     return 1;  /* !=0, we want them all ! */
 }
+#endif
 
 
 struct rotar *init_rotar(struct config_rotar *crot, int rotchar){
@@ -33,8 +46,9 @@ struct rotar *init_rotar(struct config_rotar *crot, int rotchar){
     if (!crot) return NULL;
     if (!crot->rot_type) return NULL;
 
-    rot=g_new0(struct rotar, 1);
-    
+    rot = g_new0(struct rotar, 1);
+	rot->poll_ms = crot->rot_poll_ms;
+	rot->last_req = ztimeout_init(rot->poll_ms);
     switch(crot->rot_type){
         case ROT_OK1ZIA_TTYS:
             rot->sdev=sd_open_ttys(crot->rot_saddr, crot->rot_filename, crot->rot_timeout_ms);
@@ -46,7 +60,6 @@ struct rotar *init_rotar(struct config_rotar *crot, int rotchar){
             rot->sdev->sdev_main = rotar_main;
             rot->sdev->arg = rot;
             break;
-#ifdef HAVE_LIBFTDI
         case ROT_OK1ZIA_FTDI:
             rot->sdev=sd_open_ftdi(crot->rot_saddr, crot->rot_vid, crot->rot_pid, crot->rot_serial, crot->rot_timeout_ms);
             dbg("init_rotar rot=%p  saddr=%d %04x:%04x timeout=%d   sdev=%p\n", rot->sdev, crot->rot_saddr, crot->rot_vid, crot->rot_pid, crot->rot_timeout_ms, rot->sdev);
@@ -57,7 +70,6 @@ struct rotar *init_rotar(struct config_rotar *crot, int rotchar){
             rot->sdev->sdev_main = rotar_main;
             rot->sdev->arg = rot;
             break;
-#endif
         case ROT_REMOTE:
             dbg("init_rotar rot=%p  saddr=%d hostname='%s' port='%d' timeout=%d\n", rot->sdev, crot->rot_saddr, crot->rot_hostname, crot->rot_port, crot->rot_timeout_ms);
             break;
@@ -65,10 +77,10 @@ struct rotar *init_rotar(struct config_rotar *crot, int rotchar){
         case ROT_HAMLIB:
             rot->rot = rot_init(crot->rot_model);
 			if (!rot->rot){
-				log_addf("Unknown rotarot number %d or initialization error", crot->rot_model);
+				log_addf(VTEXT(T_UNKNOWN_ROTNR_OR_INIT_ERROR), crot->rot_model);
 				g_free(rot);
 
-				log_addf("Registrated rotators:");
+				log_addf(VTEXT(T_REGISTRATED_ROTATORS));
 				rot_load_all_backends();
 				rot_list_foreach(rot_print_model_list, NULL);
 
@@ -81,14 +93,14 @@ struct rotar *init_rotar(struct config_rotar *crot, int rotchar){
 
             rot->hl_azim = -1;
             rot->hl_model = crot->rot_model;
-			rot->hl_thread = g_thread_create(rot_hamlib_main, (gpointer)rot, TRUE, NULL); 
+			rot->hl_thread = g_thread_try_new("hamlib rotator", rot_hamlib_main, (gpointer)rot, NULL); 
 			if (!rot->hl_thread) {
-				log_addf("Can't create hamlib rotar thread\n");
+				log_addf(VTEXT(T_CANT_CREATE_HL_ROT_THR));
 			}
             break;
 #endif            
         default:
-            log_addf("Unsupported rotator type %d\n", crot->rot_type);
+            log_addf(VTEXT(T_UNSUPPORTED_ROTATOR), crot->rot_type);
             g_free(rot);
             return NULL;
             break;
@@ -117,13 +129,17 @@ struct rotar *init_rotar(struct config_rotar *crot, int rotchar){
 }
 
 void free_rotar(struct rotar *rot){
+#ifdef HAVE_HAMLIB
 	if (rot->hl_thread){
+		progress(VTEXT(T_WAIT_HL_ROT_THREAD));		
+
         rot->hl_thread_break = 1;
         dbg("join hamlib rotar...\n");
         g_thread_join(rot->hl_thread);
         dbg("done\n");
         rot->hl_thread=0;
     }
+#endif
     g_free(rot->desc);
     g_free(rot->netid);
     g_free(rot->rem_rotstr);
@@ -137,6 +153,7 @@ int init_rotars(){
     int i;
     char rotchar='A';
 
+	progress(VTEXT(T_INIT_ROTARS2));		
 
     MUTEX_INIT(rotars);
     rotars=g_ptr_array_new();
@@ -149,7 +166,7 @@ int init_rotars(){
         g_ptr_array_add(rotars, rot); 
         MUTEX_UNLOCK(rotars);
     }
-#ifdef HAVE_SDL
+#ifdef Z_HAVE_SDL
     rot_update_colors();
 #endif
     return 0;
@@ -159,9 +176,11 @@ int free_rotars(void){
     int i;
     struct rotar *rot;
 
+	progress(VTEXT(T_INIT_ROTARS2));		
+
     MUTEX_LOCK(rotars);
     for (i=0;i<rotars->len;i++){
-        rot = (struct rotar *)g_ptr_array_index(rotars, i);
+        rot = (struct rotar *) g_ptr_array_index(rotars, i);
         free_rotar(rot);
     }
     MUTEX_UNLOCK(rotars);
@@ -170,7 +189,7 @@ int free_rotars(void){
     return 0;
 }
 
-#ifdef HAVE_SDL
+#ifdef Z_HAVE_SDL
 int rot_update_colors(){
 	int i;
 	struct rotar *rot;
@@ -182,19 +201,19 @@ int rot_update_colors(){
 	
 		switch(i%4){
 			case 0:
-				rot->color=makecol(200,0,0);
+				rot->color=z_makecol(200,0,0);
                 rot->termcolor=COL_RED;
 				break;
 			case 1:
-				rot->color=makecol(200,0,200);
+				rot->color=z_makecol(200,0,200);
                 rot->termcolor=COL_MAGENTA;
 				break;
 			case 2:
-				rot->color=makecol(0,200,0);
+				rot->color=z_makecol(0,200,0);
                 rot->termcolor=COL_GREEN;
 				break;
 			case 3:
-				rot->color=makecol(0,200,200);
+				rot->color=z_makecol(0,200,200);
                 rot->termcolor=COL_CYAN;
 				break;
 		}
@@ -234,6 +253,7 @@ int rot_seek(struct rotar *rot, int uhel){
     return 0;
 }
 
+
 struct config_rotar *get_config_rotar_by_number(GPtrArray *crots, int nr){
     struct config_rotar *crot;
     int i;
@@ -291,61 +311,57 @@ struct rotar *get_rotar_by_netid_rotstr(char *netid, char *rotstr){
 }
 
 
-void rotar_read_handler(char *line){
-    struct zstring *zs, *zs2;
+void rotar_read_handler(int n, char **items){
+    struct zstring *zs2;
     char *cmd, *rotstr, *c;
 	struct rotar *rot;
     
 //    dbg("rotar_read_handler('%s')  gses=%p\n", line, gses);
 
-    zs = zstrdup(line);
-                          
-    ztokenize(zs, 1);
-	rotstr = ztokenize(zs, 0);
+	rotstr = items[1];
 
 	rot = get_rotar_by_rotstr(rotstr);
 	if (!rot){
-		log_addf("Unknown rotar '%s'\n", rotstr); 
-		dbg("Unknown rotar '%s'\n", rotstr); 
+		log_addf(VTEXT(T_UNKNOWN_ROTAR), rotstr); 
+		//dbg("Unknown rotar '%s'\n", rotstr); 
 		goto x;
 	}
-	cmd = ztokenize(zs, 0);
+	cmd = items[2];
 	
 //    dbg("rotar_read_handler   rcvd: '%s'  cmd='%s' rotstr='%s'\n", line, cmd, rotstr);
     if (strcmp(cmd, "!")==0){  /* error */
-        log_addf("Rotar error: %s", ztokenize(zs, 0));
+        log_addf(VTEXT(T_ROTAR_ERROR_S), items[3]);
     }
     if (strcasecmp(cmd, "QTF")==0){
-        char *qtf = ztokenize(zs, 0);
+        char *qtf = items[3];
         rot->qtf=atoi(qtf);
         
-        zs2 = zconcatesc(net->myid, rotstr, qtf, "0", NULL);
+        zs2 = zconcatesc(gnet->myid, rotstr, qtf, "0", NULL);
         c = g_strconcat("RU ", zs2->str, "\n", NULL);
         rel_write_all(c);
         g_free(c);
         zfree(zs2);
 
-#ifdef HAVE_SDL
+#ifdef Z_HAVE_SDL
         //if (!gses->redraw_timer_id)
-        //    gses->redraw_timer_id = install_timer(DELAY_AFTER_UPDATE_ROTAR, timer_redraw, CBA0);
+        //    gses->redraw_timer_id = zselect_timer_new(zsel, DELAY_AFTER_UPDATE_ROTAR, timer_redraw, NULL);
         if (gses && gses->ontop->type == SWT_MAP){
             if (!gses->update_rotar_timer_id)
-                gses->update_rotar_timer_id = install_timer(2*DELAY_AFTER_UPDATE_ROTAR, timer_update_rotar, CBA0);
+                gses->update_rotar_timer_id = zselect_timer_new(zsel, 2*DELAY_AFTER_UPDATE_ROTAR, timer_update_rotar, NULL);
         }
 #endif
-#ifdef HAVE_LIBFTDI
+#ifdef Z_HAVE_LIBFTDI
 		hdkeyb_draw_rotar(hdkeyb, rot->rotchar - 'A'); 
 #endif
         redraw_later();
         goto x;
     }
 x:;	
-    zfree(zs);
     redraw_later();
 }
 
-void timer_update_rotar(cba_t cba){
-#ifdef HAVE_SDL
+void timer_update_rotar(void *arg){
+#ifdef Z_HAVE_SDL
     if (gses && gses->ontop->type == SWT_MAP){
         sw_map_update_rotar(gses->ontop);
     }
@@ -356,19 +372,15 @@ void timer_update_rotar(cba_t cba){
 // called from sconn thread
 int rotar_main(struct sdev *sdev){
     struct rotar *rot = (struct rotar *)sdev->arg;
-	char s[256], data[256];
+	char data[256];
 	int len, ret, qtf;
   //  static int xx = 0;
-    struct timeval tv;
-    int req;
 
     //dbg("rotar_main rot=%p\n", rot);
     if (!rot) return -1;
-    usleep(10000);
-    gettimeofday(&tv, NULL);
-    req = tv.tv_usec / 250000;
-    if (req == rot->last_req) return 0;
-    rot->last_req = req;
+    
+	if (!ztimeout_occured(rot->last_req)) return 113;
+	rot->last_req = ztimeout_init(rot->poll_ms);
     
     len=0;
 
@@ -383,8 +395,9 @@ int rotar_main(struct sdev *sdev){
     if (qtf == rot->oldqtf) return 0;
     
     rot->oldqtf = qtf;
-    g_snprintf(s, sizeof(s), "%d", qtf);
-    ret = zwrite(tpipe->threadpipe_write, zconcatesc("ROT", rot->rotstr, "QTF", s, NULL));
+    //g_snprintf(s, sizeof(s), "%d", qtf);
+    //ret = zwrite(tpipe->threadpipe_write, zconcatesc("ROT", rot->rotstr, "QTF", s, NULL));
+	zselect_msg_send(zsel, "%s;%s;%s;%d", "ROT", rot->rotstr, "QTF", qtf);
 //    dbg("zwrite ret=%d\n", ret);
     return 0;
 }
@@ -495,14 +508,14 @@ void rotar_set_desc(void){
     switch(rotar->type){
 #ifdef HAVE_HAMLIB
         case ROT_HAMLIB:
-            g_snprintf(rotar_desc1, MAX_STR_LEN, "Rotr %c (hamlib %u)", rotar->rotchar, rotar->hl_model);
+            g_snprintf(rotar_desc1, MAX_STR_LEN, "%s %c (hamlib %u)", VTEXT(T_ROTAR), rotar->rotchar, rotar->hl_model);
             break;
 #endif            
         case ROT_REMOTE:
-            g_snprintf(rotar_desc1, MAX_STR_LEN, "Rotr %c (%s %s)", rotar->rotchar, rotar->netid, rotar->rem_rotstr);
+            g_snprintf(rotar_desc1, MAX_STR_LEN, "%s %c (%s %s)", VTEXT(T_ROTAR), rotar->rotchar, rotar->netid, rotar->rem_rotstr);
             break;
         default:
-            g_snprintf(rotar_desc1, MAX_STR_LEN, "Rotr %c (%u)", rotar->rotchar, (unsigned char)rotar->sdev->saddr);
+            g_snprintf(rotar_desc1, MAX_STR_LEN, "%s %c (%u)", VTEXT(T_ROTAR), rotar->rotchar, (unsigned char)rotar->sdev->saddr);
             break;
     }
     g_snprintf(rotar_desc2, MAX_STR_LEN, "%s", rotar->desc);
@@ -516,7 +529,7 @@ int rotar_func(struct dialog_data *data, struct event *ev){
         case EV_KBD:
             if (ev->y & KBD_ALT) break;
             if (ev->y & KBD_CTRL) break;
-            rotchar=upcase(ev->x);
+            rotchar=z_char_uc(ev->x);
             if (rotchar<'A' || rotchar>'Z') {
                 return -1;
             }
@@ -527,7 +540,7 @@ int rotar_func(struct dialog_data *data, struct event *ev){
 
             rotar=get_rotar(rotchar-'A');
             rotar_set_desc();
-            resize_terminal(CBA0);
+            resize_terminal(NULL);
             return EVENT_PROCESSED;
         
     }
@@ -536,7 +549,7 @@ int rotar_func(struct dialog_data *data, struct event *ev){
         
         
 
-void menu_rotar(cba_t cba){
+void menu_rotar(void *arg){
 
     struct dialog *d;
     int i;
@@ -551,9 +564,9 @@ void menu_rotar(cba_t cba){
     strcpy(rotar_elev_str, "");
 
     
-    if (!(d = mem_alloc(sizeof(struct dialog) + 55 * sizeof(struct dialog_item)))) return;
+    if (!(d = (struct dialog *) g_malloc(sizeof(struct dialog) + 55 * sizeof(struct dialog_item)))) return;
     memset(d, 0, sizeof(struct dialog) + 55 * sizeof(struct dialog_item));
-    d->title = "Rotator";
+    d->title = VTEXT(T_ROTAR);
     d->fn = rotar_fn;
     d->refresh = (void (*)(void *))refresh_rotar;
     d->refresh_data = (void *)rotar;
@@ -592,9 +605,11 @@ gpointer rot_hamlib_main(gpointer xxx){
     azimuth_t azimuth;
     elevation_t elevation;
     int qtf;
-    char s[256];
+	int tmo;
 
     dbg("rot_hl_main\n");
+	zg_thread_set_name("Tucnak hamlib rotator");
+
 
     ret = rot_open(rot->rot);
     if (ret != RIG_OK){
@@ -621,13 +636,23 @@ gpointer rot_hamlib_main(gpointer xxx){
             qtf = (int)azimuth;
             if (qtf != rot->oldqtf){
                 rot->oldqtf = qtf;
-                g_snprintf(s, sizeof(s), "%d", qtf);
-                ret = zwrite(tpipe->threadpipe_write, zconcatesc("ROT", rot->rotstr, "QTF", s, NULL));
+               // g_snprintf(s, sizeof(s), "%d", qtf);
+                zselect_msg_send(zsel, "ROT;%s;QTF;%d", rot->rotstr, qtf);
+               // ret = zwrite(tpipe->threadpipe_write, zconcatesc("ROT", rot->rotstr, "QTF", s, NULL));
                 //    dbg("zwrite ret=%d\n", ret);
             }
 
         }
-        usleep(10000);
+
+
+
+		if (rot->poll_ms > 0){
+			for (tmo = ztimeout_init(rot->poll_ms); !ztimeout_occured(tmo); )
+			{
+				if (rot->hl_azim >= 0) break;
+				usleep(10000);
+			}
+		}
     }
     dbg("rot_hl_main exiting\n");
     rot_close(rot->rot);
@@ -637,5 +662,12 @@ gpointer rot_hamlib_main(gpointer xxx){
 }
 #endif
 
+int rot_beamwidth(){
+	int beamwidth = 20;
+	struct config_rotar *rot;
 
+	if (!cfg->crotars->len) return beamwidth;
+	rot = (struct config_rotar *)g_ptr_array_index(cfg->crotars, 0);
+	return rot->rot_beamwidth;
+}
 
