@@ -1,6 +1,6 @@
 /*
     Tucnak - VHF contest log
-    Copyright (C) 2002-2008  Ladislav Vaiz <ok1zia@nagano.cz>
+    Copyright (C) 2002-2015  Ladislav Vaiz <ok1zia@nagano.cz>
 
     This program is free software; you can redistribute it and/or
     modify it under the terms of the GNU General Public License
@@ -8,13 +8,28 @@
 
 */
 
-
 #include "header.h"
 
+#include "edi.h"
+#include "fifo.h"
+#include "inputln.h"
+#include "kbdbind.h"
+#include "kbd.h"
+#include "main.h"
+#include "net.h"
+#include "qsodb.h"
+#include "tsdl.h"
+#include "session.h"
+#include "sked.h"
+#include "subwin.h"
+#include "subwin.h"
+
 char callsign[EQSO_LEN], locator[EQSO_LEN];
 char time_str[EQSO_LEN], remark[MAX_STR_LEN]; 
 
 int sw_sked_kbd_func(struct subwin *sw, struct event *ev, int fw){
+	struct sked *sked;
+	int i;
     if (!aband) return 0;
 
     if (sw->il && (
@@ -65,6 +80,16 @@ int sw_sked_kbd_func(struct subwin *sw, struct event *ev, int fw){
 			sw->ho++;
             redraw_later();
             return 1;
+		case ACT_ENTER:
+			if (aband->readonly) return 1;
+			i = aband->skeds->len - sw->offset - 1;
+			if (i < 0 || i >= aband->skeds->len) break;
+			sked = (struct sked *)g_ptr_array_index(aband->skeds, i);
+			process_input(aband, sked->callsign, 0);
+			il_add_to_history(INPUTLN(aband), sked->callsign);
+			process_input(aband, sked->locator, 0);
+			aband->tmpqsos[0].ulocator = 1;
+			return 1;
     }
     return 0;
 }
@@ -87,16 +112,14 @@ gchar *sw_sked_sprint(struct sked *sked){
         qtf_int=0;
     }  
 
-#ifdef HAVE_SDL    
+#ifdef Z_HAVE_SDL    
     if (sdl)
         deg = "";
     else
 #endif      
-    deg = "deg";
+    deg = VTEXT(T_DEGREE);
             
-    c=g_strdup_printf("%6s@%s: %s %-6s "
-                      "%-13s in %-6s (%dkm, %d"
-                      "%s) at %s  %s",
+    c=g_strdup_printf(VTEXT(T_SKED_FORMAT),
             sked->operator_, sked->src_shortpband, sked->pband, sked->qrg, 
             sked->callsign, sked->locator, qrb_int, qtf_int, 
             deg, sked->time_str, sked->remark);
@@ -104,20 +127,30 @@ gchar *sw_sked_sprint(struct sked *sked){
     return c;
 }
 
-void sw_sked_redraw(struct subwin *sw, int flags){
+void sw_sked_redraw(struct subwin *sw, struct band *band, int flags){
     int i, index;
     gchar *c;
     struct sked *sked;
+	int color;
     
     if (!aband) return;
     /*dbg("sw_log_redraw  log->len=%d sw->h=%d sw->offset=%d \n", fifo_len(sw->fifo), sw->h, sw->offset);*/
     
-    for (i=0;i<sw->h; i++){
+	
+    for (i = 0; i < sw->h; i++){
         index = aband->skeds->len - sw->offset - sw->h+i;
-        if (index<0 || index>=aband->skeds->len) continue;
-        sked = g_ptr_array_index(aband->skeds, index);
+        if (index < 0 || index >= aband->skeds->len) continue;
+        sked = (struct sked *)g_ptr_array_index(aband->skeds, index);
         c = sw_sked_sprint(sked);
-	    print_text(sw->x, sw->y+i, sw->w, c+sw->ho, sked->worked ? COL_DARKYELLOW : COL_NORM);
+		
+		if (i == sw->h - 1) {
+			color = sked->worked ? COL_INV | COL_DARKYELLOW : COL_INV;
+			fill_area(sw->x, sw->y + i, sw->w, 1, COL_INV);
+		}else{
+			color = sked->worked ? COL_DARKYELLOW : COL_NORM;
+		}
+
+	    print_text(sw->x, sw->y + i, sw->w, c + sw->ho, color);
         g_free(c);
     }
 }
@@ -126,8 +159,8 @@ void sw_sked_check_bounds(struct subwin *sw){
     if (!aband) return;
     
     /*if (sw->offset < 0) sw->offset=0;*/
-    if (sw->offset > aband->skeds->len - sw->h) {
-        sw->offset = aband->skeds->len - sw->h;
+    if (sw->offset > (int)aband->skeds->len - 1) {
+        sw->offset = (int)aband->skeds->len - 1;
     }
     if (sw->offset < 0) sw->offset=0;
 }
@@ -138,7 +171,7 @@ void sw_sked_read(gchar *str, int from_my){
     struct sked *sked;
     struct zstring *zs;
     struct subwin *sw;
-    gchar *c;
+    gchar *c, *d;
 
     
     dbg("sw_sked_read('%s', %d)\n", str, from_my);
@@ -169,8 +202,10 @@ void sw_sked_read(gchar *str, int from_my){
             sw->unread=1;
         }
         c = sw_sked_sprint(sked);
-        log_adds(c); //FIXME
+		d = g_strdup_printf("Sked %s", c);
+        log_adds(d);
         g_free(c);
+        g_free(d);
         skedw_create(b);
     }
     check_autosave();
@@ -180,7 +215,6 @@ void sw_sked_read(gchar *str, int from_my){
 
 
 char qrg_str[EQSO_LEN];
-int we_call;
 char pband[EQSO_LEN]="Select"; /* TODO VTEXT(T_SELECT) */
 char oldpband[EQSO_LEN]=""; 
 /*, time_str[EQSO_LEN], callsign[EQSO_LEN]*/;
@@ -194,7 +228,7 @@ void refresh_sked(struct qso *q)
     time_t now;
     struct tm utc;
     struct zstring *zs;
-    char sked_time_str[32], we_call_str[32];
+    char sked_time_str[32];
 
     time(&now);
     gmtime_r(&now, &utc);
@@ -205,12 +239,11 @@ void refresh_sked(struct qso *q)
     safe_strncpy0(s, aband->pband, 10); 
     c = strchr(s, ' ');
     if (c) *c='\0';
-    g_snprintf(we_call_str, 30, "%d", we_call);
-    uc(callsign);
-    uc(locator);
+    z_str_uc(callsign);
+    z_str_uc(locator);
 
     zs = zconcatesc(sked_time_str, aband->operator_, s, qrg_str, 
-            we_call_str, pband, time_str, callsign, 
+            "0", pband, time_str, callsign, 
             locator, remark, NULL);
     
     sw_sked_read(zs->str,1);
@@ -221,23 +254,12 @@ void refresh_sked(struct qso *q)
     zfree(zs);
 }                    
 
-char *sked_msg[] = {
-    CTEXT(T_QRG), /* 0 */
-    CTEXT(T_WE_CALL), 
-    CTEXT(T_BAND),
-    CTEXT(T_TIME), 
-    CTEXT(T_CALLSIGN2),
-    CTEXT(T_WWL), /* 5 */
-    CTEXT(T_REMARK),
-    "", /* OK */  /* 7 */
-    "", /* Cancel */
-};
 void sked_fn(struct dialog_data *dlg)
 {
-    struct terminal *term = dlg->win->term;
-    int max = 0, min = 0;
-    int w, rw;
-    int y = -1;
+    //struct terminal *term = dlg->win->term;
+    //int max = 0, min = 0;
+    //int w, rw;
+    //int y = -1;
     struct band *band;
 
 /*    dbg("items[0]=%p\n",dlg->items[0]); 
@@ -248,67 +270,10 @@ void sked_fn(struct dialog_data *dlg)
  /*       dbg("pband=%s\n");*/
         band = find_band_by_pband(pband);
         if (band) {
-            safe_strncpy0(dlg->items[0].cdata, band->skedqrg, EQSO_LEN);
-            dlg->items[0].cpos = strlen(dlg->items[0].cdata);
+            safe_strncpy0(dlg->items[1].cdata, band->skedqrg, EQSO_LEN);
+            dlg->items[1].cpos = strlen(dlg->items[1].cdata);
         }
     }
-
-    max_group_width(term, sked_msg + 0, dlg->items + 0, 1, &max);
-    min_group_width(term, sked_msg + 0, dlg->items + 0, 1, &min);
-    max_group_width(term, sked_msg + 1, dlg->items + 1, 1, &max);
-    min_group_width(term, sked_msg + 1, dlg->items + 1, 1, &min);
-    max_group_width(term, sked_msg + 2, dlg->items + 2, 1, &max);
-    min_group_width(term, sked_msg + 2, dlg->items + 2, 1, &min);
-    max_group_width(term, sked_msg + 3, dlg->items + 3, 1, &max);
-    min_group_width(term, sked_msg + 3, dlg->items + 3, 1, &min);
-    max_group_width(term, sked_msg + 4, dlg->items + 4, 1, &max);
-    min_group_width(term, sked_msg + 4, dlg->items + 4, 1, &min);
-    max_group_width(term, sked_msg + 5, dlg->items + 5, 1, &max);
-    min_group_width(term, sked_msg + 5, dlg->items + 5, 1, &min);
-    max_group_width(term, sked_msg + 6, dlg->items + 6, 1, &max);
-    min_group_width(term, sked_msg + 6, dlg->items + 6, 1, &min);
-    
-    max_buttons_width(term, dlg->items +7, 2, &max);
-    min_buttons_width(term, dlg->items +7, 2, &min);
-    
-    w = dlg->win->term->x * 9 / 10 - 2 * DIALOG_LB;
-    if (w > max) w = max;
-    if (w < min) w = min;
-    if (w > dlg->win->term->x - 2 * DIALOG_LB) w = dlg->win->term->x - 2 * DIALOG_LB;
-    if (w < 1) w = 1;
-    
-    rw = 0;
-    y ++;
-    dlg_format_group(NULL, term, sked_msg + 0, dlg->items + 0, 1, 0, &y, w, &rw);
-    dlg_format_group(NULL, term, sked_msg + 1, dlg->items + 1, 1, 0, &y, w, &rw);
-    dlg_format_group(NULL, term, sked_msg + 2, dlg->items + 2, 1, 0, &y, w, &rw);
-    dlg_format_group(NULL, term, sked_msg + 3, dlg->items + 3, 1, 0, &y, w, &rw);
-    dlg_format_group(NULL, term, sked_msg + 4, dlg->items + 4, 1, 0, &y, w, &rw);
-    dlg_format_group(NULL, term, sked_msg + 5, dlg->items + 5, 1, 0, &y, w, &rw);
-    dlg_format_group(NULL, term, sked_msg + 6, dlg->items + 6, 1, 0, &y, w, &rw);
-    y++;
-    dlg_format_buttons(NULL, term, dlg->items +7, 2, 0, &y, w, &rw, AL_LEFT);
-    
-    
-    w = rw;
-    dlg->xw = w + 2 * DIALOG_LB;
-    dlg->yw = y + 2 * DIALOG_TB;
-
-    
-    center_dlg(dlg);
-    draw_dlg(dlg);
-    y = dlg->y + DIALOG_TB;
-    y++;
-    dlg_format_group(term, term, sked_msg + 0, dlg->items + 0, 1, dlg->x + DIALOG_LB, &y, w, AL_LEFT);
-    dlg_format_group(term, term, sked_msg + 1, dlg->items + 1, 1, dlg->x + DIALOG_LB, &y, w, AL_LEFT);
-    dlg_format_group(term, term, sked_msg + 2, dlg->items + 2, 1, dlg->x + DIALOG_LB, &y, w, AL_LEFT);
-    dlg_format_group(term, term, sked_msg + 3, dlg->items + 3, 1, dlg->x + DIALOG_LB, &y, w, AL_LEFT);
-    dlg_format_group(term, term, sked_msg + 4, dlg->items + 4, 1, dlg->x + DIALOG_LB, &y, w, AL_LEFT);
-    dlg_format_group(term, term, sked_msg + 5, dlg->items + 5, 1, dlg->x + DIALOG_LB, &y, w, AL_LEFT);
-    dlg_format_group(term, term, sked_msg + 6, dlg->items + 6, 1, dlg->x + DIALOG_LB, &y, w, AL_LEFT);
-    y++;
-    dlg_format_buttons(term, term, dlg->items +7, 2, dlg->x + DIALOG_LB, &y, w, NULL, AL_LEFT);
-    
 }
 
 void sked(void){
@@ -333,47 +298,50 @@ void sked(void){
 	if (band) safe_strncpy0(qrg_str, band->skedqrg, EQSO_LEN);
 	
 
-    if (!(d = mem_alloc(sizeof(struct dialog) + 15 * sizeof(struct dialog_item)))) return;
+    if (!(d = (struct dialog*)g_malloc(sizeof(struct dialog) + 15 * sizeof(struct dialog_item)))) return;
     memset(d, 0, sizeof(struct dialog) + 15 * sizeof(struct dialog_item));
     d->title = VTEXT(T_SEND_SKED);
-    d->fn = sked_fn;
+	d->fn = dlg_pf_fn;
+    d->fn2 = sked_fn;
     d->refresh = (void (*)(void *))refresh_sked;
     d->refresh_data = NULL;
-
-    d->items[i=0].type = D_FIELD;
-    d->items[i].dlen = EQSO_LEN;
-    d->items[i].data = qrg_str;
-    
-    d->items[++i].type = D_CHECKBOX;
-    d->items[i].gid = 0;
-    d->items[i].gnum = 1;
-    d->items[i].dlen = sizeof(int);
-    d->items[i].data = (void *)&we_call;
-    
+	d->y0 = 1;
+	i = -1;
 
     d->items[++i].type = D_BUTTON;
     d->items[i].gid  = 0;
     d->items[i].fn   = dlg_pband;
     d->items[i].text = pband;
+	d->items[i].msg = CTEXT(T_BAND);
+	d->items[i].wrap = 1;
     
     d->items[++i].type = D_FIELD;
     d->items[i].dlen = EQSO_LEN;
-    d->items[i].data = time_str;
+    d->items[i].data = qrg_str;
+	d->items[i].msg = CTEXT(T_QRG);
+	d->items[i].wrap = 1;
     
-    d->items[++i].type = D_FIELD;
-    d->items[i].dlen = EQSO_LEN;
-    d->items[i].data = callsign;
-    
-    d->items[++i].type = D_FIELD;   /* 5 */
-    d->items[i].dlen = EQSO_LEN;
-    d->items[i].data = locator;
-   
     d->items[++i].type = D_FIELD;
     d->items[i].dlen = 61;
     d->items[i].data = remark;
     d->items[i].maxl = EQSO_LEN;
+	d->items[i].msg = CTEXT(T_REMARK),
+	d->items[i].wrap = 1;
     
+    d->items[++i].type = D_FIELD;
+    d->items[i].dlen = EQSO_LEN;
+    d->items[i].data = callsign;
+	d->items[i].msg = CTEXT(T_CALLSIGN2);
+	d->items[i].wrap = 1;
     
+    d->items[++i].type = D_FIELD;
+    d->items[i].dlen = EQSO_LEN;
+    d->items[i].data = locator;
+	d->items[i].msg = CTEXT(T_WWL);
+	d->items[i].wrap = 1;
+   
+    
+	d->items[i].wrap++;
     d->items[++i].type = D_BUTTON;
     d->items[i].gid = B_ENTER;
     d->items[i].fn = ok_dialog;
@@ -383,6 +351,8 @@ void sked(void){
     d->items[i].gid = B_ESC;
     d->items[i].fn = cancel_dialog;
     d->items[i].text = VTEXT(T_CANCEL);
+	d->items[i].align = AL_BUTTONS;
+	d->items[i].wrap = 1;
     
     d->items[++i].type = D_END;
     do_dialog(d, getml(d, NULL));
@@ -394,15 +364,14 @@ void sked_from_tmpqso(struct tmpqso *tmpqso)
     struct tm utc;
 
     time(&now);
-    now+=cfg->skedshift*60;
+    //now+=cfg->skedshift*60;
     gmtime_r(&now, &utc);
            
     strcpy(qrg_str,"");
-    we_call=0;
     /* pband */
     g_snprintf(time_str, EQSO_LEN, "%d.%02d", utc.tm_hour, utc.tm_min);
-    safe_strncpy0(callsign, tmpqso->callsign, EQSO_LEN); uc(callsign);
-    safe_strncpy0(locator,  tmpqso->locator,  EQSO_LEN); uc(locator);
+    safe_strncpy0(callsign, tmpqso->callsign, EQSO_LEN); z_str_uc(callsign);
+    safe_strncpy0(locator,  tmpqso->locator,  EQSO_LEN); z_str_uc(locator);
     strcpy(remark, "");
 
     sked();
@@ -414,15 +383,14 @@ void sked_from_qso(struct qso *qso)
     struct tm utc;
 
     time(&now);
-    now+=cfg->skedshift*60;
+    //now+=cfg->skedshift*60;
     gmtime_r(&now, &utc);
            
     strcpy(qrg_str,"");
-    we_call=0;
     /* pband */
     g_snprintf(time_str, EQSO_LEN, "%d.%02d", utc.tm_hour, utc.tm_min);
-    safe_strncpy0(callsign, qso->callsign, EQSO_LEN); uc(callsign);
-    safe_strncpy0(locator,  qso->locator,  EQSO_LEN); uc(locator);
+    safe_strncpy0(callsign, qso->callsign, EQSO_LEN); z_str_uc(callsign);
+    safe_strncpy0(locator,  qso->locator,  EQSO_LEN); z_str_uc(locator);
     strcpy(remark, "");
 
     sked();
@@ -430,11 +398,11 @@ void sked_from_qso(struct qso *qso)
 
 
 
-void sked_pband_func (cba_t cba){
+void sked_pband_func(void *arg){
     struct band *band;
     int active;
 
-    active=(int)GETCBA(cba, int_);
+    active=GPOINTER_TO_INT(arg);
     dbg("sked_pband_func(%d)\n", active);
     if (active<0 || active>=ctest->bands->len) return;
     
@@ -454,12 +422,12 @@ void sked_pband(int apband_int)
     if (!(mi = new_menu(1))) return;
     for (i = 0; i < ctest->bands->len; i++) {
         band = (struct band *)g_ptr_array_index(ctest->bands, i);
-        add_to_menu(&mi, band->pband, "", "", MENU_FUNC sked_pband_func, (cba_t)i, 0);
+        add_to_menu(&mi, band->pband, "", "", MENU_FUNC sked_pband_func, GINT_TO_POINTER(i), 0);
     }
     sel = apband_int;
     if (sel < 0) sel = 0;
     if (sel>=ctest->bands->len) sel=0;
-    do_menu_selected(mi, CBA0, sel);
+    do_menu_selected(mi, NULL, sel);
 }
 
 
@@ -470,12 +438,9 @@ int dlg_pband(struct dialog_data *dlg, struct dialog_item_data *di){
 
 struct zstring *sked_format(struct sked *sked){
     struct zstring *zs;
-    char we_call_str[20];
-
-    g_snprintf(we_call_str, 16, "%d", sked->we_call);
 
     zs = zconcatesc(sked->sked_time, sked->operator_, sked->src_shortpband, sked->qrg,
-            we_call_str, sked->pband, sked->time_str, sked->callsign,
+			"0", sked->pband, sked->time_str, sked->callsign,
             sked->locator, sked->remark, NULL);
 
     return zs;
@@ -496,7 +461,7 @@ struct sked *sked_parse(struct zstring *zstr){
     sked->operator_ = g_strdup(ztokenize(zstr, 0));
     sked->src_shortpband = g_strdup(ztokenize(zstr, 0));
     sked->qrg = g_strdup(ztokenize(zstr, 0));
-    sked->we_call = atoi(ztokenize(zstr, 0));
+    ztokenize(zstr, 0); // we_call
     sked->pband = g_strdup(ztokenize(zstr, 0));
     sked->time_str = g_strdup(ztokenize(zstr, 0));
     sked->callsign = g_strdup(ztokenize(zstr, 0));
@@ -559,10 +524,8 @@ void draw_skedwindow(struct skedwin_data *skedwdata){
 }
 
 void skedwindow_func(struct window *win, struct event *ev, int fwd){
-    struct terminal *term;
     struct skedwin_data *skedwdata;
     
-    term = win->term;
     skedwdata = (struct skedwin_data *)win->data;
     
 //    dbg("skedwindow_func [%d,%d,%d,%d]\n",ev->ev,ev->x,ev->y,ev->b);
@@ -602,7 +565,7 @@ void skedwindow_func(struct window *win, struct event *ev, int fwd){
     }
 }
 
-void skedw_timer(cba_t cba){
+void skedw_timer(void *arg){
     struct window *w;
     if (!gses) return;
 
@@ -618,15 +581,15 @@ void skedw_timer(cba_t cba){
 
 void skedw_create(struct band *b){
     if (cfg->skedcount <= 0) return;
-    if (gses->skedw_timer_id > 0) kill_timer(gses->skedw_timer_id); 
-    gses->skedw_timer_id = install_timer(5000, skedw_timer, CBA0);
+    if (gses->skedw_timer_id > 0) zselect_timer_kill(zsel, gses->skedw_timer_id); 
+    gses->skedw_timer_id = zselect_timer_new(zsel, 5000, skedw_timer, NULL);
 
     if (gses->skedwdata) {
         gses->skedwdata->band = b;
         return;
     }
 
-    gses->skedwdata = mem_alloc(sizeof(struct skedwin_data));
+    gses->skedwdata = g_malloc(sizeof(struct skedwin_data));
     memset(gses->skedwdata, 0, sizeof(struct skedwin_data));
     gses->skedwdata->band = b;
             
