@@ -1,6 +1,6 @@
 /*
     Tucnak - VHF contest log
-    Copyright (C) 2002-2006  Ladislav Vaiz <ok1zia@nagano.cz>
+    Copyright (C) 2002-2015  Ladislav Vaiz <ok1zia@nagano.cz>
 
     This program is free software; you can redistribute it and/or
     modify it under the terms of the GNU General Public License
@@ -8,7 +8,19 @@
 
 */
 
+
+#include "bfu.h"
+#include "cwdb.h"
+#include "dwdb.h"
+#include "excdb.h"
+#include "fifo.h"
 #include "header.h"
+#include "main.h"
+#include "misc.h"
+#include "qsodb.h"
+#include "session.h"
+#include "stats.h"
+#include "tsdl.h"
 
 char *get_wwl(char *buf, char *wwl){
     safe_strncpy(buf, wwl,5);
@@ -115,8 +127,8 @@ void free_stats(struct stats *st){
     g_hash_table_destroy(st->excs);
     g_hash_table_destroy(st->calls);
     g_hash_table_destroy(st->hours);
-    CONDGFREE(st->mycontinent);
-    CONDGFREE(st->mydxcc);
+    zg_free0(st->mycontinent);
+    zg_free0(st->mydxcc);
     g_free(st);
     
 }
@@ -151,11 +163,11 @@ void clear_stats(struct stats *st){
     st->odxqrb_int = 0;
 
     dxi = get_dw_item_by_call(dw, ctest->pcall);
-    if (!dxi) internal_("unknown dxcc of my callsign");
+    if (!dxi) zinternal("unknown dxcc of my callsign");
 
-    CONDGFREE(st->mycontinent);
+    zg_free0(st->mycontinent);
     st->mycontinent = g_strdup(dxi->continent);
-    CONDGFREE(st->mydxcc);
+    zg_free0(st->mydxcc);
     st->mydxcc = g_strdup(get_dxcc(dw, s, ctest->pcall));
 }
 
@@ -173,7 +185,9 @@ int find_preferred_dxc(int type, char *dxc){
 			prefs=prefs9;
 			multi=3;
 			break;
-		case 10: 
+		case 10:
+		case 17:
+		case 18:
 			prefs=prefs10;
 			multi=5;
 			break;
@@ -185,7 +199,6 @@ int find_preferred_dxc(int type, char *dxc){
             prefs=prefs16;
             multi=4;
             break;
-
 		default:
 			return 1;	
 	}
@@ -241,15 +254,15 @@ void update_stats(struct band *band, struct stats *st, struct qso *q){
 /*    dbg("update_stats(%s) calls->size=%d \n", q->qsonrs, g_hash_table_size(st->calls));
     dbg("   dupe=%d  qsop=%d \n", q->dupe, q->qsop);*/
     
-    get_raw_call(s, q->callsign);
-    if (is_rover(q->callsign) && q->locator){
+    z_get_raw_call(s, q->callsign);
+    if (z_call_is_rover(q->callsign) && q->locator){
         strcat(s, "_");
         safe_strncpy0(s + strlen(s), q->locator, 4+1);
     } 
 
     //dbg("update_stats(%s)\n", s);    
 
-    if (g_hash_table_lookup_extended(st->calls, s, &orig_key, (gpointer) &orig_value)){
+    if (g_hash_table_lookup_extended(st->calls, s, &orig_key, (gpointer*) &orig_value)){
         SAD_INT(dupe,1);
     }else{
         g_hash_table_insert(st->calls, g_strdup(s), NULL);
@@ -258,39 +271,39 @@ void update_stats(struct band *band, struct stats *st, struct qso *q){
     
     switch (ctest->qsop_method){
         case 1: /* rounded distance */
-            qsop = iaru_round(q->qrb) * ctest->qsomult;
+			qsop = iaru_round(q->qrb) * ctest->qsomult * band->qsomultb;
             break;
         case 2: /* 1 */
-            qsop = ctest->qsomult;
+            qsop = ctest->qsomult * band->qsomultb;
             break;
 		case 3: /* FIXME! it's really correct? */
-			qsop = (int)(((q->qrb/50)*2+1 ) * ctest->qsomult);
+			qsop = (int)(((q->qrb/50)*2+1 ) * ctest->qsomult * band->qsomultb);
 			break;	
         case 4: /* WWL ring */
-            qsop = qsopwr(ctest->pwwlo, q->locator) * ctest->qsomult;
+            qsop = qsopwr(ctest->pwwlo, q->locator) * ctest->qsomult * band->qsomultb;
             break;
 		case 5: /* statute miles (anglicka mile) */
-			qsop = (int)(q->qrb/1.6094 * ctest->qsomult);
+			qsop = (int)(q->qrb/1.6094 * ctest->qsomult * band->qsomultb);
 			break;
 		case 6: /* 1 * qso-points multi + 1 point for CW QSO FIXME! bug in taclog? */
-			qsop = ctest->qsomult;
+			qsop = ctest->qsomult * band->qsomultb;
 			if (q->mode==MOD_CW_CW) q->qsop++;
 			break;
 		case 7: /* distance(min 50) */	
-			qsop = (int)((q->qrb>50 ? q->qrb : 50) * ctest->qsomult);
+			qsop = (int)((q->qrb>50 ? q->qrb : 50) * ctest->qsomult * band->qsomultb);
 			break;
 		case 8: /* distance(min 10) */	
-			qsop = (int)( (q->qrb>10 ? q->qrb : 10) * ctest->qsomult );
+			qsop = (int)( (q->qrb>10 ? q->qrb : 10) * ctest->qsomult * band->qsomultb );
 			break;
 		case 9:
 		case 10:
 		case 11:
-			qsop = ctest->qsomult * 
+			qsop = ctest->qsomult * band->qsomultb * 
 				find_preferred_dxc(ctest->qsop_method,
 						get_dxcc(dw,s,q->callsign));
 			break;
         case 12:
-            qsop = iaru_round(q->qrb) * ctest->qsomult;
+            qsop = iaru_round(q->qrb) * ctest->qsomult * band->qsomultb;
             break;
         case 13:
 //            dbg("call=%s \n", q->callsign);
@@ -354,11 +367,18 @@ void update_stats(struct band *band, struct stats *st, struct qso *q){
                 qsop = 1;    // 144 MHz and below
             break;
         case 16:
-            qsop = iaru_round(q->qrb) * ctest->qsomult *
+            qsop = iaru_round(q->qrb) * ctest->qsomult * band->qsomultb *
 				find_preferred_dxc(ctest->qsop_method,
 						get_dxcc(dw,s,q->callsign));
             qsop *= qsomult16(band);
             break;
+		case 17:
+			qsop = iaru_round(q->qrb) * ctest->qsomult * band->qsomultb;
+			break;
+		case 18:
+			qsop = iaru_round(q->qrb) * ctest->qsomult * band->qsomultb;
+			break;
+
     }
     if (q->dupe) qsop=0;
     SAD_INT(qsop,qsop);
@@ -386,23 +406,42 @@ void update_stats(struct band *band, struct stats *st, struct qso *q){
     anew=q->new_;
 
     if (ctest->wwlused){    
+		int accept_loc = 1;
+		int double_loc = 0;
+		char tmpdxcc[20];
+
         get_wwl(s, q->locator);
-        if (g_hash_table_lookup_extended(st->wwls, s, &orig_key, (gpointer) &orig_value)){
-            orig_value->count++; 
-			orig_value->points+=qsop;
-            anew &= ~NEW_WWL;
-        }else{
-            orig_value = g_new0(struct cntpts, 1);
-            orig_value->count = 1;
-            orig_value->points = qsop;
-            g_hash_table_insert(st->wwls, g_strdup(s), orig_value);
-            anew |= NEW_WWL;
-        }
+		if (ctest->qsop_method == 17){
+			if (find_preferred_dxc(17, get_dxcc(dw, tmpdxcc, q->callsign)) == 1) accept_loc = 0;
+		}
+		if (ctest->qsop_method == 18){
+			accept_loc = 1;
+			if (find_preferred_dxc(18, get_dxcc(dw, tmpdxcc, q->callsign)) != 1) double_loc = 1;
+		}
+		if (accept_loc){
+			if (g_hash_table_lookup_extended(st->wwls, s, &orig_key, (gpointer*) &orig_value)){
+				orig_value->count++; 
+				orig_value->points+=qsop;
+				anew &= ~NEW_WWL;
+			}else{
+				orig_value = g_new0(struct cntpts, 1);
+				orig_value->count = 1;
+				orig_value->points = qsop;
+				g_hash_table_insert(st->wwls, g_strdup(s), orig_value);
+				if (double_loc) {
+					orig_value = g_new0(struct cntpts, 1);
+					orig_value->count = 0;
+					orig_value->points = 0;
+					g_hash_table_insert(st->wwls, g_strdup_printf("%s_M7", s), orig_value);
+				}
+				anew |= NEW_WWL;
+			}
+		}
     }
     
  
     get_dxcc(dw, s, q->callsign);
-    if (g_hash_table_lookup_extended(st->dxcs, s, &orig_key, (gpointer) &orig_value)){
+    if (g_hash_table_lookup_extended(st->dxcs, s, &orig_key, (gpointer*) &orig_value)){
         orig_value->count++; 
 		orig_value->points+=qsop;
         anew &= ~NEW_DXC;
@@ -416,7 +455,7 @@ void update_stats(struct band *band, struct stats *st, struct qso *q){
 
     if (ctest->prefmult>0){
         get_pref(s, q->callsign);
-        if (g_hash_table_lookup_extended(st->prefs, s, &orig_key, (gpointer) &orig_value)){
+        if (g_hash_table_lookup_extended(st->prefs, s, &orig_key, (gpointer*) &orig_value)){
             orig_value->count++; 
             orig_value->points+=qsop;
             anew &= ~NEW_PREF;
@@ -431,13 +470,13 @@ void update_stats(struct band *band, struct stats *st, struct qso *q){
         
         
 
-    if (ctest->excused>0){
+    if (ctest->excused > 0 && ctest->excname){
         safe_strncpy0(s, q->exc, 20);
         if (strcasecmp(ctest->excname, "USACA")==0){
             usaca_multiple_exc(s);
         }
         if (*s){
-            if (g_hash_table_lookup_extended(st->excs, s, &orig_key, (gpointer) &orig_value)){
+            if (g_hash_table_lookup_extended(st->excs, s, &orig_key, (gpointer*) &orig_value)){
                 orig_value->count++; 
                 orig_value->points+=qsop;
                 anew &= ~NEW_EXC;
@@ -463,7 +502,7 @@ void update_stats(struct band *band, struct stats *st, struct qso *q){
     } 
 
     safe_strncpy0(s, q->time_str, 3);
-	if (!g_hash_table_lookup_extended(st->hours, s, &orig_key, (gpointer) &orig_value)){
+	if (!g_hash_table_lookup_extended(st->hours, s, &orig_key, (gpointer*) &orig_value)){
 		orig_value = g_new0(struct cntpts, 1);
 		g_hash_table_insert(st->hours, g_strdup(s), orig_value);
     }
@@ -586,6 +625,7 @@ void redraw_stats(struct band *band){
     gchar *c;
     struct stats *st=band->stats;    
     int nqsos, nqsop, nwwlp, nexcp, ndxcp, ntotal;
+	GString *gs;
     
     MUTEX_LOCK(band->stats);
 
@@ -616,13 +656,11 @@ void redraw_stats(struct band *band){
         g_free(c);
     }    
     print_text(STATS_X,STATS_Y+2,term->x-QSONR_WIDTH-BAND_WIDTH-3,VTEXT(T_SES_STAT_TIT),COL_NORM);
-    c = g_strdup_printf("%4d  %6d  %4d  %4d   %4d  %7d %6.1f",
-            nqsos, nqsop, 
-            nexcp, nwwlp, ndxcp,
-            ntotal,
-            (double)ntotal/(double)nqsos);
-    print_text(STATS_X,STATS_Y+3,term->x-QSONR_WIDTH-BAND_WIDTH-3,c,COL_NORM);
-    g_free(c);
+	gs = g_string_sized_new(100);
+	g_string_sprintfa(gs, "%4d  %6d  %4d  %4d   %4d  %7d ", nqsos, nqsop, nexcp, nwwlp, ndxcp, ntotal);
+	if (nqsos != 0) g_string_sprintfa(gs, "%6.1f", (double)ntotal/(double)nqsos);
+    print_text(STATS_X,STATS_Y+3,term->x-QSONR_WIDTH-BAND_WIDTH-3,gs->str,COL_NORM);
+    g_string_free(gs, TRUE);
 
 /*    if (st->nqsos){
         c = g_strdup_printf(VTEXT(T_SES_AVG), (double)st->ntotal/(double)st->nqsos);
@@ -639,14 +677,17 @@ void redraw_stats(struct band *band){
 void sf_wwl_func(gpointer key, gpointer value, gpointer user_data){
     gchar       *wwl;
     struct cntpts *n;
-    GIndexArray *ia;
+    ZPtrArray *ia;
     
     
     wwl   = (gchar *)       key;  
 	n     = (struct cntpts *) value;
-    ia    = (GIndexArray *) user_data;
+    ia    = (ZPtrArray *) user_data;
 
-    g_index_array_add(ia, g_strdup_printf("%4s:%6d %3d    ", wwl, n->points, n->count));
+	if (ctest->qsop_method == 18)
+		z_ptr_array_add(ia, g_strdup_printf("%7s:%6d %3d    ", wwl, n->points, n->count));
+	else
+		z_ptr_array_add(ia, g_strdup_printf("%4s:%6d %3d    ", wwl, n->points, n->count));
 }
 
 
@@ -662,7 +703,7 @@ void sf_miss_exc_func(gpointer key, gpointer value, gpointer user_data){
 
     maxlen = strlen(missexc);
     if (maxlen > ms->maxlen) ms->maxlen = maxlen;
-    g_index_array_add(ms->ia, g_strdup(missexc));
+    z_ptr_array_add(ms->ia, g_strdup(missexc));
 }
 
 
@@ -683,32 +724,29 @@ gint compare_colon_int (gconstpointer a, gconstpointer b){
 	return ib-ia;
 }
 
-gint compare_string (gconstpointer a, gconstpointer b){
-	gchar **ca, **cb;
-
-	ca=(gchar **)a;
-	cb=(gchar **)b;
-
-	return strcmp(*ca, *cb);
-}
 
 #define COLS 4
 void add_sf_wwls(struct band *b){
     GString *gs;
-    GIndexArray *ia;
+    ZPtrArray *ia;
     int i,j,k,lines;
     gchar *c;
     
     gs = g_string_new("");
     fifo_addf(F, "");
-    fifo_addf(F, VTEXT(T_SF_WWLS), g_hash_table_size(b->stats->wwls));
-    fifo_addf(F, VTEXT(T_SF_WWLS_UNDER));
+	if (ctest->qsop_method == 17 || ctest->qsop_method == 18){
+		fifo_addf(F, VTEXT(T_MULTIPLIED_WWLS_D), g_hash_table_size(b->stats->wwls));
+	    fifo_addf(F, VTEXT(T_HYPHENS1));
+	}else{
+		fifo_addf(F, VTEXT(T_SF_WWLS), g_hash_table_size(b->stats->wwls));
+		fifo_addf(F, VTEXT(T_SF_WWLS_UNDER));
+	}
    /* for (i=0;i<COLS;i++) g_string_append(gs, " WWL  QSO-p cnt    ");
 	fifo_addf(F, gs->str);*/
     
-    ia = g_index_array_new();
+    ia = z_ptr_array_new();
     g_hash_table_foreach(b->stats->wwls, sf_wwl_func, ia);
-    g_index_array_qsort(ia, compare_colon_int);
+    z_ptr_array_qsort(ia, compare_colon_int);
     g_string_truncate(gs, 0);
     j = 0;
     lines=(ia->len+COLS-1)/COLS;
@@ -717,14 +755,14 @@ void add_sf_wwls(struct band *b){
         for (j=0;j<COLS; j++){
             k = i+j*lines;
             if (k>=ia->len) continue;
-            c = (gchar *)g_index_array_index(ia, k);
+            c = (gchar *)z_ptr_array_index(ia, k);
             g_string_append(gs, c);        
         }
         fifo_addf(F, "%s", gs->str);
         g_string_truncate(gs, 0);
     }
     g_string_free(gs,1);
-    g_index_array_free_all(ia);
+    z_ptr_array_free_all(ia);
 }
 
 ////////////
@@ -732,19 +770,19 @@ void add_sf_wwls(struct band *b){
 void sf_exc_func(gpointer key, gpointer value, gpointer user_data){
     gchar       *exc;
     struct cntpts *n;
-    GIndexArray *ia;
+    ZPtrArray *ia;
     
     
     exc   = (gchar *)       key;  
 	n     = (struct cntpts *) value;
-    ia    = (GIndexArray *) user_data;
+    ia    = (ZPtrArray *) user_data;
 
-    g_index_array_add(ia, g_strdup_printf("%4s:%6d %3d    ", exc, n->points, n->count));
+    z_ptr_array_add(ia, g_strdup_printf("%4s:%6d %3d    ", exc, n->points, n->count));
 }
 
 void add_sf_excs(struct band *b){
     GString *gs;
-    GIndexArray *ia;
+    ZPtrArray *ia;
     int i,j,k,lines;
     gchar *c;
     
@@ -755,9 +793,9 @@ void add_sf_excs(struct band *b){
    /* for (i=0;i<COLS;i++) g_string_append(gs, " EXC  QSO-p cnt    ");
 	fifo_addf(F, gs->str);*/
     
-    ia = g_index_array_new();
+    ia = z_ptr_array_new();
     g_hash_table_foreach(b->stats->excs, sf_wwl_func, ia);
-    g_index_array_qsort(ia, compare_colon_int);
+    z_ptr_array_qsort(ia, compare_colon_int);
     g_string_truncate(gs, 0);
     j = 0;
     lines=(ia->len+COLS-1)/COLS;
@@ -766,29 +804,29 @@ void add_sf_excs(struct band *b){
         for (j=0;j<COLS; j++){
             k = i+j*lines;
             if (k>=ia->len) continue;
-            c = (gchar *)g_index_array_index(ia, k);
+            c = (gchar *)z_ptr_array_index(ia, k);
             g_string_append(gs, c);        
         }
         fifo_addf(F, "%s", gs->str);
         g_string_truncate(gs, 0);
     }
     g_string_free(gs,1);
-    g_index_array_free_all(ia);
+    z_ptr_array_free_all(ia);
 }
 
 #define MISS_EXC_COLS 10
 
 void miss_compute_ms(struct miss_struct *ms, struct band *b){
-    ms->ia = g_index_array_new();
+    ms->ia = z_ptr_array_new();
     ms->st = b->stats;
     ms->maxlen = 0;
     g_hash_table_foreach(excdb->vexc, sf_miss_exc_func, ms);
-    g_index_array_qsort(ms->ia, compare_string);
+    z_ptr_array_qsort(ms->ia, z_compare_string);
 }
 
 void add_sf_miss_excs(struct band *b){
     GString *gs;
-//    GIndexArray *ia;
+//    ZPtrArray *ia;
     int i,j,k,lines;
     gchar *c;
     struct miss_struct ms;
@@ -805,40 +843,40 @@ void add_sf_miss_excs(struct band *b){
     j = 0;
     lines=(ms.ia->len+MISS_EXC_COLS-1)/MISS_EXC_COLS;
     
-    dbg("lines=%d\n", lines);
+    //dbg("lines=%d\n", lines);
     for (i=0; i<lines ; i++){
         for (j=0;j<MISS_EXC_COLS; j++){
             k = i+j*lines;
             if (k>=ms.ia->len) continue;
-            c = (gchar *)g_index_array_index(ms.ia, k);
+            c = (gchar *)z_ptr_array_index(ms.ia, k);
             g_string_sprintfa(gs, "%-*s", ms.maxlen+1, c);        
         }
         fifo_addf(F, "%s", gs->str);
         g_string_truncate(gs, 0);
     }
     g_string_free(gs,1);
-    g_index_array_free_all(ms.ia);
+    z_ptr_array_free_all(ms.ia);
 }
 
 ////////////////////
 void sf_dxc_func(gpointer key, gpointer value, gpointer user_data){
     gchar       *dxc;
     struct cntpts *n;
-    GIndexArray *ia;
+    ZPtrArray *ia;
     
     
     dxc   = (gchar *)       key;  
 	n     = (struct cntpts *) value;
-    ia    = (GIndexArray *) user_data;
+    ia    = (ZPtrArray *) user_data;
 
-    g_index_array_add(ia, g_strdup_printf("%4s:%6d %3d    ", dxc, n->points, n->count));
+    z_ptr_array_add(ia, g_strdup_printf("%4s:%6d %3d    ", dxc, n->points, n->count));
 }
 
 #undef COLS
 #define COLS 4
 void add_sf_dxcs(struct band *b){
     GString *gs;
-    GIndexArray *ia;
+    ZPtrArray *ia;
     int i,j,k,lines;
     gchar *c;
     
@@ -849,9 +887,9 @@ void add_sf_dxcs(struct band *b){
 /*	for (i=0;i<COLS;i++) g_string_append(gs, " DXCC QSO-p cnt    ");
 	fifo_addf(F, gs->str);*/
     
-    ia = g_index_array_new();
+    ia = z_ptr_array_new();
     g_hash_table_foreach(b->stats->dxcs, sf_dxc_func, ia);
-    g_index_array_qsort(ia, compare_colon_int);
+    z_ptr_array_qsort(ia, compare_colon_int);
     g_string_truncate(gs, 0);
     j = 0;
     lines = (ia->len+COLS-1)/COLS;
@@ -859,19 +897,19 @@ void add_sf_dxcs(struct band *b){
         for (j=0;j<COLS; j++){
             k = i+j*lines;
             if (k>=ia->len) continue;
-            c = (gchar *)g_index_array_index(ia, k);
+            c = (gchar *)z_ptr_array_index(ia, k);
             g_string_append(gs, c);        
         }
         fifo_addf(F, "%s", gs->str);
         g_string_truncate(gs, 0);
     }
     g_string_free(gs,1);
-    g_index_array_free_all(ia);
+    z_ptr_array_free_all(ia);
 }
 
 void add_sf_prefs(struct band *b){
     GString *gs;
-    GIndexArray *ia;
+    ZPtrArray *ia;
     int i,j,k,lines;
     gchar *c;
     GHashTable *hash;
@@ -889,9 +927,9 @@ void add_sf_prefs(struct band *b){
 /*	for (i=0;i<COLS;i++) g_string_append(gs, " DXCC QSO-p cnt    ");
 	fifo_addf(F, gs->str);*/
     
-    ia = g_index_array_new();
+    ia = z_ptr_array_new();
     g_hash_table_foreach(hash, sf_dxc_func, ia);
-    g_index_array_qsort(ia, compare_colon_int);
+    z_ptr_array_qsort(ia, compare_colon_int);
     g_string_truncate(gs, 0);
     j = 0;
     lines = (ia->len+COLS-1)/COLS;
@@ -899,14 +937,14 @@ void add_sf_prefs(struct band *b){
         for (j=0;j<COLS; j++){
             k = i+j*lines;
             if (k>=ia->len) continue;
-            c = (gchar *)g_index_array_index(ia, k);
+            c = (gchar *)z_ptr_array_index(ia, k);
             g_string_append(gs, c);        
         }
         fifo_addf(F, "%s", gs->str);
         g_string_truncate(gs, 0);
     }
     g_string_free(gs,1);
-    g_index_array_free_all(ia);
+    z_ptr_array_free_all(ia);
 }
 
 void add_sf_wkd_wwls(struct band *b){
@@ -1086,14 +1124,14 @@ void sf_hour_max_func(gpointer key, gpointer value, gpointer user_data){
 void sf_hour_func(gpointer key, gpointer value, gpointer user_data){
     gchar       *hour;
     struct cntpts *n;
-    GIndexArray *ia;
+    ZPtrArray *ia;
 	int h, i, l, avg;
     char ps[40], cs[40];
     
     
     hour   = (gchar *)       key;  
 	n     = (struct cntpts *) value;
-    ia    = (GIndexArray *) user_data;
+    ia    = (ZPtrArray *) user_data;
 
 	h=atoi(hour)+1;
 	if (h==25) h=0;
@@ -1112,12 +1150,12 @@ void sf_hour_func(gpointer key, gpointer value, gpointer user_data){
     else avg=0;
     
 	
-    g_index_array_add(ia, g_strdup_printf("%2s-%02d: [%s]%6d  :  [%s]%3d  : %4d  odx %s", hour, h,ps, n->points, cs, n->count, avg, n->qso?n->qso->callsign:"")); /* n->qso should not be nul but for sure... */
+    z_ptr_array_add(ia, g_strdup_printf("%2s-%02d: [%s]%6d  :  [%s]%3d  : %4d  odx %s", hour, h,ps, n->points, cs, n->count, avg, n->qso?n->qso->callsign:"")); /* n->qso should not be nul but for sure... */
 }
 
 #undef COLS
 void add_sf_hours(struct band *b){
-    GIndexArray *ia;
+    ZPtrArray *ia;
     int i/*,j,k,lines*/;
     gchar *c;
     
@@ -1126,17 +1164,17 @@ void add_sf_hours(struct band *b){
     fifo_addf(F, VTEXT(T_SF_HOURS_UNDER));
     fifo_addf(F, VTEXT(T_SF_HOURS_TITLE));
     
-    ia = g_index_array_new();
+    ia = z_ptr_array_new();
     maxhourpoints=0;
     maxhourcount=0;
     g_hash_table_foreach(b->stats->hours, sf_hour_max_func, ia);
     g_hash_table_foreach(b->stats->hours, sf_hour_func, ia);
-    g_index_array_qsort(ia, compare_hour14);
+    z_ptr_array_qsort(ia, compare_hour14);
     for (i=0; i<ia->len ; i++){
-        c = (gchar *)g_index_array_index(ia, i);
+        c = (gchar *)z_ptr_array_index(ia, i);
         fifo_addf(F, "%s", c);
     }
-    g_index_array_free_all(ia);
+    z_ptr_array_free_all(ia);
 }
 
 
@@ -1151,6 +1189,7 @@ void recalc_statsfifo(struct band *b){
     F = init_fifo(1000);
     F->withouttime = 1;
 
+	// html_band_header reads from row 5
     fifo_addf(F, "");
     fifo_addf(F, VTEXT(T_SF_STATS_BAND), b->bandname);
     fifo_addf(F, VTEXT(T_SF_STATS_BAND_UNDER));
@@ -1169,13 +1208,13 @@ void recalc_statsfifo(struct band *b){
     if (b->stats->nqsos>0){
         fifo_addf(F, VTEXT(T_SF_AVG_PTS), ((double)b->stats->ntotal) / ((double)b->stats->nqsos));
         fifo_addf(F, VTEXT(T_SF_AVG_QSOP), ((double)b->stats->nqsop) / ((double)b->stats->nqsos));
-		fifo_addf(F, VTEXT(T_SF_QSO_HOUR), b->stats->qso_per_hour);
-		fifo_addf(F, VTEXT(T_SF_QSOP_HOUR), b->stats->pts_per_hour);
+		if (b->stats->qso_per_hour > 0) fifo_addf(F, VTEXT(T_SF_QSO_HOUR), b->stats->qso_per_hour);
+		if (b->stats->pts_per_hour > 0) fifo_addf(F, VTEXT(T_SF_QSOP_HOUR), b->stats->pts_per_hour);
 		fifo_addf(F, VTEXT(T_SF_QSOP_50QSO), b->stats->pts_per_50qso);
     }
     fifo_addf(F, "");
 
-	if (ctest->qsop_method==4){
+	if (ctest->qsop_method==4 || ctest->wwlmult != 0 || ctest->wwlbonu != 0){
     	add_sf_wkd_wwls(b);
     	add_sf_cnt_in_wwls(b);
 	}
@@ -1199,11 +1238,13 @@ void export_stats_fifo(void) {
 	gchar *fname;
 	int i;
 	
+	progress(VTEXT(T_EXPORTING));
+
 	fname = NULL;
 	for (i = 0; 1; i++) {
 		struct stat st;
 		
-		fname = g_strdup_printf("%s/stat%c%d.txt", ctest->directory, aband->bandchar, i);
+		fname = g_strdup_printf("%s/stat%c%d.txt", ctest->directory, z_char_lc(aband->bandchar), i);
 		if (stat(fname, &st) != 0)
 			break;
 
@@ -1215,14 +1256,16 @@ void export_stats_fifo(void) {
 	save_fifo_to_file(aband->statsfifo1, fname);
 	
 	g_free(fname);
+	progress(NULL);
 }
 
 gpointer stats_thread_func(gpointer data){
     struct qso *q;
-    int i, ret;
+    int i;
     struct band *band=(struct band*)data;
     struct stats *tmp;
-    char s[256];
+
+	zg_thread_set_name("Tucnak stats");
 
 /*    ST_START;*/
     clear_stats(band->tmpstats);
@@ -1242,20 +1285,21 @@ gpointer stats_thread_func(gpointer data){
     
     band->dirty_save = 1;
     
-    sprintf(s, "ST\n");
-    ret = write(tpipe->threadpipe_write, s, strlen(s));
-    
+    //sprintf(s, "ST\n");
+    //ret = write(tpipe->threadpipe_write, s, strlen(s));
+	zselect_msg_send(zsel, "ST");
+                    
     band->dirty_stats = 0;
    /* dbg("stats done %c\n", band->bandchar);*/
     return 0;
 }
 
 void stats_thread_create(struct band *band){
-    if (band->stats_thread) internal_("stats_threadcreate band->stats_thread");
+    if (band->stats_thread) zinternal("stats_threadcreate band->stats_thread");
     /*dbg("stats_thread_create(%c)\n", band->bandchar);*/
     band->stats_break=0;
-    band->stats_thread=g_thread_create(stats_thread_func, (gpointer)band, TRUE, NULL);
-    if (!band->stats_thread) internal_("stats_threadcreate !band->stats_thread");
+    band->stats_thread=g_thread_try_new("stats", stats_thread_func, (gpointer)band, NULL);
+    if (!band->stats_thread) zinternal("stats_threadcreate !band->stats_thread");
 }
 
 void stats_thread_join(struct band *band){
@@ -1290,6 +1334,7 @@ void minute_stats(struct band *band){
     gmtime_r(&hb, &utc);
     sprintf(hourbefore, "%4d%02d%02d%02d%02d",1900+utc.tm_year, 1+utc.tm_mon, utc.tm_mday, utc.tm_hour, utc.tm_min);  
 
+
 	
 	if (band->qsos->len==0) return;
 	qso_per_hour=pts_per_hour=pts_per_50qso=0;
@@ -1397,9 +1442,16 @@ void minute_stats_all(void){
     }
 }
 
-void timer_minute_stats_all(cba_t cba){
+void timer_minute_stats_all(void *arg){
 	minute_stats_all();
-	ctest->minute_timer_id=install_timer(60000, timer_minute_stats_all, CBA0);
+	ctest->minute_timer_id=zselect_timer_new(zsel, 60000, timer_minute_stats_all, NULL);
+
+/*#ifdef Z_ANDROID
+    struct tm utc;
+	time_t now = time(NULL);
+    gmtime_r(&now, &utc);
+    dbg("%02d:%02d:%02dz", utc.tm_hour, utc.tm_min, utc.tm_sec);
+#endif*/
 }
 
 void stats_merge_prefs_fn(gpointer key, gpointer value, gpointer data){
