@@ -1,6 +1,6 @@
 /*
     Tucnak - VHF contest log
-    Copyright (C) 2002-2006  Ladislav Vaiz <ok1zia@nagano.cz>
+    Copyright (C) 2002-2015  Ladislav Vaiz <ok1zia@nagano.cz>
     and authors of web browser Links 0.96
 
     This program is free software; you can redistribute it and/or
@@ -9,14 +9,27 @@
 
 */
 
-
 #include "header.h"
+#include "charsets.h"
+#include "kbd.h"
+#include "main.h"
+#include "qsodb.h"
+#include "rc.h"
+#include "tsdl.h"
+#include "session.h"
+#include "terminal.h"
+#include "zosk.h"
 
+#define ALLOC_GR 0x100
 struct terminal *term;
 struct window *gseswin;
 
 int hard_write(int fd, char *p, int l)
 {
+#ifdef Z_MSC_MINGW
+	zinternal("hard_write unimplemented (and should not be called)");
+	return -1;
+#else
 	int w, t;
     if (fd<0) return -1;
     
@@ -32,10 +45,16 @@ int hard_write(int fd, char *p, int l)
         l -= w;
     }
     return t;
+#endif
 }
 
+
 int hard_read(int fd, char *p, int l)
 {
+#ifdef Z_MSC_MINGW
+	zinternal("hard_read unimplemented (and should not be called)");
+	return -1;
+#else
     int r = 1;
     int t = 0;
     while (l > 0 && r) {
@@ -49,6 +68,7 @@ int hard_read(int fd, char *p, int l)
         l -= r;
     }
     return t;
+#endif
 }
 
 char *get_cwd()
@@ -56,9 +76,9 @@ char *get_cwd()
     int bufsize = 128;
     char *buf;
     while (1) {
-        if (!(buf = mem_alloc(bufsize))) return NULL;
+        if (!(buf = (char *)g_malloc(bufsize))) return NULL;
         if (getcwd(buf, bufsize)) return buf;
-        mem_free(buf);
+        g_free(buf);
         if (errno == EINTR) continue;
         if (errno != ERANGE) return NULL;
         bufsize += 128;
@@ -75,32 +95,30 @@ void set_cwd(char *path)
 
 void alloc_term_screen(struct terminal *term, int x, int y)
 {
-    unsigned *s, *t;
-    if ((s = mem_realloc(term->screen, x * y * sizeof(unsigned)))) {
-        if ((t = mem_realloc(term->last_screen, x * y * sizeof(unsigned)))) {
-            memset(t, -1, x * y * sizeof(unsigned));
-            term->x = x;
-            term->y = y;
-            term->last_screen = t;
-            memset(s, 0, x * y * sizeof(unsigned));
-            term->screen = s;
-            term->dirty = 1;
-        }
-    }
+	g_free(term->last_screen);
+	g_free(term->screen);
+	term->screen = g_new(unsigned, x * y);
+	term->last_screen = g_new0(unsigned, x * y);
+	memset(term->last_screen, -1, x * y * sizeof(unsigned));
+    term->x = x;
+    term->y = y;
+	term->dirty = 1;
+    term->lcx = -1;
+    term->lcy = -1;
 #ifdef FALL
     int i;
     if (term->fall != NULL) {
         term->fall = mem_realloc(term->fall, x * y * sizeof(int));
         for (i = 0; i < x * y; i++) term->fall[i] = i;
     }else{
-        term->fall = mem_alloc(x * y * sizeof(int));
+        term->fall = g_malloc(x * y * sizeof(int));
         for (i = 0; i < x * y; i++) term->fall[i] = i;
     } 
 #endif
         
 }
 
-void in_term(cba_t);
+void in_term(void *arg);
 void check_if_no_terminal(void);
 
 void clear_terminal(void)
@@ -157,12 +175,12 @@ void fall_func(union cba_t cba){
 
         
     }
-    gses->fall_id = install_timer(10, fall_func, CBA0);
+    gses->fall_id = zselect_timer_new(zsel, 10, fall_func, NULL);
     redraw_later();
 }
 #endif
 
-void redraw_terminal(cba_t cba)
+void redraw_terminal(void *arg)
 {
     /*if (!term) return;*/
     //dbg("redraw_terminal()\n");
@@ -175,33 +193,27 @@ void redraw_terminal_all(void)
     redraw_terminal_ev(term, EV_RESIZE);
 }
 
-/*void timer_redraw(cba_t cba){
-    register_bottom_half(redraw_terminal, CBA0);
-    gses->redraw_timer_id = 0;
-} */
 
 void redraw_later(void)
 {       
     if (!term) return;
     //dbg("redraw_later()\n");
-    register_bottom_half(redraw_terminal, CBA0);
+    zselect_bh_new(zsel, redraw_terminal, NULL);
 
     /*if (!gses){
-        register_bottom_half(redraw_terminal, CBA0);
+        zselect_bh_new(zsel, redraw_terminal, NULL);
         return;
     }
     
     if (!gses->redraw_timer_id)
-        gses->redraw_timer_id = install_timer(100, timer_redraw, CBA0);*/
+        gses->redraw_timer_id = zselect_timer_new(zsel, 100, timer_redraw, NULL);*/
 }
 
 void erase_screen(void)
 {
     if (!term) return;
     if (!term->master || !is_blocked()) {
-        if (term->master) want_draw();
-        hard_write(term->fdout, "\033[2J\033[1;1H", 10);
-        if (term->master) done_draw();
+        if (term->fdout >= 0) hard_write(term->fdout, "\033[0;37;40m\033[2J\033[1;1H", 20);
     }
 }
 
@@ -257,8 +269,8 @@ void add_window_at_pos(void (*handler)(struct window *, struct event *, int), vo
     struct window *win;
     ev.x = term->x;
     ev.y = term->y;
-    if (!(win = mem_alloc(sizeof (struct window)))) {
-        mem_free(data);
+    if (!(win = g_malloc(sizeof (struct window)))) {
+        g_free(data);
         return;
     }
     win->handler = handler;
@@ -279,9 +291,9 @@ void delete_window(struct window *win)
     struct event ev = {EV_ABORT, 0, 0, 0};
     win->handler(win, &ev, 1);
     del_from_list(win);
-    if (win->data) mem_free(win->data);
-    redraw_terminal(CBA0);
-    mem_free(win);
+    if (win->data) g_free(win->data);
+    redraw_terminal(NULL);
+    g_free(win);
 }
 
 void delete_window_ev(struct window *win, struct event *ev)
@@ -311,7 +323,7 @@ void get_parent_ptr(struct window *win, int *x, int *y)
 struct window *get_root_window()
 {
     if (list_empty(term->windows)) {
-        internal_("terminal has no windows");
+        zinternal("terminal has no windows");
         return NULL;
     }
     return (struct window *)term->windows.prev;
@@ -326,7 +338,7 @@ struct ewd {
 void empty_window_handler(struct window *win, struct event *ev, int fwd)
 {
     struct window *n;
-    struct ewd *ewd = win->data;
+    struct ewd *ewd = (struct ewd*)win->data;
     int x, y;
     void (*fn)(void *) = ewd->fn;
     void *data = ewd->data;
@@ -352,7 +364,7 @@ void empty_window_handler(struct window *win, struct event *ev, int fwd)
 /*void add_empty_window(void (*fn)(void *), void *data)
 {
     struct ewd *ewd;
-    if (!(ewd = mem_alloc(sizeof(struct ewd)))) return;
+    if (!(ewd = g_malloc(sizeof(struct ewd)))) return;
     ewd->fn = fn;
     ewd->data = data;
     ewd->b = 0;
@@ -365,7 +377,7 @@ void free_term_specs()
     while (term_specs.next != &(term_specs)) {
         struct term_spec *a=term_specs.next; 
         del_from_list(a); 
-        mem_free(a); 
+        g_free(a); 
     }
 }
 
@@ -384,7 +396,7 @@ struct term_spec *new_term_spec(char *term)
 {
     struct term_spec *t;
     foreach(t, term_specs) if (!strcasecmp(t->term, term)) return t;
-    if (!(t = mem_alloc(sizeof(struct term_spec)))) return NULL;
+    if (!(t = g_new(struct term_spec, 1))) return NULL;
     memcpy(t, &dumb_term, sizeof(struct term_spec));
     if (strlen(term) < MAX_TERM_LEN) strcpy(t->term, term);
     else memcpy(t->term, term, MAX_TERM_LEN - 1), t->term[MAX_TERM_LEN - 1] = 0;
@@ -397,6 +409,7 @@ void sync_term_specs()
 {
     if (!term) return;
     term->spec = get_term_spec(term->term);
+    //dbg("sync_term_specs: %p\n", term->spec);
 } 
 
 struct terminal *init_term(int fdin, int fdout, 
@@ -404,34 +417,28 @@ struct terminal *init_term(int fdin, int fdout,
 {
     struct terminal *term;
     struct window *win;
-    if (!(term = mem_alloc(sizeof (struct terminal)))) {
-        terminate=1;
+    if (!(term = g_new0(struct terminal, 1))) {
+        zselect_terminate(zsel);
         return NULL;
     }
-    memset(term, 0, sizeof(struct terminal));
     term->fdin = fdin;
     term->fdout = fdout;
     term->master = term->fdout == get_output_handle();
-    /*term->x = 0;
-    term->y = 0;
-    term->cx = 0;
-    term->cy = 0;*/
     term->lcx = -1;
     term->lcy = -1;
     term->dirty = 1;
     term->redrawing = 0;
     term->blocked = -1;
-    term->screen = DUMMY;
-    term->last_screen = DUMMY;
+    term->screen = NULL;
+    term->last_screen = NULL;
     term->spec = &dumb_term;
     term->term[0] = 0;
-    term->cwd[0] = 0;
-    term->input_queue = DUMMY;
+    term->input_queue = NULL;
     term->qlen = 0;
     init_list(term->windows);
-    if (!(win = mem_alloc(sizeof (struct window)))) {
-        mem_free(term);
-        terminate=1;
+    if (!(win = g_new(struct window, 1))){
+        g_free(term);
+        zselect_terminate(zsel);
         return NULL;
     }
     win->handler = root_window;
@@ -439,7 +446,7 @@ struct terminal *init_term(int fdin, int fdout,
     win->term = term;
     add_to_list(term->windows, win);
     if (fdin!=-1){
-        set_handlers(fdin, in_term, NULL, destroy_terminal, CBA0);
+        zselect_set(zsel, fdin, in_term, NULL, destroy_terminal, NULL);
     }
     gseswin = win;
     return term;
@@ -449,9 +456,52 @@ void term_send_event(struct terminal *term, struct event *ev)
 {
 	struct window *win;
 
+#ifdef Z_HAVE_SDL
+	if (gses && gses->osk){
+		switch (ev->ev){
+			case EV_MOUSE:
+				//if (ev->b & (B_MOVE | B_DRAG)) goto to;
+				if (ev->b == (B_LEFT | B_DOWN)) {
+					if (zosk_click(gses->osk, ev->mx - gses->osk->parent_x, ev->my - gses->osk->parent_y, 1)) {
+						if (cfg->portrait) goto toterm;
+					}
+					redraw_later();
+					return;
+				}
+				if ((ev->b & ~B_CLICK) == (B_LEFT | B_UP))   {
+					if (zosk_click(gses->osk, ev->mx - gses->osk->parent_x, ev->my - gses->osk->parent_y, 0)) {
+						if (cfg->portrait) goto toterm;
+					}
+					redraw_later();
+					return;
+				}
+				if (cfg->portrait) goto toterm;
+				return;
+			case EV_KBD:
+				if ((ev->x == KBD_ESC || ev->x == KBD_ENTER) && (gses->osk->flags & ZOSK_ENTERCLOSE)){
+                    int flg = gses->osk->flags;
+					zosk_free(gses->osk);
+					gses->osk = NULL;
+					if (cfg->portrait){
+						resize_terminal(NULL);
+					}else{
+						redraw_later();
+						sdl_force_redraw();
+						SDL_FillRect(sdl->screen, NULL, z_makecol(0, 0, 0));
+						SDL_UpdateRect(sdl->screen, 0, 0, 0, 0);
+					}
+					if (flg & ZOSK_IGNOREESC) return;
+                    if (flg & ZOSK_SENDENTER) break;
+                    return;
+				}
+				break;
+		}
+	}
+toterm:;
+#endif
+
 /*	assert(ev && term && !list_empty(term->windows));*/
 /*	if_assert_failed return;*/
-
 	win = term->windows.next;
 	win->handler(win, ev, 0);
 }
@@ -471,30 +521,30 @@ static void term_send_ucs(struct terminal *term, struct event *ev, unicode_val u
 
 #define get_bool_opt(s, opt) (s->opt == 1 ? 1 : 0)
 
-void in_term(cba_t cba)
+void in_term(void *arg)
 {
     struct event *ev;
     int r, err;
     char *iq, *c;
-    if (!(iq = (char*)mem_realloc(term->input_queue, term->qlen + ALLOC_GR))) {
-        destroy_terminal(CBA0);
+
+    if (!(iq = (char*)g_realloc(term->input_queue, term->qlen + ALLOC_GR))) {
+        destroy_terminal(NULL);
+		zselect_hint(zsel, "a");
         return;
     }
     term->input_queue = iq;
-#if defined(_MSC_VER) || defined(__MINGW32__)
-	r = recv(term->fdin, iq + term->qlen, ALLOC_GR, 0); 
-    err = sock_errno;
-#else
-	r = read(term->fdin, iq + term->qlen, ALLOC_GR);
-    err = errno;
-#endif
+	r = z_pipe_read(term->fdin, iq + term->qlen, ALLOC_GR);
+    err = z_sock_errno;
+	
+	//zselect_hint(zsel, "fdin=%d qlen=%d   r=%d err=%d", term->fdin, term->qlen, r, err);
 
     if (r <= 0) {
-        if (r == -1/* && errno != ECONNRESET*/) internal_("ERROR: error %d on terminal: could not read event", err);
-        destroy_terminal(CBA0);
+        if (r == -1/* && errno != ECONNRESET*/) zinternal("ERROR: error %d on terminal: could not read event", err);
+        destroy_terminal(NULL);
+		zselect_hint(zsel, "b");
         return;
     }
-#ifdef HAVE_SDL    
+#ifdef Z_HAVE_SDL    
     if (sdl){
         MUTEX_LOCK(sdl->eventpipestate);
         sdl->eventpipestate-=r;
@@ -503,10 +553,13 @@ void in_term(cba_t cba)
     }
 #endif
 
-    
     term->qlen += r;
-    test_queue:
-    if (term->qlen < sizeof(struct event)) return;
+test_queue:
+	//dbg("next event\n");
+    if (term->qlen < sizeof(struct event)) {
+		//zselect_hint(zsel, "%d < %d", term->qlen, sizeof(struct event));
+		return;
+	}
     ev = (struct event *)iq;
     
 //    dbg("in_term [%d,%d,%d,%d]\n",ev->ev,ev->x,ev->y,ev->b);
@@ -525,7 +578,42 @@ void in_term(cba_t cba)
     
     
     ev = (struct event *)iq;
-    
+
+	if (zselect_profiling(zsel)){
+		switch (ev->ev){
+			case EV_INIT:
+				zselect_hint(zsel, "EV_INIT");
+				break;
+			case EV_RESIZE:
+				zselect_hint(zsel, "EV_RESIZE %dx%d", ev->x, ev->y);
+				break;
+			case EV_REDRAW:
+				zselect_hint(zsel, "EV_REDRAW");
+				break;
+			case EV_KBD:
+				if (ev->x < 0x100)
+					zselect_hint(zsel, "EV_KBD '%c' %s %s %s", ev->x, ev->y & KBD_CTRL ? "CTRL" : "", ev->y & KBD_ALT ? "ALT" : "", ev->y & KBD_SHIFT ? "SHIFT" : "");
+				else
+					zselect_hint(zsel, "EV_KBD 0x%x %s %s %s", ev->x, ev->y & KBD_CTRL ? "CTRL" : "", ev->y & KBD_ALT ? "ALT" : "", ev->y & KBD_SHIFT ? "SHIFT" : "");
+				break;
+			case EV_MOUSE:
+				zselect_hint(zsel, "EV_MOUSE 0x%x %x %y", ev->b, ev->mx, ev->my);
+				break;
+			case EV_ABORT:
+				zselect_hint(zsel, "EV_ABORT");
+				break;
+			case EV_SDLRESIZE:
+				zselect_hint(zsel, "EV_SDLRESIZE");
+				break;
+			case EV_SKIP:
+				zselect_hint(zsel, "EV_SKIP");
+				break;
+			default:
+				zselect_hint(zsel, "unknown %d", ev->ev);
+				break;
+		}
+
+	}
     r = sizeof(struct event);
     if (ev->ev != EV_INIT && ev->ev != EV_RESIZE && ev->ev != EV_REDRAW && ev->ev != EV_KBD && ev->ev != EV_MOUSE && ev->ev != EV_ABORT && ev->ev!=EV_SDLRESIZE && ev->ev!=EV_SKIP) {
         error("ERROR: error on terminal: bad event %d", ev->ev);
@@ -535,29 +623,26 @@ void in_term(cba_t cba)
         char *ts;
         int xwin;
         
-        xwin = is_xterm() * ENV_XWIN + can_twterm() * ENV_TWIN + (!!getenv("STY")) * ENV_SCREEN + get_system_env();
+        xwin = is_xterm() * ENV_XWIN + can_twterm() * ENV_TWIN + (!!getenv("STY")) * ENV_SCREEN;
         if (!(ts = getenv("TERM"))) ts = "";
 //        dbg("in_term ts=%s\n", ts);
         strcpy(term->term, ts);
-    
-        if (!(ts = get_cwd()) && !(ts = stracpy(""))) goto neni_pamet;
-        strcpy(term->cwd, ts);
-        mem_free(ts);
-        neni_pamet:;
         term->environment=xwin;
-        
         sync_term_specs();
     }
     if (ev->ev == EV_SDLRESIZE){
-//        dbg("EV_SDLRESIZE(%d,%d) (%d,%d)\n", ev->x, ev->y, ev->mx, ev->my);
-#ifdef HAVE_SDL        
+#ifdef Z_HAVE_SDL        
+        dbg("EV_SDLRESIZE(%d,%d) (%d,%d)\n", ev->x, ev->y, sdl->resize_w, sdl->resize_h/*ev->mx, ev->my*/);
+		MUTEX_LOCK(sdl->eventpipestate);
+		sdl->resizeevents--;
+		MUTEX_UNLOCK(sdl->eventpipestate);
         if (sdl) {
-			cfg->gfx_x=ev->mx;
-			cfg->gfx_y=ev->my;
-			sdl_setvideomode(ev->mx, ev->my, 0);
+			cfg->gfx_x=sdl->resize_w;//ev->mx;
+			cfg->gfx_y=sdl->resize_h;//ev->my;
+			sdl_setvideomode(sdl->resize_w, sdl->resize_h/*ev->mx, ev->my*/, 0);
 		}
 #endif
-        resize_terminal(CBA0);
+        resize_terminal(NULL);
     }
     if (ev->ev == EV_REDRAW || ev->ev == EV_RESIZE || ev->ev == EV_INIT) {
         struct window *win;
@@ -573,10 +658,16 @@ void in_term(cba_t cba)
         foreachback(win, term->windows) win->handler(win, ev, 0);
         term->redrawing = 0;
     }
-	if (ev->ev == EV_MOUSE && preferred_func(ev))
+	if (ev->ev == EV_MOUSE && preferred_func(ev)){
 		term_send_event(term, ev);
+	}
 	if (ev->ev == EV_KBD) {
-		if (upcase(ev->x) == (int)'L' && ev->y == KBD_CTRL) {
+		if (
+#ifdef Z_HAVE_SDL
+            !sdl &&     
+#endif
+            z_char_uc(ev->x) == (int)'L' && ev->y == KBD_CTRL) {
+
 			ev->ev = EV_REDRAW;
 			ev->x = term->x;
 			ev->y = term->y;
@@ -625,11 +716,12 @@ void in_term(cba_t cba)
 			term_send_ucs(term, ev, UCS_NO_CHAR);
 		}
     }
-    if (ev->ev == EV_ABORT) destroy_terminal(CBA0);
+    if (ev->ev == EV_ABORT) zselect_terminate(zsel);//destroy_terminal(NULL);
     /*redraw_screen(term);*/
     mm:
     if (term->qlen == r) term->qlen = 0;
     else memmove(iq, iq + r, term->qlen -= r);
+	//return;
     goto test_queue;
 }
 
@@ -672,6 +764,8 @@ char frame_restrict[48] = {
 /* characters 128-159 are special for xterm unless
  * *VT100*allowC1Printable property is not set */
 
+#define MODE(ch) (((ch) >> 15) & MODE_MASK)
+
 #define PRINT_CHAR(p)                                                       \
 {                                                                           \
 	unsigned ch = term->screen[p];                                          \
@@ -680,24 +774,24 @@ char frame_restrict[48] = {
 	int charset = s->charset;                                               \
 	if (s->mode == TERM_LINUX) {                                            \
 		if (s->m11_hack) {                                                  \
-			if (ch >> 15 != mode) {         	                            \
-				if (!(mode = ch >> 15))                                     \
-					add_to_str(&a, &l, "\033[10m");                         \
+			if (MODE(ch) != mode) {         	                            \
+				if (!(mode = MODE(ch)))                                     \
+					g_string_append(a, "\033[10m");                         \
                 else                                                        \
-					add_to_str(&a, &l, "\033[11m");	                        \
+					g_string_append(a, "\033[11m");	                        \
 			}                                                               \
 		}                                                                   \
-		if ((mode = ch >> 15) && c >= 176 && c < 224) {                     \
+		if ((mode = MODE(ch)) && c >= 176 && c < 224) {                     \
 			charset = get_cp_index("cp437");                                \
 			if (s->restrict_852 && frame_restrict[c - 176])                 \
 				c = frame_restrict[c - 176];                                \
 		}                                                                   \
 	} else if (s->mode == TERM_VT100) {                                     \
-		if (ch >> 15 != mode) {                                             \
-			if (!(mode = ch >> 15))                                         \
-				add_to_str(&a, &l, "\x0f");                                 \
+		if (MODE(ch) != mode) {                                             \
+			if (!(mode = MODE(ch)))                                         \
+				g_string_append(a, "\x0f");                                 \
 			else                                                            \
-				add_to_str(&a, &l, "\x0e");                                 \
+				g_string_append(a, "\x0e");                                 \
 		}                                                                   \
 		if (mode && c >= 176 && c < 224) {                                  \
 			charset = get_cp_index("cp437");                                \
@@ -708,124 +802,123 @@ char frame_restrict[48] = {
 				c = frame_vt100[c - 176];                                   \
 		}                                                                   \
 	} else if (s->mode == TERM_KOI8) {                                      \
-		if (ch >> 15 != mode && (mode = ch >> 15))                          \
+		if (MODE(ch) != mode && (mode = MODE(ch)))                          \
 			charset = get_cp_index("koi8-r");                               \
 		if (mode && c >= 176 && c < 224)                                    \
 			c = frame_koi[c - 176];                                         \
-	} else if (s->mode == TERM_DUMB && (ch >> 15) && c >= 176 && c < 224)   \
+	} else if (s->mode == TERM_DUMB && MODE(ch) && c >= 176 && c < 224)   \
 		c = frame_dumb[c - 176];                                            \
 	if (!(A & 0100) && (A >> 3) == (A & 7))                                 \
 		A = (A & 070) | 7 * !(A & 020);                                     \
 	if (A != attrib) {                                                      \
 		attrib = A;                                                         \
-		add_to_str(&a, &l, "\033[0");                                       \
+		g_string_append(a, "\033[0");                                       \
 		if (s->col) {                                                       \
 			char m[4];                                                      \
 			m[0] = ';'; m[1] = '3'; m[3] = 0;                               \
 			m[2] = (attrib & 7) + '0';                                      \
-			add_to_str(&a, &l, m);                                          \
+			g_string_append(a, m);                                          \
 			m[1] = '4';                                                     \
 			m[2] = (attrib >> 3 & 7) + '0';                                 \
-			add_to_str(&a, &l, m);                                          \
+			g_string_append(a, m);                                          \
 		} else if (getcompcode(attrib & 7) < getcompcode(attrib >> 3 & 7))  \
-			add_to_str(&a, &l, ";7");                                       \
+			g_string_append(a, ";7");                                       \
 		if (attrib & 0100)                                                  \
-			add_to_str(&a, &l, ";1");                                       \
-		add_to_str(&a, &l, "m");                                            \
+			g_string_append(a, ";1");                                       \
+		g_string_append(a, "m");                                            \
 	}                                                                       \
 	if (c>=' ' && (c<127 || c>159) && c!=255) {                             \
 		if (get_bool_opt(s, utf_8_io))                                      \
-			add_to_str(&a, &l, cp2utf_8(charset, c));                       \
+			g_string_append(a, cp2utf_8(charset, c));                       \
 		else                                                                \
-			add_chr_to_str(&a, &l, c);                                      \
+			g_string_append_c(a, c);                                        \
 	}                                                                       \
-	else if (!c || c == 1) add_chr_to_str(&a, &l, ' ');                     \
-	else add_chr_to_str(&a, &l, '.');                                       \
+	else if (!c || c == 1) g_string_append_c(a, ' ');                       \
+	else g_string_append_c(a, '.');                                         \
 	cx++;                                                                   \
 }                                                                           \
 
-/*void redraw_all_terminals()
-{
-    struct terminal *term;
-    foreach(term, terminals) redraw_screen(term);
-}*/
 
 void term_redraw_screen()
 {
     int x, y, p = 0;
     int cx = -1, cy = -1;
-    /*unsigned*/ char *a;
+    GString *a;
     int attrib = -1;
     int mode = -1;
-    int l = 0;
     struct term_spec *s;
     if (!term->dirty || (term->master && is_blocked())) return;
-    if (!(a = init_str())) return;
+    a = g_string_new("");
     s = term->spec;
     for (y = 0; y < term->y; y++)
         for (x = 0; x < term->x; x++, p++) {
             if (y == term->y - 1 && x == term->x - 1) break;
             if (term->screen[p] == term->last_screen[p]) continue;
-            if ((term->screen[p] & 0x3800) == (term->last_screen[p] & 0x3800) && ((term->screen[p] & 0xff) == 0 || (term->screen[p] & 0xff) == 1 || (term->screen[p] & 0xff) == ' ') && ((term->last_screen[p] & 0xff) == 0 || (term->last_screen[p] & 0xff) == 1 || (term->last_screen[p] & 0xff) == ' ')) continue;
+            if ((term->screen[p] & 0x3800) == (term->last_screen[p] & 0x3800) && 
+                ((term->screen[p] & 0xff) == 0 || 
+                 (term->screen[p] & 0xff) == 1 || 
+                 (term->screen[p] & 0xff) == ' ') &&
+                ((term->last_screen[p] & 0xff) == 0 || 
+                 (term->last_screen[p] & 0xff) == 1 || 
+                 (term->last_screen[p] & 0xff) == ' ')) continue;
             if (cx == x && cy == y) goto pc;/*PRINT_CHAR(p)*/
             else if (cy == y && x - cx < 10) {
                 int i;
                 for (i = x - cx; i >= 0; i--) PRINT_CHAR(p - i);
             } else {
-                add_to_str(&a, &l, "\033[");
-                add_num_to_str(&a, &l, y + 1);
-                add_to_str(&a, &l, ";");
-                add_num_to_str(&a, &l, x + 1);
-                add_to_str(&a, &l, "H");
+                g_string_sprintfa(a, "\033[%d;%dH", y + 1, x + 1);
                 cx = x; cy = y;
                 pc:
                 PRINT_CHAR(p);
             }
         }
-    if (l) {
-        if (s->col) add_to_str(&a, &l, "\033[37;40m");
-        add_to_str(&a, &l, "\033[0m");
-        if (s->mode == TERM_LINUX && s->m11_hack) add_to_str(&a, &l, "\033[10m");
-        if (s->mode == TERM_VT100) add_to_str(&a, &l, "\x0f");
+    if (a->len) {
+        if (s->col) g_string_append(a, "\033[37;40m");
+        g_string_append(a, "\033[0m");
+        if (s->mode == TERM_LINUX && s->m11_hack) g_string_append(a, "\033[10m");
+        if (s->mode == TERM_VT100) g_string_append(a, "\x0f");
     }
-    if (l || term->cx != term->lcx || term->cy != term->lcy) {
+    if (a->len || term->cx != term->lcx || term->cy != term->lcy) {
         term->lcx = term->cx;
         term->lcy = term->cy;
-        add_to_str(&a, &l, "\033[");
-        add_num_to_str(&a, &l, term->cy + 1);
-        add_to_str(&a, &l, ";");
-        add_num_to_str(&a, &l, term->cx + 1);
-        add_to_str(&a, &l, "H");
+        g_string_sprintfa(a, "\033[%d;%dH", term->cy + 1, term->cx + 1);
     }
-    if (l && term->master) want_draw();
-    hard_write(term->fdout, a, l);
-    if (l && term->master) done_draw();
+    hard_write(term->fdout, a->str, a->len);
 	
-    mem_free(a);
+    g_string_free(a, TRUE);
     memcpy(term->last_screen, term->screen, term->x * term->y * sizeof(int));
     term->dirty = 0;
 }
 
-void destroy_terminal(cba_t cba)
+void destroy_terminal(void *arg)
 {
     if (!term) return;
-    terminate=1;
-    while ((term->windows.next) != &term->windows) delete_window(term->windows.next);
-    /*if (term->cwd) mem_free(term->cwd);*/
-    if (term->title) mem_free(term->title);
-    mem_free(term->screen);
+	//zselect_terminate(zsel);
+	while ((term->windows.next) != &term->windows) delete_window(term->windows.next);
+    /*if (term->cwd) g_free(term->cwd);*/
+    if (term->title) g_free(term->title);
+    g_free(term->screen);
     term->screen=NULL;
-    mem_free(term->last_screen);
+    g_free(term->last_screen);
     term->screen=NULL;
-    set_handlers(term->fdin, NULL, NULL, NULL, CBA0);
-    mem_free(term->input_queue);
+    zselect_set(zsel, term->fdin, NULL, NULL, NULL, NULL);
+    g_free(term->input_queue);
     if (term->blocked != -1) {
+#ifdef Z_MSC_MINGW
+		zinternal("term->blocked unimplemented (and should not be called)");
+#else
         close(term->blocked);
-        set_handlers(term->blocked, NULL, NULL, NULL, CBA0);
+        zselect_set(zsel, term->blocked, NULL, NULL, NULL, NULL);
+#endif
     }
-    close(term->fdin);
-    if (term->fdout != 1) {
+    z_pipe_close(term->fdin);
+    if (term->fdout != 1 && term->fdout >= 0) {
+#ifdef Z_MSC_MINGW
+		zinternal("term->fdout unimplemented (and should not be called)");
+#else
         if (term->fdout != term->fdin) close(term->fdout);
+#endif
+
     } else {
         unhandle_terminal_signals();
         free_all_itrms();
@@ -835,7 +928,8 @@ void destroy_terminal(cba_t cba)
         }
 #endif */
     }
-    mem_free(term);
+
+    g_free(term);
     term=NULL;
 }
 
@@ -940,7 +1034,7 @@ void print_text(int x, int y, int l, char *text, unsigned c)
 
 void set_cursor(int x, int y, int altx, int alty)
 {
-    //dbg("set_cursor(%d,%d, %d,%d)\n", x, y, altx, alty);
+//    dbg("set_cursor(%d,%d,  %d,%d)\n", x, y, altx, alty);
     term->dirty = 1;
     if (term->spec->block_cursor) x = altx, y = alty;
     if (x >= term->x) x = term->x - 1;
@@ -953,31 +1047,41 @@ void set_cursor(int x, int y, int altx, int alty)
 
 void exec_thread(char *path, int p)
 {
+#ifdef Z_MSC_MINGW
+	zinternal("exec_thread unimplemented (and should not be called)");
+#else
+
 #if defined(HAVE_SETPGID) && !defined(BEOS) && !defined(HAVE_BEGINTHREAD)
     if (path[0] == 2) setpgid(0, 0);
 #endif
     exe(path + 1);
     close(p);
     if (path[1 + strlen(path + 1) + 1]) unlink(path + 1 + strlen(path + 1) + 1);
+#endif
 }
 
-void close_handle(cba_t cba)
+void close_handle(void *arg)
 {
-    int h = (int)GETCBA(cba, int_);
+#ifdef Z_MSC_MINGW
+	zinternal("close_handle unimplemented");
+#else
+    int h = GPOINTER_TO_INT(arg);
     close(h);
-    set_handlers(h, NULL, NULL, NULL, CBA0);
+    zselect_set(zsel, h, NULL, NULL, NULL, NULL);
+#endif
 }
 
-void unblock_terminal(cba_t cba)
+#ifndef Z_MSC_MINGW
+void unblock_terminal(void *arg)
 {
-    close_handle((cba_t)term->blocked);
+    close_handle(GINT_TO_POINTER(term->blocked));
     term->blocked = -1;
-    set_handlers(term->fdin, in_term, NULL, destroy_terminal, CBA0);
+    zselect_set(zsel, term->fdin, in_term, NULL, destroy_terminal, NULL);
     unblock_itrm(term->fdin);
     redraw_terminal_cls();
 }
 
-void exec_on_terminal(char *path, char *delete, int fg)
+void exec_on_terminal(char *path, char *delete1, int fg)
 {
     if (path && !*path) return;
     if (!path) path="";
@@ -985,43 +1089,43 @@ void exec_on_terminal(char *path, char *delete, int fg)
     fg = 0;
 #endif
     if (term->master) {
-        if (!*path) dispatch_special(delete);
+        if (!*path) dispatch_special(delete1);
         else {
             int blockh;
             char *param;
             if (is_blocked() && fg) {
-                unlink(delete);
+                unlink(delete1);
                 return;
             }
-            if (!(param = mem_alloc(strlen(path) + strlen(delete) + 3))) return;
+            if (!(param = g_malloc(strlen(path) + strlen(delete1) + 3))) return;
             param[0] = fg;
             strcpy(param + 1, path);
-            strcpy(param + 1 + strlen(path) + 1, delete);
+            strcpy(param + 1 + strlen(path) + 1, delete1);
             if (fg == 1) block_itrm(term->fdin);
-            if ((blockh = start_thread((void (*)(void *, int))exec_thread, param, strlen(path) + strlen(delete) + 3)) == -1) {
+            if ((blockh = start_thread((void (*)(void *, int))exec_thread, param, strlen(path) + strlen(delete1) + 3)) == -1) {
                 if (fg == 1) unblock_itrm(term->fdin);
-                mem_free(param);
+                g_free(param);
                 return;
             }
-            mem_free(param);
+            g_free(param);
             if (fg == 1) {
                 term->blocked = blockh;
-                set_handlers(blockh, unblock_terminal, NULL, unblock_terminal, CBA0);
-                set_handlers(term->fdin, NULL, NULL, destroy_terminal, CBA0);
+                zselect_set(zsel, blockh, unblock_terminal, NULL, unblock_terminal, NULL);
+                zselect_set(zsel, term->fdin, NULL, NULL, destroy_terminal, NULL);
                 /*block_itrm(term->fdin);*/
             } else {
-                set_handlers(blockh, close_handle, NULL, close_handle, (cba_t)blockh);
+                zselect_set(zsel, blockh, close_handle, NULL, close_handle, GINT_TO_POINTER(blockh));
             }
         }
     } else {
         char *data;
-        if ((data = mem_alloc(strlen(path) + strlen(delete) + 4))) {
+        if ((data = g_malloc(strlen(path) + strlen(delete1) + 4))) {
             data[0] = 0;
             data[1] = fg;
             strcpy(data + 2, path);
-            strcpy(data + 3 + strlen(path), delete);
-            hard_write(term->fdout, data, strlen(path) + strlen(delete) + 4);
-            mem_free(data);
+            strcpy(data + 3 + strlen(path), delete1);
+            hard_write(term->fdout, data, strlen(path) + strlen(delete1) + 4);
+            g_free(data);
         }
         /*char x = 0;
         hard_write(term->fdout, &x, 1);
@@ -1035,22 +1139,27 @@ void exec_on_terminal(char *path, char *delete, int fg)
 void do_terminal_function(char code, char *data)
 {
     char *x_data;
-    if (!(x_data = (char *)mem_alloc(strlen(data) + 2))) return;
+    if (!(x_data = (char *)g_malloc(strlen(data) + 2))) return;
     x_data[0] = code;
     strcpy(x_data + 1, data);
     exec_on_terminal(NULL, x_data, 0);
-    mem_free(x_data);
+    g_free(x_data);
 }
+#endif
 
 void set_terminal_title(char *title)
 {
     if (term->title && !strcmp(title, term->title)) return;
-    if (term->title) mem_free(term->title);
-    term->title = stracpy(title);
-#ifdef HAVE_SDL    
+    if (term->title) g_free(term->title);
+    term->title = g_strdup(title);
+#ifdef Z_HAVE_SDL    
     if (!sdl) 
-#endif
+#endif 
+	{
+#ifndef Z_MSC_MINGW
         do_terminal_function(TERM_FN_TITLE, title);
+#endif
+	}
 }
     
 
@@ -1058,11 +1167,11 @@ void set_ctest_title(){
     gchar *c;
 
     if (ctest){
-        c = g_strdup_printf(" %s %s - %s %s", ctest->pcall, ctest->tname, PACKAGE_NAME, PACKAGE_VERSION);
+        c = g_strdup_printf(" %s %s - %s-%s %s", ctest->pcall, ctest->tname, PACKAGE_NAME, Z_PLATFORM, PACKAGE_VERSION);
         set_terminal_title(c);
         g_free(c);
     }else{
-        c = g_strdup_printf(" %s %s", PACKAGE_NAME, PACKAGE_VERSION);
+        c = g_strdup_printf(" %s-%s %s", PACKAGE_NAME, Z_PLATFORM, PACKAGE_VERSION);
         set_terminal_title(c);
         g_free(c);
     }
